      SUBROUTINE ADPTIN
 
C     *************************************************************************
C
C     ADPTIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     15-AUG-2017:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     ADAPTS THE SPATIAL MESH
C     ACCORDING TO A SPECIFIED INITIAL DISTRIBUTION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      INTEGER NLOOPS
      PARAMETER(NLOOPS = 10)


C     LOCAL DATA
C     ==========
      INTEGER ILOOPS
      INTEGER N2RTOT,NGRTOT
C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 0

C     =========================================================================

C     LOOP UNTIL INITIAL ADAPTION IS COMPLETE
      ILOOPS = 1
1000  CONTINUE

C       =======================================================================

C       EVALUATE DATA FOR REFINEMENT/DEREFINEMENT
        CALL REFDIN

C       CHECK LIMITS
        CALL ADMCHK

C       DEREFINE THE MESH
        IF(N2DREF.GT.0)CALL DEREFN

C       REFINE THE MESH
        IF(N2REFN.GT.0)CALL REFINE

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("ADPTIN",3I5)')ILOOPS,N2DREF,N2REFN
          IF((N2DREF.GT.0).OR.(N2REFN.GT.0))THEN
            WRITE(6,'("ADPTIN",3I8)')ITIME,N2DREF,N2REFN
          ENDIF
        ENDIF

C       HOUSEKEEPING
        N2RTOT = N2DREF + N2REFN
        CALL P_GMXI(N2RTOT,NGRTOT)

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)WRITE(6,'("ADPTIN",I5)')NGRTOT

        IF(NGRTOT.GT.0)THEN

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'("ADPTIN: call TREBAL",I5)')NCELL

C         BALANCE THE OCTREE
          CALL TREBAL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)WRITE(6,'("ADPTIN: post TREBAL",I5)')NCELL

C         LOAD-BALANCE ACROSS PROCESSORS
          CALL PARBAL

C         LOOP BACK AND RE-EVALUATE
          IF(ILOOPS.LT.NLOOPS)THEN

            ILOOPS = ILOOPS + 1
            GOTO 1000

          ELSE

C           TOO MANY LOOPS - ERROR CONDITION
            WRITE(6,*)'ADPTIN:  warning: too many loops'

          ENDIF

        ENDIF

C       =======================================================================

C     END OF LOOP 1000 UNTIL INITIAL ADAPTION IS COMPLETE

C     =========================================================================


      RETURN
      END
