      SUBROUTINE CFLBCF(IFACE,IFCODE)

C     *************************************************************************
C
C     CFLBCF
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     06-MAY-2009:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES CELL FACE FLUXES FOR ALL PDEs
C     INCLUDES MULTIPLE SCALARS AND MULTI-STEP CHEMISTRY
C     CELL FACE WITH BOUNDARY CONDITIONS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENT
C     ========
      INTEGER IFCODE(NFACMX)
      INTEGER IFACE


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YEVALF(NSPCMX),YDIFFF(NSPCMX)
      DOUBLE PRECISION YDVALF(NSPCMX)
      DOUBLE PRECISION YMVALT(NSPCMX)
      DOUBLE PRECISION YFLUXF(NSPCMX)
      DOUBLE PRECISION DIFFMF(NSPCMX)
      DOUBLE PRECISION HEVALF(NSPCMX)
      DOUBLE PRECISION VISTAU(NCMPMX,NCMPMX)
      DOUBLE PRECISION UEVALF(NCMPMX),UDVALF(NCMPMX,NCMPMX)
      DOUBLE PRECISION GPFACF(NCMPMX)
      DOUBLE PRECISION UFLUXF(NCMPMX)
      DOUBLE PRECISION UGRADN(NCMPMX),UGRADU(NCMPMX)
      DOUBLE PRECISION RSOVSZ,RSOVS2,RSHALF
      DOUBLE PRECISION DEVALF,TEVALF,PEVALF,EEVALF,AEVALF
      DOUBLE PRECISION CPVLFF,RGVLFF,GAMMAF
      DOUBLE PRECISION DMVALT,TMVALT,PMVALT
      DOUBLE PRECISION DFLUXF,EFLUXF
      DOUBLE PRECISION TDVALF
      DOUBLE PRECISION TCONDF,VISTRF
      DOUBLE PRECISION DIVUCF
      DOUBLE PRECISION VISCOF,VISTNF,VIST1F,VIST2F
      DOUBLE PRECISION PCONST,PINFTY
      DOUBLE PRECISION CHARL2,CHARLI,CHARLX,DAVALF,ACSQRD
      DOUBLE PRECISION YDCORR,YDCORY
      DOUBLE PRECISION CSIZE,OCSIZE,FFACTR
      DOUBLE PRECISION UINFTY
      DOUBLE PRECISION DGRADN,PGRADN,DGRADU,PGRADU
      DOUBLE PRECISION UPLUSA,UMNUSA
      DOUBLE PRECISION CHARL1,CHARL5
      DOUBLE PRECISION CHRL1P,CHRL2P,CHRL3P,CHRL4P,CHRL5P
      INTEGER INEXT,ILEVL,IDIRN,ISPEC
      INTEGER IDRP1,IDRP2
      INTEGER JGPFAC

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 99

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CFLBCF'

C     =========================================================================

C     NEXT-DOOR CELL INDEX
      INEXT = NXDOOR(IFACE,ICELL)

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,'(4I6)')ICELL,IFACE,INEXT,IFCODE(IFACE)

C     LEVEL, SIZE AND RECIPROCAL SIZE OF LOCAL CELL
      ILEVL = NLEVEL(ICELL)
      CSIZE = HCSIZE(ILEVL)
      OCSIZE = OVSIZE(ILEVL)
      RSOVSZ = RSIGN(IFACE)*OCSIZE
      RSOVS2 = RSIGN(IFACE)*TWO*OCSIZE
      RSHALF = RSIGN(IFACE)*HALF*CSIZE

C     DIRECTION INDEX
      IDIRN = MDIRN(IFACE)

C     =========================================================================

C     COMPUTE NEW CELL FACE VALUES
C     -----------------------------
C     NOTE THAT L/R DISTINCTION IS IRRELEVANT FOR BCS

C     RUN THROUGH THE GAUSS POINTS
      DO JGPFAC = 1, NGPFAC

C       -----------------------------------------------------------------------

C       NEW CELL FACE VALUES
C       --------------------

C       CELL FACE VALUES
        DEVALF = DGVALF(JGPFAC,IFACE)
        UEVALF(1) = UGVALF(1,JGPFAC,IFACE)
        UEVALF(2) = UGVALF(2,JGPFAC,IFACE)
        UEVALF(3) = UGVALF(3,JGPFAC,IFACE)
        TEVALF = TGVALF(JGPFAC,IFACE)
        PEVALF = PGVALF(JGPFAC,IFACE)
        DO ISPEC = 1, NSPEC
          YEVALF(ISPEC) = YGVALF(ISPEC,JGPFAC,IFACE)
        ENDDO
        CPVLFF = CPVALF(JGPFAC,IFACE)
        RGVLFF = RGVALF(JGPFAC,IFACE)

C       DIAGNOSTICS
        GPFACF(1) = GPLOCF(1,JGPFAC,IFACE)
        GPFACF(2) = GPLOCF(2,JGPFAC,IFACE)
        GPFACF(3) = GPLOCF(3,JGPFAC,IFACE)

C       CELL FACE VALUES FOR MOLECULAR TRANSPORT
        DMVALT = DMVALF(JGPFAC,IFACE)
        TMVALT = TMVALF(JGPFAC,IFACE)
        PMVALT = PMVALF(JGPFAC,IFACE) + PREFER
        DO ISPEC = 1, NSPEC
          YMVALT(ISPEC) = YMVALF(ISPEC,JGPFAC,IFACE)
        ENDDO

C       VISCOUS STRESS TENSOR
        VISTNF = UGRADF(1,JGPFAC,IFACE)
        VIST1F = UGRADF(2,JGPFAC,IFACE)
        VIST2F = UGRADF(3,JGPFAC,IFACE)

C       CELL FACE GRADIENTS
        TDVALF = TGRADF(JGPFAC,IFACE)
        DO ISPEC = 1, NSPEC
          YDVALF(ISPEC) = YGRADF(ISPEC,JGPFAC,IFACE)
        ENDDO

C       -----------------------------------------------------------------------

C       SPEED OF SOUND
        GAMMAF = CPVLFF*RGVLFF/(CPVLFF - RGVLFF)
        AEVALF = SQRT(GAMMAF*TEVALF)

C       -----------------------------------------------------------------------

C       MOLECULAR TRANSPORT COEFFICIENTS
        CALL TRANCO(TMVALT,CPVLFF,VISCOF,TCONDF,DIFFMF)

C       =======================================================================

C       APPLY BOUNDARY CONDITION
C       ========================
        IF(IFCODE(IFACE).EQ.NSBCO1)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC NON-REFLECTING OUTFLOW WITH PRESSURE TRACKING
C         ----------------------------------------------------------

C         EULER BCS

C         IMPOSE PRESSURE AT INFINITY
          CALL BCPINF(INEXT,RKTIME,PINFTY,PCONST)

C         IMPOSE NON-REFLECTING CONDITION ON INCOMING CHARACTERISTIC
          DAVALF = DEVALF*AEVALF
          ACSQRD = AEVALF*AEVALF
          CHARLI = PINFTY - PREFER

C         EVALUATE EXTRAPOLATED CHARACTERISTICS
          CHARL2 = PEVALF - ACSQRD*DEVALF
          CHARLX = PEVALF - RSIGN(IFACE)*DAVALF*UEVALF(IDIRN)

C         UPDATE THE PRIMITIVE VARIABLES
          PEVALF = HALF*(CHARLX + CHARLI)
          DEVALF = (PEVALF - CHARL2)/ACSQRD
          UEVALF(IDIRN) = HALF*(CHARLI - CHARLX)*RSIGN(IFACE)/DAVALF

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         NAVIER-STOKES BCS

C         NORMAL COMPONENT OF VISCOUS STRESS TENSOR
C          VISTNF = ZERO

C         TRANSVERSE COMPONENTS OF VISCOUS STRESS TENSOR
          VIST1F = ZERO
          VIST2F = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCO2)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC REFLECTING OUTFLOW WITH SPECIFIED PRESSURE
C         -------------------------------------------------------

C         EULER BCS

C         IMPOSE PRESSURE AT THE BOUNDARY
          CALL BCPINF(INEXT,RKTIME,PINFTY,PCONST)
          PEVALF = PINFTY - PREFER

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         NAVIER-STOKES BCS

C         NORMAL COMPONENT OF VISCOUS STRESS TENSOR
C          VISTNF = ZERO

C         TRANSVERSE COMPONENTS OF VISCOUS STRESS TENSOR
          VIST1F = ZERO
          VIST2F = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCO3)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC NON-REFLECTING OUTFLOW WITH VELOCITY TRACKING
C         ----------------------------------------------------------

C         EULER BCS

C         IMPOSE PRESSURE AT INFINITY
          CALL BCPINF(INEXT,RKTIME,PINFTY,PCONST)

C         REFERENCE VALUES
          PINFTY = PINFTY - PREFER
C          UINFTY = 2.99D0
C          UINFTY = URIN
C          UINFTY = ZERO
          UINFTY = UEVALF(IDIRN)

C         IMPOSE NON-REFLECTING CONDITION ON INCOMING CHARACTERISTIC
          DAVALF = DEVALF*AEVALF
          ACSQRD = AEVALF*AEVALF
C          CHARLI = PINFTY - PREFER
C          CHARLI = PINFTY - PREFER + RSIGN(IFACE)*DAVALF*2.99D0
C          CHARLI = PINFTY - PREFER + RSIGN(IFACE)*DAVALF*UINFTY
C          CHARLI = (PEVALF - PINFTY)
C     +           + RSIGN(IFACE)*DAVALF*(UEVALF(IDIRN) - UINFTY)
          CHARLI = PINFTY
     +           + RSIGN(IFACE)*DAVALF*(UEVALF(IDIRN) - UINFTY)
C          CHARLI = HALF*CHARLI
C          CHARLI = ZERO

C         EVALUATE EXTRAPOLATED CHARACTERISTICS
          CHARL2 = (PEVALF -PINFTY) - ACSQRD*DEVALF
          CHARLX = (PEVALF -PINFTY)
     +           - RSIGN(IFACE)*DAVALF*(UEVALF(IDIRN) - UINFTY)

C         UPDATE THE PRIMITIVE VARIABLES
          PEVALF = PINFTY + HALF*(CHARLX + CHARLI)
          DEVALF = ((PEVALF - PINFTY) - CHARL2)/ACSQRD
          UEVALF(IDIRN) = UINFTY
     +                  + HALF*(CHARLI - CHARLX)*RSIGN(IFACE)/DAVALF

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         NAVIER-STOKES BCS

C         NORMAL COMPONENT OF VISCOUS STRESS TENSOR
C          VISTNF = ZERO

C         TRANSVERSE COMPONENTS OF VISCOUS STRESS TENSOR
          VIST1F = ZERO
          VIST2F = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCO4)THEN

C         ---------------------------------------------------------------------

C         BC: NSCBC GRADIENTS

C         ----------------------------------------------------------

C         EULER BCS

C         IMPOSE PRESSURE AT INFINITY
          CALL BCPINF(INEXT,RKTIME,PINFTY,PCONST)
          PINFTY = PINFTY - PREFER

C         NON-NORMAL DIRECTIONS
          IDRP1 = MNVIST(1,IDIRN)
          IDRP2 = MNVIST(2,IDIRN)

C         BOUNDARY-NORMAL GRADIENTS
C          DGRADN = (DEVALC-DEVALF)*RSOVS2
C          PGRADN = (PRUN(ICELL)-PEVALF)*RSOVS2
C          UGRADN(1) = (UEVALC(1)-UEVALF(1))*RSOVS2
C          UGRADN(2) = (UEVALC(2)-UEVALF(2))*RSOVS2
C          UGRADN(3) = (UEVALC(3)-UEVALF(3))*RSOVS2
          DGRADN = DGRADB(JGPFAC,IFACE)
          PGRADN = PGRADB(JGPFAC,IFACE)
          UGRADN(IDIRN) = UGRADB(IDIRN,JGPFAC,IFACE)
          UGRADN(IDRP1) = UGRADB(IDRP1,JGPFAC,IFACE)
          UGRADN(IDRP2) = UGRADB(IDRP2,JGPFAC,IFACE)

C         SONIC QUANTITIES
          DAVALF = DEVALF*AEVALF
          ACSQRD = AEVALF*AEVALF
          UPLUSA = UEVALF(IDIRN) + AEVALF
          UMNUSA = UEVALF(IDIRN) - AEVALF

C         CHARACTERISTICS
          CHARL1 = HALF*UMNUSA*(PGRADN - DAVALF*UGRADN(IDIRN))
          CHRL2P = DGRADN - PGRADN/ACSQRD
          CHRL3P = UGRADN(IDRP1)
          CHRL4P = UGRADN(IDRP2)
          CHARL5 = HALF*UPLUSA*(PGRADN + DAVALF*UGRADN(IDIRN))

C         IMPOSE NON-REFLECTING CONDITION ON INCOMING CHARACTERISTIC
          IF(MSIGN(IFACE).LT.0)THEN
            CHARL1 = PCONST*(PEVALF - PINFTY)
          ELSE 
            CHARL5 = PCONST*(PEVALF - PINFTY)
          ENDIF
          CHRL1P = CHARL1/UMNUSA
          CHRL5P = CHARL5/UPLUSA

C         DIAGNOSTICS
C          WRITE(6,'(3I5)')IFACE,MSIGN(IFACE),IDIRN
C          WRITE(6,'(2(1PE12.4))')PINFTY,PCONST
C          WRITE(6,'(6(1PE12.4))')PEVALF,
C     +         UEVALF(IDIRN),UEVALF(IDRP1),UEVALF(IDRP2),DEVALF,AEVALF
C          WRITE(6,'(5(1PE12.4))')PGRADN,
C     +         UGRADN(IDIRN),UGRADN(IDRP1),UGRADN(IDRP2),DGRADN
C          WRITE(6,'(3(1PE12.4))')VISTNF,VIST1F,VIST2F
C          WRITE(6,'(4(1PE12.4))')UMNUSA,CHARL1,UPLUSA,CHARL5
C          WRITE(6,'(4(1PE12.4))')CHRL1P,CHRL2P,CHRL5P

C         EVALUATE UPDATED GRADIENTS
          PGRADU = CHRL5P + CHRL1P
          UGRADU(IDIRN) = (CHRL5P - CHRL1P)/DAVALF
C          UGRADU(IDIRN) = CHRL5P/DAVALF
          UGRADU(IDRP1) = CHRL3P
          UGRADU(IDRP2) = CHRL4P
          DGRADU = CHRL2P + PGRADU/ACSQRD

C         UPDATE THE PRIMITIVE VARIABLES
C          PEVALF = PRUN(ICELL) - PGRADN*RSHALF
C          DEVALF = DEVALC - DGRADN*RSHALF
C          UEVALF(IDIRN) = UEVALC(IDIRN) - UGRADN(IDIRN)*RSHALF
C          UEVALF(IDRP1) = UEVALC(IDRP1) - UGRADN(IDRP1)*RSHALF
C          UEVALF(IDRP2) = UEVALC(IDRP2) - UGRADN(IDRP2)*RSHALF
C          PEVALF = PRUN(ICELL) - PGRADU*RSHALF
C          DEVALF = DEVALC - DGRADU*RSHALF
C          UEVALF(IDIRN) = UEVALC(IDIRN) - UGRADU(IDIRN)*RSHALF
C          UEVALF(IDRP1) = UEVALC(IDRP1) - UGRADU(IDRP1)*RSHALF
C          UEVALF(IDRP2) = UEVALC(IDRP2) - UGRADU(IDRP2)*RSHALF
          PEVALF = PEVALF - (PGRADU - PGRADN)*RSHALF
          DEVALF = DEVALF - (DGRADU - DGRADN)*RSHALF
          UEVALF(IDIRN) = UEVALF(IDIRN)
     +                  - (UGRADU(IDIRN) - UGRADN(IDIRN))*RSHALF
          UEVALF(IDRP1) = UEVALF(IDRP1)
     +                  - (UGRADU(IDRP1) - UGRADN(IDRP1))*RSHALF
          UEVALF(IDRP2) = UEVALF(IDRP2)
     +                  - (UGRADU(IDRP2) - UGRADN(IDRP2))*RSHALF

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         NAVIER-STOKES BCS

C         NORMAL COMPONENT OF VISCOUS STRESS TENSOR
C          VISTNF = ZERO

C         TRANSVERSE COMPONENTS OF VISCOUS STRESS TENSOR
          VIST1F = ZERO
          VIST2F = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCI1)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC NON-REFLECTING LAMINAR INFLOW
C         ------------------------------------------

C         NOT IMPLEMENTED
          CONTINUE

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCI2)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC REFLECTING INFLOW WITH SPECIFIED VELOCITY AND TEMP
C         ---------------------------------------------------------------

C         EULER BCS

C         IMPOSE VELOCITY COMPONENTS
          CALL BCVELC(INEXT,RKTIME,UEVALF)

C         IMPOSE TEMPERATURE AND MASS FRACTIONS
          CALL BCTEMP(INEXT,RKTIME,TEVALF)
          CALL BCMASF(INEXT,RKTIME,YEVALF)

C         NAVIER-STOKES BCS

C         NONE IMPOSED

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCI3)THEN

C         ---------------------------------------------------------------------

C         BC: SUBSONIC REFLECTING INFLOW WITH SPECIFIED VELOCITY AND DENSITY
C         ------------------------------------------------------------------

C         EULER BCS

C         IMPOSE VELOCITY COMPONENTS
          CALL BCVELC(INEXT,RKTIME,UEVALF)

C         IMPOSE DENSITY AND MASS FRACTIONS
          CALL BCDENS(INEXT,RKTIME,DEVALF)
          CALL BCMASF(INEXT,RKTIME,YEVALF)

C         TEMPERATURE FROM THERMAL EOS
          CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C         NAVIER-STOKES BCS
C          CALL PRESSR(DEVALF,TEVALF,YEVALF,PEVALF)
C          PEVALF = PEVALF - PREFER

C         NORMAL COMPONENT OF VISCOUS STRESS TENSOR
          VISTNF = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCW1)THEN

C         ---------------------------------------------------------------------

C         BC: ADIABATIC WALL
C         ------------------

C         EULER BCS

C         IMPOSE VELOCITY COMPONENTS
          CALL BCVELC(INEXT,RKTIME,UEVALF)

CC         MIXTURE INTERNAL ENERGY
C          EEVALF = UEVALF(1)*UEVALF(1)
C     +           + UEVALF(2)*UEVALF(2)
C     +           + UEVALF(3)*UEVALF(3)
C          EEVALF = HALF*EEVALF
C          DO ISPEC = 1, NSPEC
C            EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
C          ENDDO
C
CC         TEMPERATURE
C          CALL TEMPER(DEVALF,UEVALF,EEVALF,YEVALF,TEVALF)
C
CC         PRESSURE
C          PEVALF = DEVALF*RGVLFF*TEVALF
C          PEVALF = PEVALF - PREFER

C         NAVIER-STOKES BCS

C         NORMAL GRADIENTS OF VELOCITY
C          UDVALF(1,IDIRN) = (URUN(1,ICELL)-UEVALF(1))/RSOVSZ
C          UDVALF(2,IDIRN) = (URUN(2,ICELL)-UEVALF(2))/RSOVSZ
C          UDVALF(3,IDIRN) = (URUN(3,ICELL)-UEVALF(3))/RSOVSZ
          UDVALF(1,1) = ZERO
          UDVALF(1,2) = ZERO
          UDVALF(1,3) = ZERO
          UDVALF(2,1) = ZERO
          UDVALF(2,2) = ZERO
          UDVALF(2,3) = ZERO
          UDVALF(3,1) = ZERO
          UDVALF(3,2) = ZERO
          UDVALF(3,3) = ZERO
          UDVALF(1,IDIRN) = (UEVALC(1)-UEVALF(1))*RSOVS2
          UDVALF(2,IDIRN) = (UEVALC(2)-UEVALF(2))*RSOVS2
          UDVALF(3,IDIRN) = (UEVALC(3)-UEVALF(3))*RSOVS2

C         VISCOUS STRESS TENSOR
          IDRP1 = MNVIST(1,IDIRN)
          IDRP2 = MNVIST(2,IDIRN)
          DIVUCF = UDVALF(1,1) + UDVALF(2,2) + UDVALF(3,3)
          VISTNF = TWO*UDVALF(IDIRN,IDIRN) - TTHD*DIVUCF
          VIST1F = UDVALF(IDRP1,IDIRN) + UDVALF(IDIRN,IDRP1)
          VIST2F = UDVALF(IDRP2,IDIRN) + UDVALF(IDIRN,IDRP2)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("CFLBCF",6I6)')IPROC,ITIME,IRKSTP,
     +                               ICELL,IFACE,IFCODE(IFACE)
            WRITE(6,'(3(1PE12.4))')UEVALC(1),UEVALC(2),UEVALC(3)
            WRITE(6,'(3(1PE12.4))')UEVALF(1),UEVALF(2),UEVALF(3)
            WRITE(6,'(3(1PE12.4))')UDVALF(1,1),UDVALF(2,1),UDVALF(3,1)
            WRITE(6,'(3(1PE12.4))')VISTNF,VIST1F,VIST2F
          ENDIF

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCW2)THEN

C         ---------------------------------------------------------------------

C         BC: ISOTHERMAL WALL
C         -------------------

C         EULER BCS

C         IMPOSE VELOCITY COMPONENTS
          CALL BCVELC(INEXT,RKTIME,UEVALF)

C         IMPOSE TEMPERATURE
          CALL BCTEMP(INEXT,RKTIME,TEVALF)

CC         MIXTURE INTERNAL ENERGY
C          CALL ENTHPY(DEVALF,TEVALF,HEVALF)
C          EEVALF = UEVALF(1)*UEVALF(1)
C     +           + UEVALF(2)*UEVALF(2)
C     +           + UEVALF(3)*UEVALF(3)
C          EEVALF = HALF*EEVALF
C          DO ISPEC = 1, NSPEC
C            EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
C          ENDDO
C
CC         PRESSURE
C          PEVALF = DEVALF*RGVLFF*TEVALF
C          PEVALF = PEVALF - PREFER

C         NAVIER-STOKES BCS

C         MOLECULAR TRANSPORT COEFFICIENTS
          CALL TRANCO(TEVALF,CPVLFF,VISCOF,TCONDF,DIFFMF)

C         NORMAL GRADIENTS OF VELOCITY
C          UDVALF(1,IDIRN) = (URUN(1,ICELL)-UEVALF(1))/RSOVSZ
C          UDVALF(2,IDIRN) = (URUN(2,ICELL)-UEVALF(2))/RSOVSZ
C          UDVALF(3,IDIRN) = (URUN(3,ICELL)-UEVALF(3))/RSOVSZ
          UDVALF(1,1) = ZERO
          UDVALF(1,2) = ZERO
          UDVALF(1,3) = ZERO
          UDVALF(2,1) = ZERO
          UDVALF(2,2) = ZERO
          UDVALF(2,3) = ZERO
          UDVALF(3,1) = ZERO
          UDVALF(3,2) = ZERO
          UDVALF(3,3) = ZERO
          UDVALF(1,IDIRN) = (UEVALC(1)-UEVALF(1))*RSOVS2
          UDVALF(2,IDIRN) = (UEVALC(2)-UEVALF(2))*RSOVS2
          UDVALF(3,IDIRN) = (UEVALC(3)-UEVALF(3))*RSOVS2

C         VISCOUS STRESS TENSOR
          IDRP1 = MNVIST(1,IDIRN)
          IDRP2 = MNVIST(2,IDIRN)
          DIVUCF = UDVALF(1,1) + UDVALF(2,2) + UDVALF(3,3)
          VISTNF = TWO*UDVALF(IDIRN,IDIRN) - TTHD*DIVUCF
          VIST1F = UDVALF(IDRP1,IDIRN) + UDVALF(IDIRN,IDRP1)
          VIST2F = UDVALF(IDRP2,IDIRN) + UDVALF(IDIRN,IDRP2)

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = (TRUN(ICELL)-TEVALF)*RSOVS2
C          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
                  WRITE(6,'(4I5,4(1PE12.4))')ITIME,IRKSTP,ICELL,IFACE,
     +                                 TRUN(ICELL),TEVALF,TDVALF,TCONDF
          ENDIF

C         ---------------------------------------------------------------------

        ELSE IF(IFCODE(IFACE).EQ.NSBCW3)THEN

C         ---------------------------------------------------------------------

C         BC: ADIABATIC SLIP WALL
C         -----------------------

C         EULER BCS

C         IMPOSE VELOCITY COMPONENTS
          UEVALF(IDIRN) = ZERO

C         NAVIER-STOKES BCS

CC         NORMAL GRADIENTS OF VELOCITY
C          UDVALF(1,IDIRN) = (URUN(1,ICELL)-UEVALF(1))/RSOVSZ
C          UDVALF(2,IDIRN) = (URUN(2,ICELL)-UEVALF(2))/RSOVSZ
C          UDVALF(3,IDIRN) = (URUN(3,ICELL)-UEVALF(3))/RSOVSZ
C          UDVALF(1,IDIRN) = ZERO
C          UDVALF(2,IDIRN) = ZERO
C          UDVALF(3,IDIRN) = ZERO

C         VISCOUS STRESS TENSOR
C          IDRP1 = MNVIST(1,IDIRN)
C          IDRP2 = MNVIST(2,IDIRN)
C          DIVUCF = UDVALF(1,1) + UDVALF(2,2) + UDVALF(3,3)
C          VISTNF = TWO*UDVALF(IDIRN,IDIRN) - TTHD*DIVUCF
C          VIST1F = UDVALF(IDRP1,IDIRN) + UDVALF(IDIRN,IDRP1)
C          VIST2F = UDVALF(IDRP2,IDIRN) + UDVALF(IDIRN,IDRP2)
          VISTNF = ZERO
          VIST1F = ZERO
          VIST2F = ZERO

C         NORMAL GRADIENT OF TEMPERATURE
          TDVALF = ZERO

C         NORMAL GRADIENT OF SPECIES MASS FRACTION
          DO ISPEC = 1, NSPEC
            YDVALF(ISPEC) = ZERO
          ENDDO

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ERROR CONDITION - invalid boundary condition code
C         ---------------
C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("CFLBCF",4I5)')IPROC,ICELL,IFACE,IFCODE(IFACE)
          ENDIF
          CALL ERHAND(12,32,4)

C         ---------------------------------------------------------------------

        ENDIF
C       END OF APPLY BOUNDARY CONDITIONS

C       =======================================================================

CC       TEMPERATURE FROM THERMAL EOS
C        CALL TEMPTH(DEVALF,PEVALF+PREFER,YEVALF,TEVALF)

C       SPECIES ENTHALPIES AND MIXTURE INTERNAL ENERGY FROM CALORIC EOS
        CALL ENTHPY(DEVALF,TEVALF,HEVALF)

C       -----------------------------------------------------------------------

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("CFLBCF",4I6)')IPROC,ITIME,IRKSTP
          WRITE(6,'(3I5,2(1PE12.4))')ICELL,IFACE,IFCODE(IFACE),
     +                               PINFTY,PCONST
          WRITE(6,'(4(1PE12.4))')DEVALF,UEVALF(1),UEVALF(2),UEVALF(3)
        WRITE(6,'(5(1PE12.4))')PEVALF,TEVALF,AEVALF,YEVALF(1),YEVALF(2)
        ENDIF

C       MIXTURE INTERNAL ENERGY
        EEVALF = UEVALF(1)*UEVALF(1)
     +         + UEVALF(2)*UEVALF(2)
     +         + UEVALF(3)*UEVALF(3)
        EEVALF = HALF*EEVALF
        DO ISPEC = 1, NSPEC
          EEVALF = EEVALF + YEVALF(ISPEC)*HEVALF(ISPEC)
        ENDDO

C       -----------------------------------------------------------------------

C       PRESSURE
C        PEVALF = PEVALF - PREFER

C       VISCOUS TERMS
        IDRP1 = MNVIST(1,IDIRN)
        IDRP2 = MNVIST(2,IDIRN)
        VISTAU(IDIRN,IDIRN) = VISCOF*VISTNF
        VISTAU(IDRP1,IDIRN) = VISCOF*VIST1F
        VISTAU(IDRP2,IDIRN) = VISCOF*VIST2F
        VISTRF = UEVALF(1)*VISTAU(1,IDIRN)
     +         + UEVALF(2)*VISTAU(2,IDIRN)
     +         + UEVALF(3)*VISTAU(3,IDIRN)

C       THERMAL CONDUCTIVITY TERMS
        TCONDF = TCONDF*TDVALF

C       MASS DIFFUSIVITY TERMS
        DO ISPEC = 1, NSPEC
          YDIFFF(ISPEC) = DIFFMF(ISPEC)*YDVALF(ISPEC)
        ENDDO

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("CFLBCF",6I6)')IPROC,ITIME,IRKSTP,
     +                             ICELL,IFACE,IFCODE(IFACE)
          WRITE(6,'(4(1PE12.4))')DFLUXF,UEVALF(1),UEVALF(2),UEVALF(3)
          WRITE(6,'(3(1PE12.4))')VISTAU(1,1),VISTAU(2,1),VISTAU(3,1)
C          WRITE(6,'(4(1PE12.4))')DFLUXF,EEVALF,TCONDF,VISTRF
        ENDIF

C       -----------------------------------------------------------------------

C       FLUXES
C       ------
C       MASS
        DFLUXF = DEVALF*UEVALF(IDIRN)

C       MOMENTUM
        UFLUXF(1) = DFLUXF*UEVALF(1) + PEVALF*RDELTA(1,IDIRN)
     +            - VISTAU(1,IDIRN)
        UFLUXF(2) = DFLUXF*UEVALF(2) + PEVALF*RDELTA(2,IDIRN)
     +            - VISTAU(2,IDIRN)
        UFLUXF(3) = DFLUXF*UEVALF(3) + PEVALF*RDELTA(3,IDIRN)
     +            - VISTAU(3,IDIRN)

C       ENERGY
        EFLUXF = DFLUXF*EEVALF - TCONDF - VISTRF

C       SPECIES MASS FRACTION
C       ALSO COMPUTE THE DIFFUSIVE CORRECTION
        YDCORR = ZERO
        DO ISPEC = 1, NSPEC
          YFLUXF(ISPEC) = DFLUXF*YEVALF(ISPEC) - YDIFFF(ISPEC)
          YDCORR = YDCORR + YDIFFF(ISPEC)
        ENDDO

C       SPECIES MASS FRACTION
C       APPLY DIFFUSIVE CORRECTION
C       ENERGY EQUATION
C       SPECIES ENTHALPY TRANSPORT
        DO ISPEC = 1, NSPEC
          YDCORY = YDCORR*YEVALF(ISPEC)
          YFLUXF(ISPEC) = YFLUXF(ISPEC) + YDCORY
          EFLUXF = EFLUXF - (YDIFFF(ISPEC)-YDCORY)*HEVALF(ISPEC)
        ENDDO

C       -----------------------------------------------------------------------

C       ADD FLUXES TO RHS ACCUMULATORS
C       ------------------------------

C       SCALING
C        FFACTR = GPFLUX*OCSIZE*RSIGN(IFACE)
        FFACTR = GPFLUX*RSIGN(IFACE)

C       LOCAL CELL ONLY
        DRHS(ICELL) = DRHS(ICELL) + FFACTR*DFLUXF
        URHS(1,ICELL) = URHS(1,ICELL) + FFACTR*UFLUXF(1)
        URHS(2,ICELL) = URHS(2,ICELL) + FFACTR*UFLUXF(2)
        URHS(3,ICELL) = URHS(3,ICELL) + FFACTR*UFLUXF(3)
        ERHS(ICELL) = ERHS(ICELL) + FFACTR*EFLUXF
        DO ISPEC = 1, NSPEC
          YRHS(ISPEC,ICELL) = YRHS(ISPEC,ICELL) + FFACTR*YFLUXF(ISPEC)
        ENDDO

C       -----------------------------------------------------------------------

      ENDDO
C     END OF RUN THROUGH THE GAUSS POINTS

C     =========================================================================


      RETURN
      END
