      SUBROUTINE CHMREP
 
C     *************************************************************************
C
C     CHMREP
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT
C
C     CHANGE RECORD
C     -------------
C     05-OCT-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     REPORTS THE CHEMICAL DATA
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     INTERVAL-MATCHING TOLERANCE FOR THERMODYNAMIC DATA
      DOUBLE PRECISION TTTOL
      PARAMETER(TTTOL = 1.0D-3)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TTEMP(5),TTOLD(5)
      DOUBLE PRECISION FORNOW
      INTEGER IC,JC,JSPEC,KSPEC,JSTEP,ICP,ISTRC1,ISTRC2
      CHARACTER*132 STROUT
      CHARACTER*10 STREAC
      CHARACTER*5 STRSTP
      CHARACTER*4 STRARO
      CHARACTER*1 STRCOF
      

C     BEGIN
C     =====

C     =========================================================================

C     REPORT THE CHEMICAL DATA
C     ========================

      IF(IPROC.EQ.0)THEN

C       =======================================================================

C       OPEN THE REPORT FILE
        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='OLD',FORM='FORMATTED')

C       GO TO EOF
1000    CONTINUE
          READ(NCREPT,'(A)',END=1010)
          GOTO 1000
C       END OF LOOP 1000

1010    BACKSPACE(NCREPT)

C       =======================================================================

C       CHEMICAL DATA HEADER
        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Chemical data:'
        WRITE(NCREPT,*)'-------------'
        WRITE(NCREPT,*)

C       SPECIES LIST
        WRITE(NCREPT,*)'Number of species:'
        WRITE(NCREPT,'(I5)')NSPEC
C       CHECK NUMBER OF SPECIES
        IF(NSPEC.NE.NSPCMX)THEN
          WRITE(NCREPT,*)'Warning: INDATA: mismatch in no. of species'
          WRITE(NCREPT,*)'Global value:'
          WRITE(NCREPT,9100)NSPCMX
          WRITE(NCREPT,*)'Required value in chemical data file:'
          WRITE(NCREPT,9100)NSPEC
          WRITE(NCREPT,*)'Chemical data file value takes precedence'
          WRITE(NCREPT,*)
        ENDIF
        WRITE(NCREPT,*)'List of species:'
        WRITE(NCREPT,*)'  No.  Symbol         Mol. Mass      Lewis no.'
        DO JSPEC = 1,NSPEC
          WRITE(NCREPT,'(I5,3X,A10,3X,1PE12.4,3X,1PE12.4)')
     +          JSPEC,SPCSYM(JSPEC),WMOLAR(JSPEC),CLEWIS(JSPEC)
        ENDDO
        WRITE(NCREPT,*)

C       THERMODYNAMIC DATA
        WRITE(NCREPT,*)'Species thermodynamic data:'
        WRITE(NCREPT,'("  Reference pressure: ",1PE12.4)')PREFGB
        WRITE(NCREPT,*)' Spec.  No of T intervals   ',
     +                'Interval   T low       T high       No of coeffs'
        DO JSPEC = 1,NSPEC
          IC = 1
          WRITE(NCREPT,'(I5,6X,I5,9X,I5,8X,2(1PE12.4),I8)')
     +          JSPEC,NTINT(JSPEC),
     +          IC,TINTLO(IC,JSPEC),TINTHI(IC,JSPEC),NCOFCP(IC,JSPEC)
          DO IC = 2,NTINT(JSPEC)
            WRITE(NCREPT,'(25X,I5,8X,2(1PE12.4),I8)')
     +          IC,TINTLO(IC,JSPEC),TINTHI(IC,JSPEC),NCOFCP(IC,JSPEC)

          ENDDO
        ENDDO
        WRITE(NCREPT,*)'Cp coeffs by mass'
        WRITE(NCREPT,*)' Spec.  T int.  Coeff no.  Coeff.'
        DO JSPEC = 1,NSPEC
          DO IC = 1,NTINT(JSPEC)
            JC = 1
            WRITE(NCREPT,'(I5,2X,I5,5X,I5,4X,1PE15.7)')
     +            JSPEC,IC,JC,AMASCP(JC,IC,JSPEC)
            DO JC = 2,NCOFCP(IC,JSPEC)
              WRITE(NCREPT,'(17X,I5,4X,1PE15.7)')JC,AMASCP(JC,IC,JSPEC)
            ENDDO
          ENDDO
        ENDDO

        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Mass-specific Cp, Enthalpy, Entropy;',
     +                 '  Molar Gibbs fn.:'
        WRITE(NCREPT,*)' Spec.  T int.  Temp.       Cp',
     +                 '          Enthalpy    Entropy     Molar Gibbs'
        DO JSPEC = 1,NSPEC
          DO IC = 1,NTINT(JSPEC)
            DO ICP = 1,2

C             TEMPERATURE
              IF(ICP.EQ.1)TTEMP(1) = TINTLO(IC,JSPEC)
              IF(ICP.EQ.2)TTEMP(1) = TINTHI(IC,JSPEC)

C             MASS-SPECIFIC CP
              FORNOW = AMASCP(NCPOLY(IC,JSPEC),IC,JSPEC)
              DO JC = NCPOM1(IC,JSPEC),1,-1
                FORNOW = FORNOW*TTEMP(1) + AMASCP(JC,IC,JSPEC)
              ENDDO
              TTEMP(2) = FORNOW

C             MASS-SPECIFIC ENTHALPY
              FORNOW = AMASCH(NCPOLY(IC,JSPEC),IC,JSPEC)
              DO JC = NCPOM1(IC,JSPEC),1,-1
                FORNOW = FORNOW*TTEMP(1) + AMASCH(JC,IC,JSPEC)
              ENDDO
              FORNOW = AMASCH(NCENTH(IC,JSPEC),IC,JSPEC)
     +               + FORNOW*TTEMP(1)
              TTEMP(3) = FORNOW

C             MASS-SPECIFIC ENTROPY
              FORNOW = AMASCS(NCPOLY(IC,JSPEC),IC,JSPEC)
              DO JC = NCPOM1(IC,JSPEC),2,-1
                FORNOW = FORNOW*TTEMP(1) + AMASCS(JC,IC,JSPEC)
              ENDDO
              FORNOW = AMASCS(NCENPY(IC,JSPEC),IC,JSPEC)
     +               + FORNOW*TTEMP(1)
     +               + AMASCS(1,IC,JSPEC)*LOG(TTEMP(1))
              TTEMP(4) = FORNOW

C             MOLAR GIBBS FUNCTION
C             ACTUALLY GIBBS/(R^0 T) WITH PRESSURE TERM
              FORNOW = AMOLGB(NCPOLY(IC,JSPEC),IC,JSPEC)
              DO JC = NCPOM1(IC,JSPEC),1,-1
                FORNOW = AMOLGB(JC,IC,JSPEC)
     +                 + FORNOW*TTEMP(1)
              ENDDO
              FORNOW = AMOLGB(NCENTH(IC,JSPEC),IC,JSPEC)
     +                /TTEMP(1)
     +               - AMOLGB(NCENPY(IC,JSPEC),IC,JSPEC)
     +                *LOG(TTEMP(1))
     +               - FORNOW
              TTEMP(5) = FORNOW

              IF(ICP.EQ.1)THEN
                IF(IC.EQ.1)THEN
C                  WRITE(NCREPT,'(I5,2X,I5,X,"l",X,5(1PE12.4))')
C     +                  JSPEC,IC,(TTEMP(JC),JC=1,5)
                  WRITE(NCREPT,'(I5,2X,I5,X,"l",X,3(1PE17.9))')
     +                  JSPEC,IC,(TTEMP(JC),JC=1,3)
                ELSE
C                  WRITE(NCREPT,'(7X,I5,X,"l",X,5(1PE12.4))')
C     +                        IC,(TTEMP(JC),JC=1,5)
                  WRITE(NCREPT,'(7X,I5,X,"l",X,3(1PE17.9))')
     +                        IC,(TTEMP(JC),JC=1,3)
                  DO JC = 1,5
      IF(ABS(TTOLD(JC)-TTEMP(JC)).GT.ABS(TTTOL*TTEMP(JC)))THEN
        WRITE(NCREPT,*)'Warning: INDATA: Mismatched thermo data'
        WRITE(NCREPT,'(I7,1PE12.4)')JC,TTEMP(JC)
      ENDIF
                  ENDDO
                ENDIF
              ELSE
C                WRITE(NCREPT,'(7X,I5,X,"h",X,5(1PE12.4))')
C     +                        IC,(TTEMP(JC),JC=1,5)
                WRITE(NCREPT,'(7X,I5,X,"h",X,3(1PE17.9))')
     +                        IC,(TTEMP(JC),JC=1,3)
              ENDIF

              IF(ICP.EQ.2)THEN
                DO JC = 1,5
                  TTOLD(JC) = TTEMP(JC)
                ENDDO
              ENDIF

            ENDDO
          ENDDO
        ENDDO
        WRITE(NCREPT,*)

C       REACTION MECHANISM
C       STEP LIST
        WRITE(NCREPT,*)'Reaction mechanism:'
        WRITE(NCREPT,*)'  Number of steps:'
        WRITE(NCREPT,'(I5)')NSTEP
        IF(NSTEP.NE.NSTPMX)THEN
          WRITE(NCREPT,*)'Warning: mismatch in number of steps:' 
          WRITE(NCREPT,9200)NSTEP,NSTPMX
        ENDIF
        DO JSTEP = 1,NSTEP

          WRITE(STRSTP,'(I5)')JSTEP
          ISTRC1 = 1
          ISTRC2 = LEN(STRSTP) + 3
          STROUT(ISTRC1:ISTRC2) = STRSTP//'   '
          DO JSPEC = 1,NRSLEN(JSTEP)
            STREAC = SPCSYM(NRSPEC(JSPEC,JSTEP))
            ISTRC1 = ISTRC2 + 1
            ISTRC2 = ISTRC1 + LEN(STREAC)
            STROUT(ISTRC1:ISTRC2) = STREAC
            ISTRC1 = ISTRC2 + 1
            ISTRC2 = ISTRC1
            STROUT(ISTRC1:ISTRC2) = '+'
          ENDDO
          IF(MBLIST(JSTEP).GT.0)THEN
            STREAC = BDYSYM(MBLIST(JSTEP))
            ISTRC1 = ISTRC2 + 1
            ISTRC2 = ISTRC1 + LEN(STREAC)
            STROUT(ISTRC1:ISTRC2) = STREAC
          ELSE
            ISTRC2 = ISTRC2 - 1
          ENDIF
          STRARO = ' => '
          IF(MGLIST(JSTEP).GT.0)STRARO = ' == '
          ISTRC1 = ISTRC2 + 1
          ISTRC2 = ISTRC1 + LEN(STRARO)
          STROUT(ISTRC1:ISTRC2) = STRARO
          DO JSPEC = 1,NSSLEN(JSTEP)
            ICP = 0
            DO KSPEC = 1,NRSLEN(JSTEP)
              IF(NRSPEC(KSPEC,JSTEP).EQ.NSSPEC(JSPEC,JSTEP))THEN
                ICP = ICP + 1
              ENDIF
            ENDDO
            ICP = NINT(DIFFMU(JSPEC,JSTEP)) + ICP
            IF(ICP.GT.0)THEN
              IF(ICP.GT.1)THEN
                WRITE(STRCOF,'(I1)')ICP
                ISTRC1 = ISTRC2 + 1
                ISTRC2 = ISTRC1 + LEN(STRCOF)
                STROUT(ISTRC1:ISTRC2) = STRCOF
              ENDIF
              STREAC = SPCSYM(NSSPEC(JSPEC,JSTEP))
              ISTRC1 = ISTRC2 + 1
              ISTRC2 = ISTRC1 + LEN(STREAC)
              STROUT(ISTRC1:ISTRC2) = STREAC
              ISTRC1 = ISTRC2 + 1
              ISTRC2 = ISTRC1
              STROUT(ISTRC1:ISTRC2) = '+'
            ENDIF
          ENDDO
          IF(MBLIST(JSTEP).GT.0)THEN
            STREAC = BDYSYM(MBLIST(JSTEP))
            ISTRC1 = ISTRC2 + 1
            ISTRC2 = ISTRC1 + LEN(STREAC)
            STROUT(ISTRC1:ISTRC2) = STREAC
          ELSE
            ISTRC2 = ISTRC2 - 1
          ENDIF
          
          WRITE(NCREPT,'(A)')STROUT(1:ISTRC2)

        ENDDO

        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Reaction parameters A, n, E:'
        DO JSTEP = 1,NSTEP
          WRITE(NCREPT,'(I5,3(2X,1PE12.4))')
     +          JSTEP,(RPARAM(ICP,JSTEP),ICP=1,3)
        ENDDO

        IF(NLIND.GT.0)THEN
          WRITE(NCREPT,*)
          WRITE(NCREPT,*)'Lindemann parameters:'
          DO JSTEP = 1,NSTEP
            IF(MLLIST(JSTEP).NE.0)THEN
              WRITE(NCREPT,'(I5,3(2X,1PE12.4))')
     +              JSTEP,(RCLIND(ICP,MLLIST(JSTEP)),ICP=1,3)
            ENDIF
          ENDDO
        ENDIF

        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Third body efficiencies:'
        DO KSPEC = 1,NBODY
          WRITE(NCREPT,'(I5,3X,A10)')KSPEC,BDYSYM(KSPEC)
          DO JSPEC = 1,NSPEC
            WRITE(NCREPT,'(I5,3X,A10,1PE12.4)')
     +            JSPEC,SPCSYM(JSPEC),EFFY3B(JSPEC,KSPEC)
          ENDDO
        ENDDO

        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'End of chemical data'
        WRITE(NCREPT,*)'--------------------'
        WRITE(NCREPT,*)
        WRITE(NCREPT,9000)CLDASH

C       =======================================================================

C       CLOSE THE REPORT FILE
        CLOSE(NCREPT)

C       =======================================================================

      ENDIF

C     =========================================================================

C     END OF REPORTING OF CHEMICAL DATA

C     =========================================================================


      RETURN

C     FORMATS FOR OUTPUT OF INITIAL DATA
9000  FORMAT(A79)
9100  FORMAT(3I6)
9200  FORMAT(' Actual (NSPEC) = ',I5,2X,'Max (NSPCMX) = ',I5)

      END
