      SUBROUTINE CONTIN 
 
C     *************************************************************************
C
C     CONTIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     08-JUN-2008:  CREATED
C      
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES RUN CONTROL DATA
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION PCOUNT
      INTEGER NCOUNT
      INTEGER ISPEC,JSPEC
      INTEGER IC
      INTEGER IROOT


C     BEGIN
C     =====

C     =========================================================================

C     DATA IS READ BY THE LOWEST-RANKED PROCESSOR AND BROADCAST TO THE OTHERS
C     -----------------------------------------------------------------------

C     SET ID OF ROOT (BROADCASTING) PROCESSOR
      IROOT = 0

C     INITIALISE THE BROADCAST COUNTER
      PCOUNT = ZERO

C     INITIALISE THE BROADCAST ARRAY
      DO NCOUNT = 1,NPARMX
        PARRAY(NCOUNT) = ZERO
      ENDDO

C     =========================================================================

C     LOWEST-RANKED PROCESSOR
      IF(IPROC.EQ.0)THEN

C       =======================================================================

C       READ THE RUN CONTROL DATA FILE
C       ------------------------------
C       -----------------------------------------------------------------------

        OPEN(UNIT=NCCONT,FILE=FNCONT,STATUS='OLD',FORM='FORMATTED')

C       -----------------------------------------------------------------------

C       READ AND IGNORE THE HEADER
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
C       AND THE SUBSEQUENT BLANK LINE
        READ(NCCONT,*)

C       -----------------------------------------------------------------------

C       GRID SIZE DATA
C       --------------
C       REQUIRED GLOBAL DOMAIN SIZE X,Y,Z
        READ(NCCONT,*)
        READ(NCCONT,*)XGDLEN,YGDLEN,ZGDLEN
        READ(NCCONT,*)

C       REQD. GLOBAL INITIAL MESH (NX), BASE/NO OF LEVELS, SPATIAL STEP SWITCH
        READ(NCCONT,*)
        READ(NCCONT,*)NXGREQ,NLEVBQ,NLEVRQ,NSTMSW
        READ(NCCONT,*)

C       TIME STEPPING DATA
C       ------------------
C       TIME STEP, START STEP, NO OF STEPS, TIME STEP SWITCH
        READ(NCCONT,*)
        READ(NCCONT,*)TSTEP,NTIME1,NTIME,NSTPSW
        READ(NCCONT,*)

C       INTERVALS BETWEEN DUMPS,REPORTS,STATISTICS
C       ------------------------------------------
C       DUMP OUTPUT FORMAT SWITCH (0=UNFORMATTED. 1=FORMATTED)
        READ(NCCONT,*)
        READ(NCCONT,*)NTDUMP,NTREPT,NTSTAT,NTRESO,NDOFMT,NROFMT
        READ(NCCONT,*)

C       COLD START SWITCH (0=COLD START, 1=RESTART)
C       -----------------
C       DUMP INPUT FORMAT SWITCH (0=UNFORMATTED. 1=FORMATTED)
        READ(NCCONT,*)
        READ(NCCONT,*)ICFLAG,NDIFMT
        READ(NCCONT,*)

C       INITIAL TURBULENCE (0=INITIAL TURBULENCE OFF, 1=ON)
C       ------------------
C       RANDOM SEED, SPECTRUM PARAMETERS (FOUR)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)INTURB,ITSEED,(SPARAM(IC),IC=1,NSPARM)
        READ(NCCONT,*)

C       FLAME START OPTION (0=FLAME GENERATOR OFF, 1=ON)
C       ------------------
        READ(NCCONT,*)
        READ(NCCONT,*)INMESH,INFLAM
        READ(NCCONT,*)

C       DEFAULT INITIAL CONDITIONS
C       --------------------------
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)PRIN,TRIN,URIN,VRIN,WRIN,SRIN
        READ(NCCONT,*)
        READ(NCCONT,*)NSPREQ
        DO ISPEC = 1, NSPREQ
          READ(NCCONT,*)JSPEC,YRIN(ISPEC)
        ENDDO
        READ(NCCONT,*)

C       GLOBAL BOUNDARY CONDITION TYPES
C       -------------------------------
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)NGLOBC(1),(NGBPRM(IC,1),IC=1,NBCPRI),
     +                          (RGBPRM(IC,1),IC=1,NBCPRR)
        READ(NCCONT,*)NGLOBC(2),(NGBPRM(IC,2),IC=1,NBCPRI),
     +                          (RGBPRM(IC,2),IC=1,NBCPRR)
        READ(NCCONT,*)NGLOBC(3),(NGBPRM(IC,3),IC=1,NBCPRI),
     +                          (RGBPRM(IC,3),IC=1,NBCPRR)
        READ(NCCONT,*)NGLOBC(4),(NGBPRM(IC,4),IC=1,NBCPRI),
     +                          (RGBPRM(IC,4),IC=1,NBCPRR)
        READ(NCCONT,*)NGLOBC(5),(NGBPRM(IC,5),IC=1,NBCPRI),
     +                          (RGBPRM(IC,5),IC=1,NBCPRR)
        READ(NCCONT,*)NGLOBC(6),(NGBPRM(IC,6),IC=1,NBCPRI),
     +                          (RGBPRM(IC,6),IC=1,NBCPRR)
        READ(NCCONT,*)

C       -----------------------------------------------------------------------

C       MESH REFINEMENT CRITERIA, INTERVAL, MIN/MAX MESH LEVELS
C       -------------------------------------------------------
        READ(NCCONT,*)
        READ(NCCONT,*)DRETOL,DDETOL,NTMINT,NMLMIN,NMLMAX
        READ(NCCONT,*)

C       -----------------------------------------------------------------------

C       MESH REFINEMENT SCALING FACTORS
C       -------------------------------
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)SFREFD,SFREFT,SFREFU(1),SFREFU(2),SFREFU(3),SFREFS
        READ(NCCONT,*)
        DO ISPEC = 1, NSPREQ
          READ(NCCONT,*)JSPEC,SFREFY(ISPEC)
        ENDDO
        READ(NCCONT,*)

C       -----------------------------------------------------------------------

C       READ END-OF-FILE LINE
        READ(NCCONT,*)

        CLOSE(NCCONT)

C       =======================================================================

C       COMPRESS THE DATA
C       -----------------

C       REQUIRED GLOBAL DOMAIN SIZE (x,y,z)
        NCOUNT = 1
        PARRAY(NCOUNT) = XGDLEN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = YGDLEN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = ZGDLEN

C       REQUIRED GLOBAL BASE MESH SIZE (NX), NO OF LEVELS, SPATIAL STEP SWITCH
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NXGREQ)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NLEVBQ)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NLEVRQ)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NSTMSW)

C       TIME STEP, START STEP, NO OF STEPS, TIME STEP SWITCH
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = TSTEP
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTIME1)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTIME)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NSTPSW)

C       INTERVALS BETWEEN DUMPS,REPORTS,STATISTICS; DUMP OUTPUT SWITCH
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTDUMP)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTREPT)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTSTAT)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTRESO)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NDOFMT)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NROFMT)

C       COLD START SWITCH, DUMP INPUT SWITCH
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(ICFLAG)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NDIFMT)

C       INITIAL TURBULENCE
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(INTURB)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(ITSEED)
        DO ISPEC = 1, NSPARM
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = SPARAM(ISPEC)
        ENDDO

C       MESH START OPTION
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(INMESH)

C       FLAME START OPTION
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(INFLAM)

C       DEFAULT INITIAL CONDITIONS
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = PRIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = TRIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = URIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = VRIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = WRIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SRIN
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NSPREQ)
        DO ISPEC = 1, NSPREQ
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = YRIN(ISPEC)
        ENDDO

C       GLOBAL BOUNDARY CONDITION TYPES
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(1))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,1))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,1)
        ENDDO
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(2))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,2))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,2)
        ENDDO
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(3))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,3))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,3)
        ENDDO
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(4))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,4))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,4)
        ENDDO
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(5))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,5))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,5)
        ENDDO
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NGLOBC(6))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = REAL(NGBPRM(IC,6))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = RGBPRM(IC,6)
        ENDDO

C       MESH REFINEMENT CRITERIA AND INTERVAL
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = DRETOL
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = DDETOL
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NTMINT)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NMLMIN)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = REAL(NMLMAX)

C       MESH REFINEMENT SCALING FACTORS
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFD
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFT
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFU(1)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFU(2)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFU(3)
        NCOUNT = NCOUNT + 1
        PARRAY(NCOUNT) = SFREFS
        DO ISPEC = 1, NSPREQ
          NCOUNT = NCOUNT + 1
          PARRAY(NCOUNT) = SFREFY(ISPEC)
        ENDDO

C       BROADCAST COUNTER
        PCOUNT = REAL(NCOUNT)

C       CHECK BROADCAST COUNTER AGAINST BROADCAST ARRAY SIZE
        IF(NCOUNT.GT.NPARMX)THEN
        WRITE(NCREPT,*)'Warning: INCONT: broadcast array size too small'
        ENDIF

C       =======================================================================

      ENDIF

C     =========================================================================

C     BROADCAST (OR RECEIVE) THE COUNTER
C     ----------------------------------
      CALL P_BCST(PCOUNT,1,1,IROOT)
      NCOUNT = NINT(PCOUNT)

C     BROADCAST (OR RECEIVE) THE DATA
C     -------------------------------
      CALL P_BCST(PARRAY,NPARMX,NCOUNT,IROOT)

C     =========================================================================

C     NOT THE LOWEST-RANKED PROCESSOR
      IF(IPROC.NE.0)THEN

C       =======================================================================

C       UNCOMPRESS THE DATA
C       -------------------

C       REQUIRED GLOBAL GRID SIZE X,Y,Z
        NCOUNT = 1
        XGDLEN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        YGDLEN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        ZGDLEN = PARRAY(NCOUNT)

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,3(1PE12.4))')IPROC,NCOUNT,
C     +  XGDLEN,YGDLEN,ZGDLEN 

C       REQUIRED GLOBAL BASE MESH SIZE (NX), NO OF LEVELS, SPATIAL STEP SWITCH
        NCOUNT = NCOUNT + 1
        NXGREQ = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NLEVBQ = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NLEVRQ = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NSTMSW = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,4I5)')IPROC,NCOUNT,
C     +  NXGREQ,NLEVBQ,NLEVRQ,NSTMSW

C       TIME STEP, START STEP, NO OF STEPS, TIME STEP SWITCH
        NCOUNT = NCOUNT + 1
        TSTEP = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        NTIME1 = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NTIME = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NSTPSW = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,1PE12.4,3I5)')IPROC,NCOUNT,
C     +  TSTEP,NTIME1,NTIME,NSTPSW

C       INTERVALS BETWEEN DUMPS,REPORTS,STATISTICS, DUMP OUTPUT SWITCH
        NCOUNT = NCOUNT + 1
        NTDUMP = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NTREPT = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NTSTAT = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NTRESO = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NDOFMT = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NROFMT = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,6I5)')IPROC,NCOUNT,
C     +  NTDUMP,NTREPT,NTSTAT,NTRESO,NDOFMT,NROFMT

C       COLD START SWITCH, DUMP INPUT SWITCH
        NCOUNT = NCOUNT + 1
        ICFLAG = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NDIFMT = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,2I5)')IPROC,NCOUNT,
C     +  ICFLAG,NDIFMT

C       INITIAL TURBULENCE
        NCOUNT = NCOUNT + 1
        INTURB = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        ITSEED = NINT(PARRAY(NCOUNT))
        DO ISPEC = 1, NSPARM
          NCOUNT = NCOUNT + 1
          SPARAM(ISPEC) = PARRAY(NCOUNT)
        ENDDO

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,2I5,7(1PE12.4))')IPROC,NCOUNT,
C     +  INTURB,ITSEED,(SPARAM(ISPEC),ISPEC=1,NSPARM)

C       MESH START OPTION
        NCOUNT = NCOUNT + 1
        INMESH = NINT(PARRAY(NCOUNT))

C       FLAME START OPTION
        NCOUNT = NCOUNT + 1
        INFLAM = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,I5)')IPROC,NCOUNT,
C     +  INFLAM

C       DEFAULT INITIAL CONDITIONS
        NCOUNT = NCOUNT + 1
        PRIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        TRIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        URIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        VRIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        WRIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SRIN = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        NSPREQ = NINT(PARRAY(NCOUNT))
        DO ISPEC = 1, NSPREQ
          NCOUNT = NCOUNT + 1
          YRIN(ISPEC) = PARRAY(NCOUNT)
        ENDDO

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2I5,8(1PE12.4))')IPROC,NCOUNT,
C     +  PRIN,TRIN,URIN,VRIN,WRIN,SRIN,(YRIN(ISPEC),ISPEC=1,NSPREQ)

C       GLOBAL BOUNDARY CONDITION TYPES
        NCOUNT = NCOUNT + 1
        NGLOBC(1) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,1) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,1) = PARRAY(NCOUNT)
        ENDDO
        NCOUNT = NCOUNT + 1
        NGLOBC(2) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,2) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,2) = PARRAY(NCOUNT)
        ENDDO
        NCOUNT = NCOUNT + 1
        NGLOBC(3) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,3) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,3) = PARRAY(NCOUNT)
        ENDDO
        NCOUNT = NCOUNT + 1
        NGLOBC(4) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,4) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,4) = PARRAY(NCOUNT)
        ENDDO
        NCOUNT = NCOUNT + 1
        NGLOBC(5) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,5) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,5) = PARRAY(NCOUNT)
        ENDDO
        NCOUNT = NCOUNT + 1
        NGLOBC(6) = NINT(PARRAY(NCOUNT))
        DO IC = 1, NBCPRI
          NCOUNT = NCOUNT + 1
          NGBPRM(IC,6) = NINT(PARRAY(NCOUNT))
        ENDDO
        DO IC = 1, NBCPRR
          NCOUNT = NCOUNT + 1
          RGBPRM(IC,6) = PARRAY(NCOUNT)
        ENDDO

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",I5,4I5,4(1PE12.4))')NGLOBC(6),
C     +  (NGBPRM(IC,6),IC=1,NBCPRI),
C     +  (RGBPRM(IC,6),IC=1,NBCPRR)


C       MESH REFINEMENT CRITERIA AND INTERVAL
        NCOUNT = NCOUNT + 1
        DRETOL = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        DDETOL = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        NTMINT = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NMLMIN = NINT(PARRAY(NCOUNT))
        NCOUNT = NCOUNT + 1
        NMLMAX = NINT(PARRAY(NCOUNT))

CC       DIAGNOSTICS
C        WRITE(6,'("CONTIN",2(1PE12.4),3I5)')DRETOL,DDETOL,
C     +                              NTMINT,NMLMIN,NMLMAX

C       MESH REFINEMENT SCALING FACTORS
        NCOUNT = NCOUNT + 1
        SFREFD = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SFREFT = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SFREFU(1) = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SFREFU(2) = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SFREFU(3) = PARRAY(NCOUNT)
        NCOUNT = NCOUNT + 1
        SFREFS = PARRAY(NCOUNT)
        DO ISPEC = 1, NSPREQ
          NCOUNT = NCOUNT + 1
          SFREFY(ISPEC) = PARRAY(NCOUNT)
        ENDDO

C       =======================================================================

      ENDIF

C     =========================================================================


      RETURN
      END
