      SUBROUTINE CRECAT
 
C     *************************************************************************
C
C     CRECAT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     27-OCT-2018:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL RECONSTRUCTION IN EACH CELL
C     COMPUTES AND STORES THE MOORE-PENROSE PSEUDO-INVERSE
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION AMATRX(NCSTMX,NPCFMX)
      DOUBLE PRECISION VMATRX(NPCFMX,NPCFMX)
      DOUBLE PRECISION WVECTR(NPCFMX)
      DOUBLE PRECISION XCENTC,YCENTC,ZCENTC,CCSIZE,OCSIZE
      DOUBLE PRECISION XSTENC,YSTENC,ZSTENC,CSSIZE
      DOUBLE PRECISION HCTERM,HSTERM,HSXTRA,HSDIFF
      DOUBLE PRECISION STORE1,STORE2,STORE3,STORE4
      DOUBLE PRECISION FORNOW
      INTEGER NJSTEN,NKPOLY
      INTEGER JCLEVL,JCSTEN,JSCELL,JSLEVL,JKPOLY,JCOUNT

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     PRELIMINARIES
C     -------------

C     CENTRAL CELL COORDINATES AND SIZE
      XCENTC = XCENTR(ICELL)
      YCENTC = YCENTR(ICELL)
      ZCENTC = ZCENTR(ICELL)
      JCLEVL = NLEVEL(ICELL)
      CCSIZE = HCSIZE(JCLEVL)
      OCSIZE = OVSIZE(JCLEVL)

C     CENTRAL CELL SIZE TERM
      HCTERM = TWLFTH

C     =========================================================================

C     POLYNOMIAL COEFFICIENTS
C     -----------------------

C     ASSEMBLE THE COEFFICIENT MATRIX
C     -------------------------------

C     STENCIL SIZE AND NUMBER OF POLYNOMIAL COEFFICIENTS
      NJSTEN = NOSSTN(ICELL)
      NKPOLY = NPCAC4

C     FOURTH ORDER

C     RUN THROUGH THE STENCIL
      DO JCSTEN = 1, NJSTEN

C       STENCIL CELL COORDINATES AND SIZE
        JSCELL = NSSTEN(JCSTEN,ICELL)
        XSTENC = (XCENTR(JSCELL) - XCENTC)
        XSTENC = XSTENC - XGDLEN*NINT(XSTENC/XGDLEN)
        XSTENC = XSTENC*OCSIZE
        YSTENC = (YCENTR(JSCELL) - YCENTC)
        YSTENC = YSTENC - YGDLEN*NINT(YSTENC/YGDLEN)
        YSTENC = YSTENC*OCSIZE
        ZSTENC = (ZCENTR(JSCELL) - ZCENTC)
        ZSTENC = ZSTENC - ZGDLEN*NINT(ZSTENC/ZGDLEN)
        ZSTENC = ZSTENC*OCSIZE
        JSLEVL = NLEVEL(JSCELL)
        CSSIZE = HCSIZE(JSLEVL)*OCSIZE

C       STENCIL CELL SIZE TERMS
        HSTERM = TWLFTH*CSSIZE*CSSIZE
        HSXTRA = TWO*HSTERM
        HSDIFF = HSTERM - HCTERM

C       COEFFICIENT MATRIX ELEMENTS
        AMATRX(JCSTEN,1) = XSTENC
        STORE1 = XSTENC*XSTENC
        AMATRX(JCSTEN,2) = STORE1 + HSDIFF
        STORE1 = STORE1 + HSTERM
        AMATRX(JCSTEN,3) = (STORE1 + HSXTRA)*XSTENC

        AMATRX(JCSTEN,4) = YSTENC
        STORE4 = YSTENC*XSTENC
        AMATRX(JCSTEN,5) = STORE4
        STORE2 = YSTENC*YSTENC
        AMATRX(JCSTEN,6) = STORE2 + HSDIFF
        AMATRX(JCSTEN,7) = STORE1*YSTENC
        STORE2 = STORE2 + HSTERM
        AMATRX(JCSTEN,8) = STORE2*XSTENC
        AMATRX(JCSTEN,9) = (STORE2 + HSXTRA)*YSTENC

        AMATRX(JCSTEN,10) = ZSTENC
        AMATRX(JCSTEN,11) = ZSTENC*XSTENC
        AMATRX(JCSTEN,12) = ZSTENC*YSTENC
        STORE3 = ZSTENC*ZSTENC
        AMATRX(JCSTEN,13) = STORE3 + HSDIFF
        AMATRX(JCSTEN,14) = STORE1*ZSTENC
        AMATRX(JCSTEN,15) = STORE4*ZSTENC
        STORE3 = STORE3 + HSTERM
        AMATRX(JCSTEN,16) = STORE3*XSTENC
        AMATRX(JCSTEN,17) = STORE2*ZSTENC
        AMATRX(JCSTEN,18) = STORE3*YSTENC
        AMATRX(JCSTEN,19) = (STORE3 + HSXTRA)*ZSTENC

      ENDDO
C     RUN THROUGH THE STENCIL

C     -------------------------------------------------------------------------

C     COMPUTE THE MOORE-PENROSE PSEUDO-INVERSE
C     ----------------------------------------
C     AS U,V^T,W

C     SINGULAR VALUE DECOMPOSITION
      CALL SVDCMP(AMATRX,NJSTEN,NKPOLY,NCSTMX,NPCFMX,WVECTR,VMATRX)

C     -------------------------------------------------------------------------

C     STORE THE MOORE-PENROSE PSEUDO-INVERSE
C     A^-1 = V W^-1 U^T
      DO JKPOLY = 1, NKPOLY
        DO JCSTEN = 1, NJSTEN
          FORNOW = ZERO
          DO JCOUNT = 1, NKPOLY
            FORNOW = FORNOW 
     +    + AMATRX(JCSTEN,JCOUNT)*VMATRX(JKPOLY,JCOUNT)/WVECTR(JCOUNT)
          ENDDO
          AMPINV(JCSTEN,JKPOLY,NMPINV) = FORNOW
        ENDDO
      ENDDO

C     =========================================================================


      RETURN
      END
