      SUBROUTINE CRESTN(AMATRX,UMATRX,VMATRX,WVECTR)
 
C     *************************************************************************
C
C     CRESTN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     17-OCT-2018:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL RECONSTRUCTION IN EACH CELL
C     FOR STENCIL TESTING
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLSVD
      PARAMETER(TOLSVD = 1.0D-5)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION AMATRX(NCSTMX,NPCFMX)
      DOUBLE PRECISION UMATRX(NCSTMX,NPCFMX)
      DOUBLE PRECISION VMATRX(NPCFMX,NPCFMX)
      DOUBLE PRECISION WVECTR(NPCFMX)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION BVECTR(NCSTMX),DVECTR(NCSTMX)
      DOUBLE PRECISION ACOEFF(NPCFMX)
      DOUBLE PRECISION XCENTC,YCENTC,ZCENTC,CCSIZE,OCSIZE
      DOUBLE PRECISION XSTENC,YSTENC,ZSTENC,CSSIZE
      DOUBLE PRECISION HCTERM,HSTERM,HSXTRA,HSDIFF
      DOUBLE PRECISION STORE1,STORE2,STORE3,STORE4
      DOUBLE PRECISION DFCENT
      DOUBLE PRECISION THRESH,WVCMAX
      INTEGER NJSTEN,NKPOLY
      INTEGER JCLEVL,JCSTEN,JSCELL,JSLEVL,JKPOLY

C     DIAGNOSTICS
      DOUBLE PRECISION WVCMIN
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CRESTN',ICELL

C     =========================================================================

C     PRELIMINARIES
C     -------------

C     CENTRAL CELL COORDINATES AND SIZE
      XCENTC = XCENTR(ICELL)
      YCENTC = YCENTR(ICELL)
      ZCENTC = ZCENTR(ICELL)
      JCLEVL = NLEVEL(ICELL)
      CCSIZE = HCSIZE(JCLEVL)
      OCSIZE = OVSIZE(JCLEVL)

C     CENTRAL CELL SIZE TERM
      HCTERM = TWLFTH

C     =========================================================================

C     POLYNOMIAL COEFFICIENTS
C     -----------------------

C     ASSEMBLE THE COEFFICIENT MATRIX
C     -------------------------------

C     STENCIL SIZE AND NUMBER OF POLYNOMIAL COEFFICIENTS
      NJSTEN = NOSSTN(ICELL)
      NKPOLY = NPCAC4

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,'(2I5)')NJSTEN,NKPOLY
C      IF(IPROC.EQ.LPROC)WRITE(6,'(4(1PE12.4),2I5)')
C     +                  XCENTR(ICELL),YCENTR(ICELL),ZCENTR(ICELL),
C     +                  CCSIZE,NJSTEN,NKPOLY

C     FOURTH ORDER

C     RUN THROUGH THE STENCIL
      DO JCSTEN = 1, NJSTEN

C       STENCIL CELL COORDINATES AND SIZE
        JSCELL = NSSTEN(JCSTEN,ICELL)
C        XSTENC = (XCENTR(JSCELL) - XCENTC)*OCSIZE
C        YSTENC = (YCENTR(JSCELL) - YCENTC)*OCSIZE
C        ZSTENC = (ZCENTR(JSCELL) - ZCENTC)*OCSIZE
C       RSC 21-APR-2018 MODIFIED FOR PERIODIC BOUNDARY CONDITIONS
        XSTENC = (XCENTR(JSCELL) - XCENTC)
        XSTENC = XSTENC - XGDLEN*NINT(XSTENC/XGDLEN)
        XSTENC = XSTENC*OCSIZE
        YSTENC = (YCENTR(JSCELL) - YCENTC)
        YSTENC = YSTENC - YGDLEN*NINT(YSTENC/YGDLEN)
        YSTENC = YSTENC*OCSIZE
        ZSTENC = (ZCENTR(JSCELL) - ZCENTC)
        ZSTENC = ZSTENC - ZGDLEN*NINT(ZSTENC/ZGDLEN)
        ZSTENC = ZSTENC*OCSIZE
        JSLEVL = NLEVEL(JSCELL)
        CSSIZE = HCSIZE(JSLEVL)*OCSIZE

C       STENCIL CELL SIZE TERMS
        HSTERM = TWLFTH*CSSIZE*CSSIZE
        HSXTRA = TWO*HSTERM
        HSDIFF = HSTERM - HCTERM

C       COEFFICIENT MATRIX ELEMENTS
        AMATRX(JCSTEN,1) = XSTENC
        STORE1 = XSTENC*XSTENC
        AMATRX(JCSTEN,2) = STORE1 + HSDIFF
        STORE1 = STORE1 + HSTERM
        AMATRX(JCSTEN,3) = (STORE1 + HSXTRA)*XSTENC

        AMATRX(JCSTEN,4) = YSTENC
        STORE4 = YSTENC*XSTENC
        AMATRX(JCSTEN,5) = STORE4
        STORE2 = YSTENC*YSTENC
        AMATRX(JCSTEN,6) = STORE2 + HSDIFF
        AMATRX(JCSTEN,7) = STORE1*YSTENC
        STORE2 = STORE2 + HSTERM
        AMATRX(JCSTEN,8) = STORE2*XSTENC
        AMATRX(JCSTEN,9) = (STORE2 + HSXTRA)*YSTENC

        AMATRX(JCSTEN,10) = ZSTENC
        AMATRX(JCSTEN,11) = ZSTENC*XSTENC
        AMATRX(JCSTEN,12) = ZSTENC*YSTENC
        STORE3 = ZSTENC*ZSTENC
        AMATRX(JCSTEN,13) = STORE3 + HSDIFF
        AMATRX(JCSTEN,14) = STORE1*ZSTENC
        AMATRX(JCSTEN,15) = STORE4*ZSTENC
        STORE3 = STORE3 + HSTERM
        AMATRX(JCSTEN,16) = STORE3*XSTENC
        AMATRX(JCSTEN,17) = STORE2*ZSTENC
        AMATRX(JCSTEN,18) = STORE3*YSTENC
        AMATRX(JCSTEN,19) = (STORE3 + HSXTRA)*ZSTENC

      ENDDO
C     RUN THROUGH THE STENCIL

C     COLUMN SCALING VECTOR
      DO JKPOLY = 1, NKPOLY
        DVECTR(JKPOLY) = ZERO
        DO JCSTEN = 1, NJSTEN
        DVECTR(JKPOLY) = MAX(DVECTR(JKPOLY),ABS(AMATRX(JCSTEN,JKPOLY)))
        ENDDO
        DVECTR(JKPOLY) = ONE/DVECTR(JKPOLY)
      ENDDO

CC     COLUMN SCALING
C      DO JKPOLY = 1, NKPOLY
C        DO JCSTEN = 1, NJSTEN
C          AMATRX(JCSTEN,JKPOLY) = AMATRX(JCSTEN,JKPOLY)*DVECTR(JKPOLY)
C        ENDDO
C      ENDDO

C     -------------------------------------------------------------------------

C     COMPUTE THE MOORE-PENROSE PSEUDO-INVERSE
C     ----------------------------------------

C     SINGULAR VALUE DECOMPOSITION
      CALL SVDCMP(AMATRX,NJSTEN,NKPOLY,NCSTMX,NPCFMX,WVECTR,VMATRX)

C     -------------------------------------------------------------------------

C     RECORD THE U MATRIX
      DO JKPOLY = 1, NKPOLY
        DO JCSTEN = 1, NJSTEN
          UMATRX(JCSTEN,JKPOLY) = AMATRX(JCSTEN,JKPOLY)
        ENDDO
      ENDDO

C     EDIT THE SINGULAR VALUES, GIVEN A TOLERANCE
C     ------------------------
      WVCMAX = WVECTR(1)
      DO JKPOLY = 2,NKPOLY
        WVCMAX = MAX(WVCMAX,WVECTR(JKPOLY))
      ENDDO

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WVCMIN = WVECTR(1)
C        JKPOLY = 1
C        WRITE(6,'(I5,1PE12.4)')JKPOLY,WVECTR(JKPOLY)
        DO JKPOLY = 2,NKPOLY
          WVCMIN = MIN(WVCMIN,WVECTR(JKPOLY))
C          WRITE(6,'(I5,1PE12.4)')JKPOLY,WVECTR(JKPOLY)
        ENDDO
C        WRITE(6,'(3(1PE12.4))')WVCMIN,WVCMAX,WVCMAX/(WVCMIN + TOLSVD)
        WRITE(6,'(2(1PE12.4))')WVCMIN,WVCMAX
      ENDIF

      THRESH = TOLSVD*WVCMAX
      DO JKPOLY = 1,NKPOLY
        IF(WVECTR(JKPOLY).LT.THRESH)WVECTR(JKPOLY) = ZERO
      ENDDO

C     -------------------------------------------------------------------------

C     EVALUATE THE POLYNOMIAL COEFFICIENTS
C     ------------------------------------

C     DENSITY
C     -------
      DFCENT = DSOR(ICELL)
      DO JCSTEN = 1, NJSTEN
        JSCELL = NSSTEN(JCSTEN,ICELL)
        BVECTR(JCSTEN) = DSOR(JSCELL) - DFCENT
      ENDDO

      CALL SVBKSB(AMATRX,WVECTR,VMATRX,NJSTEN,NKPOLY,NCSTMX,NPCFMX,
     +            BVECTR,ACOEFF)

      DO JKPOLY = 1, NKPOLY
C        ACOEFF(JKPOLY) = ACOEFF(JKPOLY)*DVECTR(JKPOLY)
        PCOEFD(JKPOLY) = ACOEFF(JKPOLY)
        PCODFD(JKPOLY) = ACOEFF(JKPOLY)
      ENDDO
      PCOEFD(0) = DFCENT - TWLFTH*(PCOEFD(2) + PCOEFD(6) + PCOEFD(13))
      PCODFD(0) = PCOEFD(0)

C     -------------------------------------------------------------------------

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
C        IF(JCLEVL.EQ.7)THEN
C          WRITE(6,'(9I5)')ICELL,JCLEVL,NKPOLY,NJSTEN,
C     +                   (NSSTEN(JSCELL,ICELL),JSCELL=1,NJSTEN)
C        ENDIF
C        IF((ICELL.EQ.1).OR.(ICELL.EQ.NCELL))THEN
C        IF(ICELL.EQ.33)THEN
C          IF(IPROC.EQ.LPROC)WRITE(6,*)'In CRESTN',ICELL
          WRITE(6,*)'In CRESTN',IPROC,ICELL,JCLEVL
          WRITE(6,'(30I5)')NKPOLY,NJSTEN,
     +                    (NSSTEN(JSCELL,ICELL),JSCELL=1,NJSTEN)
          JKPOLY = 0
          WRITE(6,'(I5,4(1PE15.7))')JKPOLY,DSOR(ICELL)
C          WRITE(6,'(I5,4(1PE15.7))')JKPOLY,DSOR(ICELL),
C     +                        USOR(1,ICELL),USOR(2,ICELL),USOR(3,ICELL)
C          DO JSCELL=1,NJSTEN
C            JKPOLY = NSSTEN(JSCELL,ICELL)
C            WRITE(6,'(I5,3(1PE15.7))')JKPOLY,DSOR(JKPOLY),
C     +                        USOR(1,JKPOLY),USOR(2,JKPOLY)
C          ENDDO
          WRITE(6,'(5(1PE15.7))')(PCOEFD(JKPOLY),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFT(JKPOLY),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,1),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,2),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,3),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFY(JKPOLY,1),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFY(JKPOLY,2),JKPOLY=0,NKPOLY)
C          WRITE(6,'(5(1PE15.7))')(PCOEFD(JKPOLY),JKPOLY=NKPOLY+1,NPCFMX)
C          WRITE(6,'(5(1PE15.7))')(PCOEFT(JKPOLY),JKPOLY=NKPOLY+1,NPCFMX)
C        WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,1),JKPOLY=NKPOLY+1,NPCFMX)
C        WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,2),JKPOLY=NKPOLY+1,NPCFMX)
C        WRITE(6,'(5(1PE15.7))')(PCOEFU(JKPOLY,3),JKPOLY=NKPOLY+1,NPCFMX)
C        WRITE(6,'(5(1PE15.7))')(PCOEFY(JKPOLY,1),JKPOLY=NKPOLY+1,NPCFMX)
C        WRITE(6,'(5(1PE15.7))')(PCOEFY(JKPOLY,2),JKPOLY=NKPOLY+1,NPCFMX)
C        ENDIF
      ENDIF

C     =========================================================================


      RETURN
      END
