      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOR SENGA
C     15-JAN-2011:  RSC UPDATED FOR HAMISH

C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SETS INITIAL THERMOCHEMICAL FIELD
C     1D LAMINAR FLAME PROFILE (LEFT OR RIGHT FACING)
C     SPECIAL FOR MULTI-STEP CHEMISTRY
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     ESTIMATED FLAME LOCATION AND THICKNESS
      DOUBLE PRECISION XLOCAT,XTHICK,YLOCAT,YTHICK,ZLOCAT,ZTHICK
      PARAMETER(XLOCAT = 5.0D-3, XTHICK = 1.0D-3)
      PARAMETER(YLOCAT = 5.0D-6, YTHICK = 1.0D-5)
      PARAMETER(ZLOCAT = 5.0D-6, ZTHICK = 1.0D-5)


C     FUNCTION
C     ========
      DOUBLE PRECISION ERFUNC
      EXTERNAL ERFUNC


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD,ARGMNT
      DOUBLE PRECISION CRIN(NSIZMX)
      DOUBLE PRECISION TRINR,TRINP
      DOUBLE PRECISION RCGMIX
      INTEGER MCRUN(NINDMX)
      INTEGER MLEVEL,MX,MY,MZ
      INTEGER JCELL,JSPEC



C     BEGIN
C     =====

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================


C     SET PRODUCT TEMPERATURE
C     -----------------------
C     REACTANT TEMPERATURE SET IN CONTROL FILE
      TRINR = TRIN
      TRINP = 1800.0D0

C     MIXTURE GAS CONSTANT
      RCGMIX = ZERO
      DO JSPEC = 1,NSPEC
        RCGMIX = RCGMIX + RGSPEC(JSPEC)*YRIN(JSPEC)
      ENDDO


C     SET TEMPERATURE AND DENSITY
C     ---------------------------
      DO JCELL = 1,NCELL

C       LOCAL CELL-CENTRE COORDINATES
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        MLEVEL = NLEVEL(JCELL)
        CALL MCCXYZ(MCRUN,MLEVEL,MX,MY,MZ)
        XCOORD = (REAL(MX) + HALF)*HCSIZE(MLEVEL)
        YCOORD = (REAL(MY) + HALF)*HCSIZE(MLEVEL)
        ZCOORD = (REAL(MZ) + HALF)*HCSIZE(MLEVEL)

        ARGMNT = (XCOORD-XLOCAT)/XTHICK

C       SIMPLE 1D GAUSSIAN TEMPERATURE PROFILE
        CRIN(JCELL) = EXP(-ARGMNT*ARGMNT)


C       SET TEMPERATURE
C       ---------------
        TRUN(JCELL) = TRINR +  CRIN(JCELL)*(TRINP - TRINR)


C       SET DENSITY, ASSUMING CONSTANT PRESSURE
C       -----------
        DRUN(JCELL) = PRIN/(RCGMIX*TRUN(JCELL))


      ENDDO

C     =========================================================================


      RETURN
      END
