      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOr SENGA
C     15-JAN-2011:  RSC UPDATED FOR HAMISH

C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SETS INITIAL THERMOCHEMICAL FIELD
C     1D LAMINAR FLAME PROFILE (LEFT OR RIGHT FACING)
C     SPECIAL FOR 1-STEP CHEMISTRY
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     ESTIMATED FLAME LOCATION AND THICKNESS
      DOUBLE PRECISION XLOCAT,XTHICK,YLOCAT,YTHICK,ZLOCAT,ZTHICK
      PARAMETER(XLOCAT = 2.0D-3, XTHICK = 1.0D-3)
      PARAMETER(YLOCAT = 5.0D-6, YTHICK = 1.0D-5)
      PARAMETER(ZLOCAT = 5.0D-6, ZTHICK = 1.0D-5)


C     FUNCTION
C     ========
      DOUBLE PRECISION ERFUNC
      EXTERNAL ERFUNC


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD,ARGMNT
      DOUBLE PRECISION CRIN(NSIZMX)
      DOUBLE PRECISION YRINR(NSPCMX),YRINP(NSPCMX)
      DOUBLE PRECISION TRINR,TRINP
      DOUBLE PRECISION RCGMIX,FLXMAS,FLXMOM
      INTEGER MCRUN(NINDMX)
      INTEGER MLEVEL,MX,MY,MZ
      INTEGER JCELL,JSPEC



C     BEGIN
C     =====

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================


C     SET PRODUCT TEMPERATURE
C     -----------------------
C     REACTANT TEMPERATURE SET IN CONTROL FILE
      TRINR = TRIN
      TRINP = 2300.0D0


C     SET SPECIES MASS FRACTIONS
C     --------------------------
C     OVERRIDE MASS FRACTION VALUES SET IN CONTROL FILE

C     REACTANTS
      YRINR(1) = ONE

      YRINR(NSPEC) = ZERO
      DO JSPEC = 1,NSPM1
        YRINR(NSPEC) = YRINR(NSPEC) + YRINR(JSPEC)
      ENDDO
      YRINR(NSPEC) = ONE - YRINR(NSPEC)

C     PRODUCTS
      YRINP(1) = ZERO

      YRINP(NSPEC) = ZERO
      DO JSPEC = 1,NSPM1
        YRINP(NSPEC) = YRINP(NSPEC) + YRINP(JSPEC)
      ENDDO
      YRINP(NSPEC) = ONE - YRINP(NSPEC)

C     WRITE TO REPORT FILE
      IF(IPROC.EQ.0)THEN

        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='OLD',FORM='FORMATTED')

C       GO TO EOF
1000    CONTINUE
          READ(NCREPT,9000,END=1010)
          GOTO 1000
1010    BACKSPACE(NCREPT)

        WRITE(NCREPT,*)'Scalar field initialisation report'
        WRITE(NCREPT,*)'----------------------------------'
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: reactant mass fractions:'
        DO JSPEC = 1,NSPEC
          WRITE(NCREPT,'(I5,1PE15.7)')JSPEC,YRINR(JSPEC)
        ENDDO
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: product mass fractions:'
        DO JSPEC = 1,NSPEC
          WRITE(NCREPT,'(I5,1PE15.7)')JSPEC,YRINP(JSPEC)
        ENDDO
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: reactant and product temperatures:'
        WRITE(NCREPT,'(2(1PE15.7))')TRINR,TRINP
        WRITE(NCREPT,*)

        WRITE(NCREPT,9900)CLDASH
        WRITE(NCREPT,*)

        CLOSE(NCREPT)

      ENDIF


C     SET REACTION PROGRESS VARIABLE
C     ------------------------------
      FLXMAS = DRIN*URIN
      FLXMOM = FLXMAS*URIN + PRIN
      DO JCELL = 1,NCELL

C       LOCAL CELL-CENTRE COORDINATES
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        MLEVEL = NLEVEL(JCELL)
        CALL MCCXYZ(MCRUN,MLEVEL,MX,MY,MZ)
        XCOORD = (REAL(MX) + HALF)*HCSIZE(MLEVEL)
        YCOORD = (REAL(MY) + HALF)*HCSIZE(MLEVEL)
        ZCOORD = (REAL(MZ) + HALF)*HCSIZE(MLEVEL)

        ARGMNT = (XCOORD-XLOCAT)/XTHICK

C       SIMPLE 1D LEFT-FACING ERROR FUNCTION PROFILE
        CRIN(JCELL) = HALF*(ONE+ERFUNC(ARGMNT))

CC       SIMPLE 1D RIGHT-FACING ERROR FUNCTION PROFILE
C        CRIN(IC) = HALF*(ONE+ERFUNC(-ARGMNT))


C       SET SPECIES MASS FRACTION
C       -------------------------
        YRUN(NSPEC,JCELL) = ZERO
        DO JSPEC = 1, NSPM1

          YRUN(JSPEC,JCELL) = YRINR(JSPEC)
     +                      + CRIN(JCELL)*(YRINP(JSPEC) - YRINR(JSPEC))

          YRUN(NSPEC,JCELL) = YRUN(NSPEC,JCELL) + YRUN(JSPEC,JCELL)

        ENDDO
        YRUN(NSPEC,JCELL) = ONE - YRUN(NSPEC,JCELL)


C       SET TEMPERATURE
C       ---------------
        TRUN(JCELL) = TRINR +  CRIN(JCELL)*(TRINP - TRINR)


C       SET DENSITY ASSUMING CONSTANT PRESSURE
C       -----------
C       PRESSURE SET IN CONTROL FILE
C       SET VELOCITY ASSUMING CONSTANT MASS FLUX
C       ------------
C       SET PRESSURE ASSUMING CONSTANT MOMENTUM
C       ------------
C       INITIAL (INLET) VELOCITY SET IN CONTROL FILE
        RCGMIX = ZERO
        DO JSPEC = 1,NSPEC
          RCGMIX = RCGMIX + RGSPEC(JSPEC)*YRUN(JSPEC,JCELL)
        ENDDO

        DRUN(JCELL) = PRIN/(RCGMIX*TRUN(JCELL))

        URUN(1,JCELL) = FLXMAS/DRUN(JCELL)

C        PRUN(JCELL) = FLXMOM - FLXMAS*URUN(1,JCELL)

      ENDDO

C     =========================================================================


      RETURN

9000  FORMAT(A)
9900  FORMAT(A79)

      END
