      SUBROUTINE ADAPTT
 
C     *************************************************************************
C
C     ADAPTT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     ADAPTS THE TIMESTEP FOR RK TIME STEPPING SCHEME
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION ERRMAX,TRATIO,TSTLOC
      DOUBLE PRECISION CKAPPA
      INTEGER JRYLOC(NSPCMX)
      INTEGER JRDLOC,JRULOC,JRVLOC,JRWLOC,JRELOC
      INTEGER JCELL,JSPEC

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In ADAPTT:'

C     =========================================================================

C     CHECK ADAPTION FLAG
C     -------------------
      IF(FLADPT)THEN

C       =======================================================================

C       ERK MAX ERROR EVALUATION
C       ------------------------

C       INITIALISE THE ERROR TOTALS AND LOCATIONS
        ERDTOT = ZERO
        JRDLOC = 0
        ERUTOT = ZERO
        JRULOC = 0
        ERVTOT = ZERO
        JRVLOC = 0
        ERWTOT = ZERO
        JRWLOC = 0
        ERETOT = ZERO
        JRELOC = 0
        DO JSPEC = 1,NSPEC
          ERYTOT(JSPEC) = ZERO
          JRYLOC(JSPEC) = 0
        ENDDO

C       LOOP OVER CELLS
        DO JCELL = 1, NCELL

C          ERRMAX = ABS(DERR(JCELL))
C          ERRMAX = ABS(DERR(JCELL))/(ABS(DRUN(JCELL))+ERDNRM)
          ERRMAX = ABS(DERR(JCELL)*ERDNRM)
          IF(ERRMAX.GT.ERDTOT)JRDLOC = JCELL
          ERDTOT = MAX(ERDTOT,ERRMAX)
          DERR(JCELL) = ZERO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'(I5,3(1PE15.7))')JCELL,UERR(1,JCELL),URUN(1,JCELL),
     +                                ERUTOT
          ENDIF

C          ERRMAX = ABS(UERR(1,JCELL))
C          ERRMAX = ABS(UERR(1,JCELL))/(ABS(URUN(1,JCELL))+ERUNRM)
          ERRMAX = ABS(UERR(1,JCELL)*ERUNRM)
          IF(ERRMAX.GT.ERUTOT)JRULOC = JCELL
          ERUTOT = MAX(ERUTOT,ERRMAX)
          UERR(1,JCELL) = ZERO

CC         DIAGNOSTICS
C          IF(IPROC.EQ.LPROC)THEN
C            WRITE(6,'(I5,3(1PE15.7))')JCELL,UERR(2,JCELL),URUN(2,JCELL),
C     +                                ERVTOT
C          ENDIF

C          ERRMAX = ABS(UERR(2,JCELL))
C          ERRMAX = ABS(UERR(2,JCELL))/(ABS(URUN(2,JCELL))+ERVNRM)
          ERRMAX = ABS(UERR(2,JCELL)*ERVNRM)
          IF(ERRMAX.GT.ERVTOT)JRVLOC = JCELL
          ERVTOT = MAX(ERVTOT,ERRMAX)
          UERR(2,JCELL) = ZERO

C          ERRMAX = ABS(UERR(3,JCELL))
C          ERRMAX = ABS(UERR(3,JCELL))/(ABS(URUN(3,JCELL))+ERWNRM)
          ERRMAX = ABS(UERR(3,JCELL)*ERWNRM)
          IF(ERRMAX.GT.ERWTOT)JRWLOC = JCELL
          ERWTOT = MAX(ERWTOT,ERRMAX)
          UERR(3,JCELL) = ZERO

C          ERRMAX = ABS(EERR(JCELL))
C          ERRMAX = ABS(EERR(JCELL))/(ABS(ERUN(JCELL))+ERENRM)
          ERRMAX = ABS(EERR(JCELL)*ERENRM)
          IF(ERRMAX.GT.ERETOT)JRELOC = JCELL
          ERETOT = MAX(ERETOT,ERRMAX)
          EERR(JCELL) = ZERO

          DO JSPEC = 1,NSPEC
C            ERRMAX = ABS(YERR(JSPEC,JCELL))
C            ERRMAX = ABS(YERR(JSPEC,JCELL))
C     +             /(ABS(YRUN(JSPEC,JCELL))+ERYNRM(JSPEC))
            ERRMAX = ABS(YERR(JSPEC,JCELL)*ERYNRM(JSPEC))
            IF(ERRMAX.GT.ERYTOT(JSPEC))JRYLOC(JSPEC) = JCELL
            ERYTOT(JSPEC) = MAX(ERYTOT(JSPEC),ERRMAX)
            YERR(JSPEC,JCELL) = ZERO
          ENDDO

        ENDDO
C       END OF LOOP OVER CELLS

C       =======================================================================

C       NORMALISE THE ERROR NORM TOTALS
C       -------------------------------
C        ERDTOT = ERDTOT*ERDNRM
C        ERUTOT = ERUTOT*ERUNRM
C        ERVTOT = ERVTOT*ERVNRM
C        ERWTOT = ERWTOT*ERWNRM
C        ERETOT = ERETOT*ERENRM
C        DO JSPEC = 1,NSPEC
C          ERYTOT(JSPEC) = ERYTOT(JSPEC)*ERYNRM(JSPEC)
C        ENDDO

C       =======================================================================

C       FIND THE MAXIMUM 
C       ----------------
        ERRMAX = ZERO
        JINDIC = 0
        IF(ERDTOT.GT.ERRMAX)THEN
          JINDIC = -4
          JERLOC = JRDLOC
        ENDIF
        ERRMAX = MAX(ERRMAX,ERDTOT)
        IF(ERUTOT.GT.ERRMAX)THEN
          JERLOC = JRULOC
          JINDIC = -1
        ENDIF
        ERRMAX = MAX(ERRMAX,ERUTOT)
        IF(ERVTOT.GT.ERRMAX)THEN
          JERLOC = JRVLOC
          JINDIC = -2
        ENDIF
        ERRMAX = MAX(ERRMAX,ERVTOT)
        IF(ERWTOT.GT.ERRMAX)THEN
          JERLOC = JRWLOC
          JINDIC = -3
        ENDIF
        ERRMAX = MAX(ERRMAX,ERWTOT)
        IF(ERETOT.GT.ERRMAX)THEN
          JERLOC = JRELOC
          JINDIC = -5
        ENDIF
        ERRMAX = MAX(ERRMAX,ERETOT)
        DO JSPEC = 1,NSPEC
          IF(ERYTOT(JSPEC).GT.ERRMAX)THEN
            JERLOC = JRYLOC(JSPEC)
            JINDIC = JSPEC
          ENDIF
          ERRMAX = MAX(ERRMAX,ERYTOT(JSPEC))
        ENDDO

C       =======================================================================

C       GLOBAL MAX ERROR
        TSTLOC = ERRMAX
        CALL P_GMAX(TSTLOC,ERRMAX)

C       =======================================================================

C       EVALUATE THE NEW TIME STEP
C       --------------------------
C       ZERO CHECK
        IF(ERRMAX.LT.ERRLOW)ERRMAX = ERRLOW

C       -----------------------------------------------------------------------

CC       I-CONTROLLER
C        TRATIO = CTMULT*EXP(CTALPH*LOG(ERRTOL/ERRMAX))

C       -----------------------------------------------------------------------

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'(6(1PE12.4))')ERRMAX,ERROLD,ERRTOL,
     +                           CTMULT,CTALPH,CTBETA
        ENDIF

C       PI-CONTROLLER
        TRATIO = CTMULT*EXP(CTALPH*LOG(ERRTOL/ERRMAX)
     +                     +CTBETA*LOG(ERROLD/ERRTOL))

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'(1PE12.4)')TRATIO
        ENDIF

C       -----------------------------------------------------------------------

C       ANTI-WINDUP
        CKAPPA = ONE
        TRATIO = ONE + CKAPPA*ATAN((TRATIO - ONE)/CKAPPA)

C       -----------------------------------------------------------------------

CC       PID-CONTROLLER
C        TRATIO = CTMULT*EXP(CTALPH*LOG(ERRTOL/ERRMAX)
C     +                     +CTBETA*LOG(ERROLD/ERRTOL)
C     +                     +CTGAMA*LOG(ERRTOL/ERRLDR))

C       -----------------------------------------------------------------------

C       ROLL OVER PREVIOUS MAX ERRORS
        ERRLDR = ERROLD
        ERROLD = ERRMAX

C       -----------------------------------------------------------------------

CC       LIMIT CHANGES TO TIME STEP
C        IF(TRATIO.GT.TRMAX)TRATIO = TRMAX
C        IF(TRATIO.LT.TRMIN)TRATIO = TRMIN

C       -----------------------------------------------------------------------

C       SET THE NEW TIME STEP
        TSTEP = TSTEP*TRATIO

C       LIMIT THE TIME STEP
        IF(TSTEP.GT.TSMAX)TSTEP = TSMAX
        IF(TSTEP.LT.TSMIN)TSTEP = TSMIN

C       =======================================================================

C       PARALLEL TRANSFER TO SET NEW GLOBAL TIME STEP
C       ---------------------------------------------
C       NEW TIME STEP IS THE GLOBAL MINIMUM OVER ALL PROCESSORS
C        TSTLOC = TSTEP
C        CALL P_GMIN(TSTLOC,TSTEP)

CC       DIAGNOSTICS
C        WRITE(6,'("ADAPTT",4I5,3(1PE12.4))')IPROC,ITIME,JERLOC,JINDIC,
C     +                                      ERRMAX,TSTLOC,TSTEP

C       =======================================================================

C       UPDATE THE TIME ADVANCEMENT COEFFICIENTS
C       ----------------------------------------
        DO IRKSTP = 1, NRKSTP
          RKLHS(IRKSTP) = RKLHC(IRKSTP)*TSTEP
          RKRHS(IRKSTP) = RKRHC(IRKSTP)*TSTEP
          RKERR(IRKSTP) = RKERC(IRKSTP)*TSTEP
          RKTIM(IRKSTP) = RKTMC(IRKSTP)*TSTEP
        ENDDO

C       =======================================================================

      ENDIF
C     ADAPTION FLAG

C     =========================================================================


      RETURN
      END
