      SUBROUTINE CELREP
 
C     *************************************************************************
C
C     CELREP
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     24-MAY-2009:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     REPORTS THE MESH DATA
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETER
C     =========
      DOUBLE PRECISION CELTOL
      PARAMETER(CELTOL = 1.0D-10)


C     LOCAL DATA
C     ==========
      INTEGER NCCRCV(NPROMX),NNCRCV(NPROMX),NNRRCV(NPROMX)
      INTEGER NCCTOT
      INTEGER JPROP1


C     BEGIN
C     =====

C     =========================================================================

C     REPORT THE MESH DATA
C     --------------------
      IF(IPROC.EQ.0)THEN

C       =======================================================================

C       OPEN THE REPORT FILE
        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='OLD',FORM='FORMATTED')

C       GO TO EOF
1000    CONTINUE
          READ(NCREPT,'(A)',END=1010)
          GOTO 1000
C       END OF LOOP 1000

1010    BACKSPACE(NCREPT)

C       =======================================================================

C       REPORT THE MESH DATA
        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Domain report'
        WRITE(NCREPT,*)'-------------'
        WRITE(NCREPT,*)

C       GLOBAL DOMAIN SIZE INFORMATION
        WRITE(NCREPT,*)'Global domain size:'
        WRITE(NCREPT,9010)XGDLEN,YGDLEN,ZGDLEN
        WRITE(NCREPT,9020)XGPHYR,YGPHYR,ZGPHYR
        IF((ABS(XGDLEN-XGPHYR).GT.CELTOL)
     +    .OR.(ABS(YGDLEN-YGPHYR).GT.CELTOL)
     +      .OR.(ABS(ZGDLEN-ZGPHYR).GT.CELTOL))THEN
          WRITE(NCREPT,*)'Warning: global domain size has been adjusted'
        ENDIF
        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Initial mesh:'
        WRITE(NCREPT,9110)NLVHWM
        WRITE(NCREPT,9120)HCSIZE(NLVHWM)
        WRITE(NCREPT,9130)MPMAX(1,NLVHWM)+1,
     +                    MPMAX(2,NLVHWM)+1,
     +                    MPMAX(3,NLVHWM)+1
        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'Base mesh:'
        WRITE(NCREPT,9210)NLEVBQ
        WRITE(NCREPT,9220)NBLEVL
        IF(NLEVBQ.NE.NBLEVL)THEN
          WRITE(NCREPT,*)'Warning: base level has been adjusted'
        ENDIF
        WRITE(NCREPT,9230)HCSIZE(NBLEVL)
        WRITE(NCREPT,9240)MPMAX(1,NBLEVL)+1,
     +                    MPMAX(2,NBLEVL)+1,
     +                    MPMAX(3,NBLEVL)+1
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Finest mesh:'
        WRITE(NCREPT,9310)NLEVL
        WRITE(NCREPT,9320)HCSIZE(NLEVL)
        WRITE(NCREPT,9330)MPMAX(1,NLEVL)+1,
     +                    MPMAX(2,NLEVL)+1,
     +                    MPMAX(3,NLEVL)+1
        WRITE(NCREPT,*)

      ENDIF

C     DOMAIN DECOMPOSITION INFORMATION
      CALL P_GTHI(NCELL, 1,NCCRCV,NPROMX)
      CALL P_GTHI(NRCELL,1,NNCRCV,NPROMX)
      CALL P_GTHI(NRRCEL,1,NNRRCV,NPROMX)

      IF(IPROC.EQ.0)THEN

        WRITE(NCREPT,*)'Initial domain decomposition:'
        WRITE(NCREPT,*)
        WRITE(NCREPT,*)
     +  'Processor  No of cells  Remote cells  Remote cells'
        WRITE(NCREPT,*)
     +  '    ID       in total     requested      provided'
        NCCTOT = 0
        DO JPROP1 = 1,NPROC
          WRITE(NCREPT,9410)JPROP1-1,NCCRCV(JPROP1),
     +                NNCRCV(JPROP1),NNRRCV(JPROP1)
          NCCTOT = NCCTOT + NCCRCV(JPROP1)
        ENDDO
        WRITE(NCREPT,9420)
        WRITE(NCREPT,9430)NCCTOT
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)
        WRITE(NCREPT,*)'End of domain report'
        WRITE(NCREPT,*)'--------------------'
        WRITE(NCREPT,*)

        WRITE(NCREPT,9900)CLDASH
        WRITE(NCREPT,*)

C       =======================================================================

C       CLOSE THE REPORT FILE
        CLOSE(NCREPT)

C       =======================================================================

      ENDIF

C     =========================================================================

C     FORMATS
9010  FORMAT('     x,y,x: required:',3(1PE12.4))
9020  FORMAT('     x,y,x:   actual:',3(1PE12.4))
9110  FORMAT('     level:',I4)
9120  FORMAT('     cell size:',1PE12.4)
9130  FORMAT('     global cell count: x,y,x:',3I9)
9210  FORMAT('     level: required:',I4)
9220  FORMAT('     level:   actual:',I4)
9230  FORMAT('     cell size:',1PE12.4)
9240  FORMAT('     global cell count: x,y,x:',3I9)
9310  FORMAT('     level:',I4)
9320  FORMAT('     cell size:',1PE12.4)
9330  FORMAT('     global cell count: x,y,x:',3I9)
9410  FORMAT(I7,7X,I8,5X,I8,5X,I8)
9420  FORMAT(14X,'--------')
9430  FORMAT(14X,I8)
9900  FORMAT(A79)

C     =========================================================================


      RETURN
      END
