      SUBROUTINE CEVALR(CXSGN,CYSGN,CZSGN,DEVAL,UEVAL,EEVAL,YEVAL)
 
C     *************************************************************************
C
C     CEVALR
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     19-SEP-2014:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POLYNOMIAL EVALUATION
C     FOR ALL QUANTITIES
C     FOR CELL REFINEMENT
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION CXSGN,CYSGN,CZSGN
      DOUBLE PRECISION DEVAL,EEVAL


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION PMONOM(NPCFMX)
      DOUBLE PRECISION STORE1,STORE2,STORE3,STORE4
      INTEGER NKPOLY,IPCOEF,IUCOMP,IYSPEC

C     DIAGNOSTICS
      INTEGER LPROC

C     DIAGNOSTICS
      LPROC = 20


C     BEGIN
C     =====

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In CEVALR',ICELL

C     =========================================================================

C     BUILD THE MONOMIALS
C     -------------------
      STORE1 = QRTR
      PMONOM(1) = CXSGN*STORE1
      PMONOM(2) = ZERO
      STORE2 = STORE1*STORE1
      STORE3 = HALF*STORE2
      PMONOM(3) = CXSGN*STORE3

      PMONOM(4) = CYSGN*STORE1
      PMONOM(5) = CXSGN*CYSGN*STORE2
      PMONOM(6) = ZERO
      STORE4 = THRD*STORE2
      PMONOM(7) = CYSGN*STORE4
      PMONOM(8) = CXSGN*STORE4
      PMONOM(9) = CYSGN*STORE3

      PMONOM(10) = CZSGN*STORE1
      PMONOM(11) = CXSGN*CZSGN*STORE2
      PMONOM(12) = CYSGN*CZSGN*STORE2
      PMONOM(13) = ZERO
      PMONOM(14) = CZSGN*STORE4
      PMONOM(15) = CXSGN*CYSGN*CZSGN*STORE1*STORE2
      PMONOM(16) = CXSGN*STORE4
      PMONOM(17) = CZSGN*STORE4
      PMONOM(18) = CYSGN*STORE4
      PMONOM(19) = CZSGN*STORE3

      NKPOLY = NPCAC4

C     =========================================================================

C     ASSEMBLE THE POLYNOMIAL
C     -----------------------

C     DENSITY
C     -------

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO IPCOEF = 0, NKPOLY
C          WRITE(6,'(I5,1PE12.4)')IPCOEF,PCOEFD(IPCOEF)
C        ENDDO
C      ENDIF


C     SUM ALL THE TERMS
      DEVAL = PCOEFD(0) + TWLFTH*(PCOEFD(2) + PCOEFD(6) + PCOEFD(13))
      DO IPCOEF = 1, NKPOLY

        DEVAL = DEVAL + PCOEFD(IPCOEF)*PMONOM(IPCOEF)

      ENDDO
C     SUM ALL THE TERMS

C     -------------------------------------------------------------------------

C     VELOCITY
C     --------
      DO IUCOMP = 1,NCOMP

C       SUM ALL THE TERMS
        UEVAL(IUCOMP) = PCOEFU(0,IUCOMP)
     +                + TWLFTH*(PCOEFU(2,IUCOMP)
     +                        + PCOEFU(6,IUCOMP)
     +                        + PCOEFU(13,IUCOMP))
C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'(I5,1PE15.7)')IPCOEF,UEVAL(IUCOMP)
        ENDIF

        DO IPCOEF = 1, NKPOLY

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
       WRITE(6,'(I5,3(1PE15.7))')IPCOEF,PCOEFU(IPCOEF,3),PMONOM(IPCOEF),
     +                  PCOEFU(IPCOEF,IUCOMP)*PMONOM(IPCOEF)
          ENDIF

          UEVAL(IUCOMP) = UEVAL(IUCOMP)
     +                  + PCOEFU(IPCOEF,IUCOMP)*PMONOM(IPCOEF)

        ENDDO
C       SUM ALL THE TERMS

      ENDDO

C     -------------------------------------------------------------------------

C     INTERNAL ENERGY
C     ---------------

C     SUM ALL THE TERMS
      EEVAL = PCOEFT(0) + TWLFTH*(PCOEFT(2) + PCOEFT(6) + PCOEFT(13))
      DO IPCOEF = 1, NKPOLY

        EEVAL = EEVAL + PCOEFT(IPCOEF)*PMONOM(IPCOEF)

      ENDDO
C     SUM ALL THE TERMS

C     -------------------------------------------------------------------------

C     SPECIES MASS FRACTION
C     ---------------------
      DO IYSPEC = 1,NSPEC

C       SUM ALL THE TERMS
        YEVAL(IYSPEC) = PCOEFY(0,IYSPEC)
     +                + TWLFTH*(PCOEFY(2,IYSPEC)
     +                        + PCOEFY(6,IYSPEC)
     +                        + PCOEFY(13,IYSPEC))
        DO IPCOEF = 1, NKPOLY

          YEVAL(IYSPEC) = YEVAL(IYSPEC)
     +                  + PCOEFY(IPCOEF,IYSPEC)*PMONOM(IPCOEF)

        ENDDO
C       SUM ALL THE TERMS

      ENDDO

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO IPCOEF = 0, NKPOLY
C          WRITE(6,'(I5,1PE12.4)')IPCOEF,PCOEFU(IPCOEF,3)
C        ENDDO
C      ENDIF

C     =========================================================================


      RETURN
      END
