      SUBROUTINE CONTAB
 
C     *************************************************************************
C
C     CONTAB
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     19-MAY-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES THE CELL CONNECTIVITY TABLES
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION OVTR3
      INTEGER JFACE,JCOMP,JOCQT


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE THE CELL CONNECTIVITY TABLES
C     ---------------------------------------

C     SET UP MATRIX OF INCREMENTS
      DO JFACE = 1,NFACE
        DO JCOMP = 1,NCOMP
          MATINC(JCOMP,JFACE) = 0
        ENDDO
      ENDDO
      MATINC(1,NFWEST) = -1
      MATINC(1,NFEAST) = 1
      MATINC(2,NFSUTH) = -1
      MATINC(2,NFNRTH) = 1
      MATINC(3,NFLOWR) = -1
      MATINC(3,NFUPPR) = 1

C     SET UP MAP OF OPPOSITE FACES
      MAPOPP(NFWEST) = NFEAST
      MAPOPP(NFEAST) = NFWEST
      MAPOPP(NFSUTH) = NFNRTH
      MAPOPP(NFNRTH) = NFSUTH
      MAPOPP(NFLOWR) = NFUPPR
      MAPOPP(NFUPPR) = NFLOWR

C     SET UP CELL FACE DIRECTION TABLE
      MDIRN(NFWEST) = 1
      MDIRN(NFEAST) = 1
      MDIRN(NFSUTH) = 2
      MDIRN(NFNRTH) = 2
      MDIRN(NFLOWR) = 3
      MDIRN(NFUPPR) = 3

C     SET UP DIRECTION FACE TABLE
      MFACE(1,1) = NFEAST
      MFACE(1,2) = NFWEST
      MFACE(2,1) = NFNRTH
      MFACE(2,2) = NFSUTH
      MFACE(3,1) = NFUPPR
      MFACE(3,2) = NFLOWR

C     SET UP CELL FACE HANDEDNESS TABLE
      FLHAND(NFWEST) = .TRUE.
      FLHAND(NFEAST) = .FALSE.
      FLHAND(NFSUTH) = .TRUE.
      FLHAND(NFNRTH) = .FALSE.
      FLHAND(NFLOWR) = .TRUE.
      FLHAND(NFUPPR) = .FALSE.

C     SET UP CELL FACE VISCOUS STRESS TENSOR INDEXING
      MNVIST(1,1) = 2
      MNVIST(2,1) = 3
      MNVIST(1,2) = 1
      MNVIST(2,2) = 3
      MNVIST(1,3) = 1
      MNVIST(2,3) = 2

C     SET UP QUARTET INTERNAL FACE TABLE
      MCFACE(1,1,NFWEST) = NFNRTH
      MCFACE(2,1,NFWEST) = NFUPPR
      MCFACE(1,2,NFWEST) = NFSUTH
      MCFACE(2,2,NFWEST) = NFUPPR
      MCFACE(1,3,NFWEST) = NFNRTH
      MCFACE(2,3,NFWEST) = NFLOWR
      MCFACE(1,4,NFWEST) = NFSUTH
      MCFACE(2,4,NFWEST) = NFLOWR

      MCFACE(1,1,NFEAST) = NFNRTH
      MCFACE(2,1,NFEAST) = NFUPPR
      MCFACE(1,2,NFEAST) = NFSUTH
      MCFACE(2,2,NFEAST) = NFUPPR
      MCFACE(1,3,NFEAST) = NFNRTH
      MCFACE(2,3,NFEAST) = NFLOWR
      MCFACE(1,4,NFEAST) = NFSUTH
      MCFACE(2,4,NFEAST) = NFLOWR

      MCFACE(1,1,NFSUTH) = NFEAST
      MCFACE(2,1,NFSUTH) = NFUPPR
      MCFACE(1,2,NFSUTH) = NFWEST
      MCFACE(2,2,NFSUTH) = NFUPPR
      MCFACE(1,3,NFSUTH) = NFEAST
      MCFACE(2,3,NFSUTH) = NFLOWR
      MCFACE(1,4,NFSUTH) = NFWEST
      MCFACE(2,4,NFSUTH) = NFLOWR

      MCFACE(1,1,NFNRTH) = NFEAST
      MCFACE(2,1,NFNRTH) = NFUPPR
      MCFACE(1,2,NFNRTH) = NFWEST
      MCFACE(2,2,NFNRTH) = NFUPPR
      MCFACE(1,3,NFNRTH) = NFEAST
      MCFACE(2,3,NFNRTH) = NFLOWR
      MCFACE(1,4,NFNRTH) = NFWEST
      MCFACE(2,4,NFNRTH) = NFLOWR

      MCFACE(1,1,NFLOWR) = NFEAST
      MCFACE(2,1,NFLOWR) = NFNRTH
      MCFACE(1,2,NFLOWR) = NFWEST
      MCFACE(2,2,NFLOWR) = NFNRTH
      MCFACE(1,3,NFLOWR) = NFEAST
      MCFACE(2,3,NFLOWR) = NFSUTH
      MCFACE(1,4,NFLOWR) = NFWEST
      MCFACE(2,4,NFLOWR) = NFSUTH

      MCFACE(1,1,NFUPPR) = NFEAST
      MCFACE(2,1,NFUPPR) = NFNRTH
      MCFACE(1,2,NFUPPR) = NFWEST
      MCFACE(2,2,NFUPPR) = NFNRTH
      MCFACE(1,3,NFUPPR) = NFEAST
      MCFACE(2,3,NFUPPR) = NFSUTH
      MCFACE(1,4,NFUPPR) = NFWEST
      MCFACE(2,4,NFUPPR) = NFSUTH

C     SET UP QUARTET INTERNAL NEIGHBOUR TABLE
      MCFCON(1,1) = 2
      MCFCON(2,1) = 3
      MCFCON(1,2) = 1
      MCFCON(2,2) = 4
      MCFCON(1,3) = 4
      MCFCON(2,3) = 1
      MCFCON(1,4) = 3
      MCFCON(2,4) = 2

C     SET UP FACE NEIGHBOUR QUARTET TABLE
      DO JOCQT = 0,7
        DO JFACE = 1,NFACE
          MCQTBL(JOCQT,JFACE) = -1
        ENDDO
      ENDDO
      MCQTBL(0,NFWEST) = 0
      MCQTBL(2,NFWEST) = 1
      MCQTBL(4,NFWEST) = 2
      MCQTBL(6,NFWEST) = 3

      MCQTBL(1,NFEAST) = 0
      MCQTBL(3,NFEAST) = 1
      MCQTBL(5,NFEAST) = 2
      MCQTBL(7,NFEAST) = 3

      MCQTBL(0,NFSUTH) = 0
      MCQTBL(1,NFSUTH) = 1
      MCQTBL(4,NFSUTH) = 2
      MCQTBL(5,NFSUTH) = 3

      MCQTBL(2,NFNRTH) = 0
      MCQTBL(3,NFNRTH) = 1
      MCQTBL(6,NFNRTH) = 2
      MCQTBL(7,NFNRTH) = 3

      MCQTBL(0,NFLOWR) = 0
      MCQTBL(1,NFLOWR) = 1
      MCQTBL(2,NFLOWR) = 2
      MCQTBL(3,NFLOWR) = 3

      MCQTBL(4,NFUPPR) = 0
      MCQTBL(5,NFUPPR) = 1
      MCQTBL(6,NFUPPR) = 2
      MCQTBL(7,NFUPPR) = 3

C     SET UP FACE NEIGHBOUR OCTANT TABLE
      MCFTBL(1,NFWEST) = 1
      MCFTBL(2,NFWEST) = 3
      MCFTBL(3,NFWEST) = 5
      MCFTBL(4,NFWEST) = 7

      MCFTBL(1,NFEAST) = 0
      MCFTBL(2,NFEAST) = 2
      MCFTBL(3,NFEAST) = 4
      MCFTBL(4,NFEAST) = 6

      MCFTBL(1,NFSUTH) = 2
      MCFTBL(2,NFSUTH) = 3
      MCFTBL(3,NFSUTH) = 6
      MCFTBL(4,NFSUTH) = 7

      MCFTBL(1,NFNRTH) = 0
      MCFTBL(2,NFNRTH) = 1
      MCFTBL(3,NFNRTH) = 4
      MCFTBL(4,NFNRTH) = 5

      MCFTBL(1,NFLOWR) = 4
      MCFTBL(2,NFLOWR) = 5
      MCFTBL(3,NFLOWR) = 6
      MCFTBL(4,NFLOWR) = 7

      MCFTBL(1,NFUPPR) = 0
      MCFTBL(2,NFUPPR) = 1
      MCFTBL(3,NFUPPR) = 2
      MCFTBL(4,NFUPPR) = 3

C     =========================================================================

C     FACE FLUX SIGN TABLE
      RSIGN(NFWEST) = ONE
      RSIGN(NFEAST) = -ONE
      RSIGN(NFSUTH) = ONE
      RSIGN(NFNRTH) = -ONE
      RSIGN(NFLOWR) = ONE
      RSIGN(NFUPPR) = -ONE

C     FACE FLUX INTEGER SIGN TABLE
      MSIGN(NFWEST) = 1
      MSIGN(NFEAST) = -1
      MSIGN(NFSUTH) = 1
      MSIGN(NFNRTH) = -1
      MSIGN(NFLOWR) = 1
      MSIGN(NFUPPR) = -1

C     FACE FLUX INTEGER SIGN COMPLEMENT TABLE
      MCSIGN(NFWEST) = -1
      MCSIGN(NFEAST) = 1
      MCSIGN(NFSUTH) = -1
      MCSIGN(NFNRTH) = 1
      MCSIGN(NFLOWR) = -1
      MCSIGN(NFUPPR) = 1

C     =========================================================================

C     KRONECKER DELTA
      DO JFACE = 1,3
        DO JCOMP = 1,3
          RDELTA(JCOMP,JFACE) = ZERO
        ENDDO
      ENDDO
      DO JCOMP = 1,3
        RDELTA(JCOMP,JCOMP) = ONE
      ENDDO

C     =========================================================================

C     CHILD CELL OFFSET SIGN TABLE
      OFFSGN(1,0) = -ONE
      OFFSGN(1,1) =  ONE
      OFFSGN(1,2) = -ONE
      OFFSGN(1,3) =  ONE
      OFFSGN(1,4) = -ONE
      OFFSGN(1,5) =  ONE
      OFFSGN(1,6) = -ONE
      OFFSGN(1,7) =  ONE

      OFFSGN(2,0) = -ONE
      OFFSGN(2,1) = -ONE
      OFFSGN(2,2) =  ONE
      OFFSGN(2,3) =  ONE
      OFFSGN(2,4) = -ONE
      OFFSGN(2,5) = -ONE
      OFFSGN(2,6) =  ONE
      OFFSGN(2,7) =  ONE

      OFFSGN(3,0) = -ONE
      OFFSGN(3,1) = -ONE
      OFFSGN(3,2) = -ONE
      OFFSGN(3,3) = -ONE
      OFFSGN(3,4) =  ONE
      OFFSGN(3,5) =  ONE
      OFFSGN(3,6) =  ONE
      OFFSGN(3,7) =  ONE

C     =========================================================================

C     CHILD CELL GRADIENT INDEXING
      MNGRAD(1,0) = -1
      MNGRAD(1,1) = 0
      MNGRAD(1,2) = -1
      MNGRAD(1,3) = 0
      MNGRAD(1,4) = -1
      MNGRAD(1,5) = 0
      MNGRAD(1,6) = -1
      MNGRAD(1,7) = 0

      MNGRAD(2,0) = -1
      MNGRAD(2,1) = -1
      MNGRAD(2,2) = 0
      MNGRAD(2,3) = 0
      MNGRAD(2,4) = -1
      MNGRAD(2,5) = -1
      MNGRAD(2,6) = 0
      MNGRAD(2,7) = 0

      MNGRAD(3,0) = -1
      MNGRAD(3,1) = -1
      MNGRAD(3,2) = -1
      MNGRAD(3,3) = -1
      MNGRAD(3,4) = 0
      MNGRAD(3,5) = 0
      MNGRAD(3,6) = 0
      MNGRAD(3,7) = 0

C     =========================================================================

C     NUMBERS AND SPATIAL LOCATIONS OF GAUSS POINTS
      OVTR3 = HALF/SQRT(THREE)

      NGPVOL = 8
      NGPFAC = 4

      GAUSVX(1) = -OVTR3
      GAUSVX(2) = OVTR3
      GAUSVX(3) = -OVTR3
      GAUSVX(4) = OVTR3
      GAUSVX(5) = -OVTR3
      GAUSVX(6) = OVTR3
      GAUSVX(7) = -OVTR3
      GAUSVX(8) = OVTR3
      GAUSVY(1) = -OVTR3
      GAUSVY(2) = -OVTR3
      GAUSVY(3) = OVTR3
      GAUSVY(4) = OVTR3
      GAUSVY(5) = -OVTR3
      GAUSVY(6) = -OVTR3
      GAUSVY(7) = OVTR3
      GAUSVY(8) = OVTR3
      GAUSVZ(1) = -OVTR3
      GAUSVZ(2) = -OVTR3
      GAUSVZ(3) = -OVTR3
      GAUSVZ(4) = -OVTR3
      GAUSVZ(5) = OVTR3
      GAUSVZ(6) = OVTR3
      GAUSVZ(7) = OVTR3
      GAUSVZ(8) = OVTR3

      GAUSFX(1,NFWEST) = -HALF
      GAUSFX(2,NFWEST) = -HALF
      GAUSFX(3,NFWEST) = -HALF
      GAUSFX(4,NFWEST) = -HALF
      GAUSFY(1,NFWEST) = -OVTR3
      GAUSFY(2,NFWEST) = OVTR3
      GAUSFY(3,NFWEST) = -OVTR3
      GAUSFY(4,NFWEST) = OVTR3
      GAUSFZ(1,NFWEST) = -OVTR3
      GAUSFZ(2,NFWEST) = -OVTR3
      GAUSFZ(3,NFWEST) = OVTR3
      GAUSFZ(4,NFWEST) = OVTR3

      GAUSFX(1,NFEAST) = HALF
      GAUSFX(2,NFEAST) = HALF
      GAUSFX(3,NFEAST) = HALF
      GAUSFX(4,NFEAST) = HALF
      GAUSFY(1,NFEAST) = -OVTR3
      GAUSFY(2,NFEAST) = OVTR3
      GAUSFY(3,NFEAST) = -OVTR3
      GAUSFY(4,NFEAST) = OVTR3
      GAUSFZ(1,NFEAST) = -OVTR3
      GAUSFZ(2,NFEAST) = -OVTR3
      GAUSFZ(3,NFEAST) = OVTR3
      GAUSFZ(4,NFEAST) = OVTR3

      GAUSFX(1,NFSUTH) = -OVTR3
      GAUSFX(2,NFSUTH) = OVTR3
      GAUSFX(3,NFSUTH) = -OVTR3
      GAUSFX(4,NFSUTH) = OVTR3
      GAUSFY(1,NFSUTH) = -HALF
      GAUSFY(2,NFSUTH) = -HALF
      GAUSFY(3,NFSUTH) = -HALF
      GAUSFY(4,NFSUTH) = -HALF
      GAUSFZ(1,NFSUTH) = -OVTR3
      GAUSFZ(2,NFSUTH) = -OVTR3
      GAUSFZ(3,NFSUTH) = OVTR3
      GAUSFZ(4,NFSUTH) = OVTR3

      GAUSFX(1,NFNRTH) = -OVTR3
      GAUSFX(2,NFNRTH) = OVTR3
      GAUSFX(3,NFNRTH) = -OVTR3
      GAUSFX(4,NFNRTH) = OVTR3
      GAUSFY(1,NFNRTH) = HALF
      GAUSFY(2,NFNRTH) = HALF
      GAUSFY(3,NFNRTH) = HALF
      GAUSFY(4,NFNRTH) = HALF
      GAUSFZ(1,NFNRTH) = -OVTR3
      GAUSFZ(2,NFNRTH) = -OVTR3
      GAUSFZ(3,NFNRTH) = OVTR3
      GAUSFZ(4,NFNRTH) = OVTR3

      GAUSFX(1,NFLOWR) = -OVTR3
      GAUSFX(2,NFLOWR) = OVTR3
      GAUSFX(3,NFLOWR) = -OVTR3
      GAUSFX(4,NFLOWR) = OVTR3
      GAUSFY(1,NFLOWR) = -OVTR3
      GAUSFY(2,NFLOWR) = -OVTR3
      GAUSFY(3,NFLOWR) = OVTR3
      GAUSFY(4,NFLOWR) = OVTR3
      GAUSFZ(1,NFLOWR) = -HALF
      GAUSFZ(2,NFLOWR) = -HALF
      GAUSFZ(3,NFLOWR) = -HALF
      GAUSFZ(4,NFLOWR) = -HALF

      GAUSFX(1,NFUPPR) = -OVTR3
      GAUSFX(2,NFUPPR) = OVTR3
      GAUSFX(3,NFUPPR) = -OVTR3
      GAUSFX(4,NFUPPR) = OVTR3
      GAUSFY(1,NFUPPR) = -OVTR3
      GAUSFY(2,NFUPPR) = -OVTR3
      GAUSFY(3,NFUPPR) = OVTR3
      GAUSFY(4,NFUPPR) = OVTR3
      GAUSFZ(1,NFUPPR) = HALF
      GAUSFZ(2,NFUPPR) = HALF
      GAUSFZ(3,NFUPPR) = HALF
      GAUSFZ(4,NFUPPR) = HALF

C     CONTRIBUTIONS TO VOLUME AND FLUX INTEGRALS
      GPFVOL = EIGHTH
      GPFLUX = QRTR

C     TWO-DIMENSIONAL DOMAIN
      IF(NDIMS.EQ.2)THEN

        NGPVOL = 4
        NGPFAC = 2

        GAUSVX(1) = -OVTR3
        GAUSVX(2) = OVTR3
        GAUSVX(3) = -OVTR3
        GAUSVX(4) = OVTR3
        GAUSVY(1) = -OVTR3
        GAUSVY(2) = -OVTR3
        GAUSVY(3) = OVTR3
        GAUSVY(4) = OVTR3
        GAUSVZ(1) = ZERO
        GAUSVZ(2) = ZERO
        GAUSVZ(3) = ZERO
        GAUSVZ(4) = ZERO

        GAUSFX(1,NFWEST) = -HALF
        GAUSFX(2,NFWEST) = -HALF
        GAUSFY(1,NFWEST) = -OVTR3
        GAUSFY(2,NFWEST) = OVTR3
        GAUSFZ(1,NFWEST) = ZERO
        GAUSFZ(2,NFWEST) = ZERO

        GAUSFX(1,NFEAST) = HALF
        GAUSFX(2,NFEAST) = HALF
        GAUSFY(1,NFEAST) = -OVTR3
        GAUSFY(2,NFEAST) = OVTR3
        GAUSFZ(1,NFEAST) = ZERO
        GAUSFZ(2,NFEAST) = ZERO

        GAUSFX(1,NFSUTH) = -OVTR3
        GAUSFX(2,NFSUTH) = OVTR3
        GAUSFY(1,NFSUTH) = -HALF
        GAUSFY(2,NFSUTH) = -HALF
        GAUSFZ(1,NFSUTH) = ZERO
        GAUSFZ(2,NFSUTH) = ZERO

        GAUSFX(1,NFNRTH) = -OVTR3
        GAUSFX(2,NFNRTH) = OVTR3
        GAUSFY(1,NFNRTH) = HALF
        GAUSFY(2,NFNRTH) = HALF
        GAUSFZ(1,NFNRTH) = ZERO
        GAUSFZ(2,NFNRTH) = ZERO

        GAUSFX(1,NFLOWR) = ZERO
        GAUSFX(2,NFLOWR) = ZERO
        GAUSFY(1,NFLOWR) = ZERO
        GAUSFY(2,NFLOWR) = ZERO
        GAUSFZ(1,NFLOWR) = ZERO
        GAUSFZ(2,NFLOWR) = ZERO

        GAUSFX(1,NFUPPR) = ZERO
        GAUSFX(2,NFUPPR) = ZERO
        GAUSFY(1,NFUPPR) = ZERO
        GAUSFY(2,NFUPPR) = ZERO
        GAUSFZ(1,NFUPPR) = ZERO
        GAUSFZ(2,NFUPPR) = ZERO

C       CONTRIBUTIONS TO VOLUME AND FLUX INTEGRALS
        GPFVOL = QRTR
        GPFLUX = HALF

      ENDIF

C     ONE-DIMENSIONAL DOMAIN
      IF(NDIMS.EQ.1)THEN

        NGPVOL = 2
        NGPFAC = 1

        GAUSVX(1) = -OVTR3
        GAUSVX(2) = OVTR3
        GAUSVY(1) = ZERO
        GAUSVY(2) = ZERO
        GAUSVZ(1) = ZERO
        GAUSVZ(2) = ZERO

        GAUSFX(1,NFWEST) = -HALF
        GAUSFY(1,NFWEST) = ZERO
        GAUSFZ(1,NFWEST) = ZERO

        GAUSFX(1,NFEAST) = HALF
        GAUSFY(1,NFEAST) = ZERO
        GAUSFZ(1,NFEAST) = ZERO

        GAUSFX(1,NFSUTH) = ZERO
        GAUSFY(1,NFSUTH) = ZERO
        GAUSFZ(1,NFSUTH) = ZERO

        GAUSFX(1,NFNRTH) = ZERO
        GAUSFY(1,NFNRTH) = ZERO
        GAUSFZ(1,NFNRTH) = ZERO

        GAUSFX(1,NFLOWR) = ZERO
        GAUSFY(1,NFLOWR) = ZERO
        GAUSFZ(1,NFLOWR) = ZERO

        GAUSFX(1,NFUPPR) = ZERO
        GAUSFY(1,NFUPPR) = ZERO
        GAUSFZ(1,NFUPPR) = ZERO

C       CONTRIBUTIONS TO VOLUME AND FLUX INTEGRALS
        GPFVOL = HALF
        GPFLUX = ONE

      ENDIF

C     =========================================================================

C     SPATIAL LOCATIONS OF SUBFACE CENTRES
      SUBFAX(1,NFWEST) = -QRTR
      SUBFAX(2,NFWEST) = -QRTR
      SUBFAX(3,NFWEST) = -QRTR
      SUBFAX(4,NFWEST) = -QRTR
      SUBFAY(1,NFWEST) = -QRTR
      SUBFAY(2,NFWEST) = QRTR
      SUBFAY(3,NFWEST) = -QRTR
      SUBFAY(4,NFWEST) = QRTR
      SUBFAZ(1,NFWEST) = -QRTR
      SUBFAZ(2,NFWEST) = -QRTR
      SUBFAZ(3,NFWEST) = QRTR
      SUBFAZ(4,NFWEST) = QRTR

      SUBFAX(1,NFEAST) = QRTR
      SUBFAX(2,NFEAST) = QRTR
      SUBFAX(3,NFEAST) = QRTR
      SUBFAX(4,NFEAST) = QRTR
      SUBFAY(1,NFEAST) = -QRTR
      SUBFAY(2,NFEAST) = QRTR
      SUBFAY(3,NFEAST) = -QRTR
      SUBFAY(4,NFEAST) = QRTR
      SUBFAZ(1,NFEAST) = -QRTR
      SUBFAZ(2,NFEAST) = -QRTR
      SUBFAZ(3,NFEAST) = QRTR
      SUBFAZ(4,NFEAST) = QRTR

      SUBFAX(1,NFSUTH) = -QRTR
      SUBFAX(2,NFSUTH) = QRTR
      SUBFAX(3,NFSUTH) = -QRTR
      SUBFAX(4,NFSUTH) = QRTR
      SUBFAY(1,NFSUTH) = -QRTR
      SUBFAY(2,NFSUTH) = -QRTR
      SUBFAY(3,NFSUTH) = -QRTR
      SUBFAY(4,NFSUTH) = -QRTR
      SUBFAZ(1,NFSUTH) = -QRTR
      SUBFAZ(2,NFSUTH) = -QRTR
      SUBFAZ(3,NFSUTH) = QRTR
      SUBFAZ(4,NFSUTH) = QRTR

      SUBFAX(1,NFNRTH) = -QRTR
      SUBFAX(2,NFNRTH) = QRTR
      SUBFAX(3,NFNRTH) = -QRTR
      SUBFAX(4,NFNRTH) = QRTR
      SUBFAY(1,NFNRTH) = QRTR
      SUBFAY(2,NFNRTH) = QRTR
      SUBFAY(3,NFNRTH) = QRTR
      SUBFAY(4,NFNRTH) = QRTR
      SUBFAZ(1,NFNRTH) = -QRTR
      SUBFAZ(2,NFNRTH) = -QRTR
      SUBFAZ(3,NFNRTH) = QRTR
      SUBFAZ(4,NFNRTH) = QRTR

      SUBFAX(1,NFLOWR) = -QRTR
      SUBFAX(2,NFLOWR) = QRTR
      SUBFAX(3,NFLOWR) = -QRTR
      SUBFAX(4,NFLOWR) = QRTR
      SUBFAY(1,NFLOWR) = -QRTR
      SUBFAY(2,NFLOWR) = -QRTR
      SUBFAY(3,NFLOWR) = QRTR
      SUBFAY(4,NFLOWR) = QRTR
      SUBFAZ(1,NFLOWR) = -QRTR
      SUBFAZ(2,NFLOWR) = -QRTR
      SUBFAZ(3,NFLOWR) = -QRTR
      SUBFAZ(4,NFLOWR) = -QRTR

      SUBFAX(1,NFUPPR) = -QRTR
      SUBFAX(2,NFUPPR) = QRTR
      SUBFAX(3,NFUPPR) = -QRTR
      SUBFAX(4,NFUPPR) = QRTR
      SUBFAY(1,NFUPPR) = -QRTR
      SUBFAY(2,NFUPPR) = -QRTR
      SUBFAY(3,NFUPPR) = QRTR
      SUBFAY(4,NFUPPR) = QRTR
      SUBFAZ(1,NFUPPR) = QRTR
      SUBFAZ(2,NFUPPR) = QRTR
      SUBFAZ(3,NFUPPR) = QRTR
      SUBFAZ(4,NFUPPR) = QRTR

C     TWO-DIMENSIONAL DOMAIN
      IF(NDIMS.EQ.2)THEN

        SUBFAX(1,NFWEST) = -QRTR
        SUBFAX(2,NFWEST) = -QRTR
        SUBFAY(1,NFWEST) = -QRTR
        SUBFAY(2,NFWEST) = QRTR
        SUBFAZ(1,NFWEST) = ZERO
        SUBFAZ(2,NFWEST) = ZERO

        SUBFAX(1,NFEAST) = QRTR
        SUBFAX(2,NFEAST) = QRTR
        SUBFAY(1,NFEAST) = -QRTR
        SUBFAY(2,NFEAST) = QRTR
        SUBFAZ(1,NFEAST) = ZERO
        SUBFAZ(2,NFEAST) = ZERO

        SUBFAX(1,NFSUTH) = -QRTR
        SUBFAX(2,NFSUTH) = QRTR
        SUBFAY(1,NFSUTH) = -QRTR
        SUBFAY(2,NFSUTH) = -QRTR
        SUBFAZ(1,NFSUTH) = ZERO
        SUBFAZ(2,NFSUTH) = ZERO

        SUBFAX(1,NFNRTH) = -QRTR
        SUBFAX(2,NFNRTH) = QRTR
        SUBFAY(1,NFNRTH) = QRTR
        SUBFAY(2,NFNRTH) = QRTR
        SUBFAZ(1,NFNRTH) = ZERO
        SUBFAZ(2,NFNRTH) = ZERO

        SUBFAX(1,NFLOWR) = ZERO
        SUBFAX(2,NFLOWR) = ZERO
        SUBFAY(1,NFLOWR) = ZERO
        SUBFAY(2,NFLOWR) = ZERO
        SUBFAZ(1,NFLOWR) = ZERO
        SUBFAZ(2,NFLOWR) = ZERO

        SUBFAX(1,NFUPPR) = ZERO
        SUBFAX(2,NFUPPR) = ZERO
        SUBFAY(1,NFUPPR) = ZERO
        SUBFAY(2,NFUPPR) = ZERO
        SUBFAZ(1,NFUPPR) = ZERO
        SUBFAZ(2,NFUPPR) = ZERO

      ENDIF

C     ONE-DIMENSIONAL DOMAIN
      IF(NDIMS.EQ.1)THEN

        SUBFAX(1,NFWEST) = -QRTR
        SUBFAY(1,NFWEST) = ZERO
        SUBFAZ(1,NFWEST) = ZERO

        SUBFAX(1,NFEAST) = QRTR
        SUBFAY(1,NFEAST) = ZERO
        SUBFAZ(1,NFEAST) = ZERO

        SUBFAX(1,NFSUTH) = ZERO
        SUBFAY(1,NFSUTH) = ZERO
        SUBFAZ(1,NFSUTH) = ZERO

        SUBFAX(1,NFNRTH) = ZERO
        SUBFAY(1,NFNRTH) = ZERO
        SUBFAZ(1,NFNRTH) = ZERO

        SUBFAX(1,NFLOWR) = ZERO
        SUBFAY(1,NFLOWR) = ZERO
        SUBFAZ(1,NFLOWR) = ZERO

        SUBFAX(1,NFUPPR) = ZERO
        SUBFAY(1,NFUPPR) = ZERO
        SUBFAZ(1,NFUPPR) = ZERO

      ENDIF

C     =========================================================================


      RETURN
      END
