      SUBROUTINE DIFCBC(NFSTAT)

C     *************************************************************************
C
C     DIFCBC
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     31-MAR-2011:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DATA FOR DIFFUSIVE INITIALISATION OF TURBULENCE
C     NON-CLEAN CELLS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER NFSTAT(NFACMX)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION UNEXT(NFACMX,NCMPMX),DUDF(NFACMX,NCMPMX)
      DOUBLE PRECISION UBCVAL(NCMPMX)
      DOUBLE PRECISION RSOVSZ,RSOVZ2,TRSOVZ
      DOUBLE PRECISION DECENT,DCURVX,DCURVY,DCURVZ
      INTEGER IFACE,ILEVL,INEXT,JFACE,KFACE
      INTEGER ICELL2,ICELL3,ICELL4
      INTEGER INEXT2,INEXT3,INEXT4
      INTEGER IFCODE,IQUART,JQUART

C     =========================================================================
C
C     BEGIN
C     =====

C     =========================================================================

C     LEVEL AND RECIPROCAL SIZE OF LOCAL CELL
      ILEVL  = NLEVEL(ICELL)

C     =========================================================================

C     COLLECT NEAREST-NEIGHBOUR VALUES AND CELL FACE DERIVATIVES
C     ----------------------------------------------------------

C     RUN THROUGH CELL FACES
      DO IFACE = 1,NFACE

C       -----------------------------------------------------------------------

C       NEXT-DOOR CELL INDEX
        INEXT = NXDOOR(IFACE,ICELL)

C       SIGN OF LOCAL CELL FACE
        RSOVSZ = RSIGN(IFACE)

C       CELL FACE STATUS
        IFCODE = NFSTAT(IFACE)

C       -----------------------------------------------------------------------

C       CHECK CELL FACE STATUS
        IF(IFCODE.EQ.NCLEAN)THEN

C         ---------------------------------------------------------------------

C         CLEAN FACE
C         ----------

C         NEIGHBOUR VALUES
          UNEXT(IFACE,1) = URUN(1,INEXT)
          UNEXT(IFACE,2) = URUN(2,INEXT)
          UNEXT(IFACE,3) = URUN(3,INEXT)

C         FIRST DERIVATIVES
          DUDF(IFACE,1) = (URUN(1,ICELL)-UNEXT(IFACE,1))*RSOVSZ
          DUDF(IFACE,2) = (URUN(2,ICELL)-UNEXT(IFACE,2))*RSOVSZ
          DUDF(IFACE,3) = (URUN(3,ICELL)-UNEXT(IFACE,3))*RSOVSZ

C         ---------------------------------------------------------------------

        ELSEIF(IFCODE.EQ.N2HTOH)THEN

C         ---------------------------------------------------------------------

C         2H TO H CELL SIZE TRANSITION
C         ----------------------------
C         CELL INDEX FOR LOWEST-INDEXED NEIGHBOUR CELL ALREADY OBTAINED
C         GET THE OTHER THREE USING THEIR OWN CONNECTIVITY
          JFACE = MCFACE(1,1,IFACE)
          INEXT2 = NXDOOR(JFACE,INEXT)
          JFACE = MCFACE(2,1,IFACE)
          INEXT3 = NXDOOR(JFACE,INEXT)
          JFACE = MCFACE(2,2,IFACE)
          INEXT4 = NXDOOR(JFACE,INEXT2)

C         NEIGHBOUR VALUES
          UNEXT(IFACE,1) = (URUN(1,INEXT)+URUN(1,INEXT2)
     +                    + URUN(1,INEXT3)+URUN(1,INEXT4))
          UNEXT(IFACE,2) = (URUN(2,INEXT)+URUN(2,INEXT2)
     +                    + URUN(2,INEXT3)+URUN(2,INEXT4))
          UNEXT(IFACE,3) = (URUN(3,INEXT)+URUN(3,INEXT2)
     +                    + URUN(3,INEXT3)+URUN(3,INEXT4))
C         FIRST DERIVATIVES
          DUDF(IFACE,1) =(FTHD*URUN(1,ICELL)-THRD*UNEXT(IFACE,1))*RSOVSZ
          DUDF(IFACE,2) =(FTHD*URUN(2,ICELL)-THRD*UNEXT(IFACE,2))*RSOVSZ
          DUDF(IFACE,3) =(FTHD*URUN(3,ICELL)-THRD*UNEXT(IFACE,3))*RSOVSZ

C         ---------------------------------------------------------------------

        ELSEIF(IFCODE.GE.NHTO2H)THEN

C         ---------------------------------------------------------------------

C         H TO 2H CELL SIZE TRANSITION
C         ----------------------------
C         GET QUARTET INDEX OF LOCAL CELL FROM THE CELL FACE CODE
          IQUART = IFCODE - NHTO2H + 1

C         GET THE OTHER THREE MEMBERS OF THE QUARTET
          KFACE = MAPOPP(IFACE)
          JFACE = MCFACE(1,IQUART,KFACE)
          ICELL2 = NXDOOR(JFACE,ICELL)
          JFACE = MCFACE(2,IQUART,KFACE)
          ICELL3 = NXDOOR(JFACE,ICELL)
          JQUART = MCFCON(1,IQUART)
          JFACE = MCFACE(2,JQUART,KFACE)
          ICELL4 = NXDOOR(JFACE,ICELL2)

C         NEIGHBOUR VALUES
          UNEXT(IFACE,1) = URUN(1,INEXT)
          UNEXT(IFACE,2) = URUN(2,INEXT)
          UNEXT(IFACE,3) = URUN(3,INEXT)

C         FIRST DERIVATIVES
          TRSOVZ = TTHD*RSOVSZ
          DECENT = URUN(1,ICELL)-URUN(1,ICELL4)
          DUDF(IFACE,1) = (URUN(1,ICELL)-UNEXT(IFACE,1)
     +                    - HALF*DECENT)*TRSOVZ
          DECENT = URUN(2,ICELL)-URUN(2,ICELL4)
          DUDF(IFACE,2) = (URUN(2,ICELL)-UNEXT(IFACE,2)
     +                    - HALF*DECENT)*TRSOVZ
          DECENT = URUN(3,ICELL)-URUN(3,ICELL4)
          DUDF(IFACE,3) = (URUN(3,ICELL)-UNEXT(IFACE,3)
     +                    - HALF*DECENT)*TRSOVZ

C         ---------------------------------------------------------------------

        ELSEIF(IFCODE.GE.NBCOND)THEN

C         ---------------------------------------------------------------------

C         BOUNDARY TREATMENT
C         ------------------
          IF(IFCODE.EQ.NSBCO1)THEN

C           BOUNDARY CONDITION: NON-REFLECTING OUTFLOW WITH PRESSURE TRACKING
            DUDF(IFACE,1) = ZERO
            DUDF(IFACE,2) = ZERO
            DUDF(IFACE,3) = ZERO

          ELSEIF(IFCODE.EQ.NSBCI1)THEN

C           BOUNDARY CONDITION: NON-REFLECTING INFLOW
            DUDF(IFACE,1) = ZERO
            DUDF(IFACE,2) = ZERO
            DUDF(IFACE,3) = ZERO

          ELSEIF(IFCODE.EQ.NSBCI2)THEN

C           BOUNDARY CONDITION: INFLOW WITH SPECIFIED VELOCITY AND TEMPERATURE
            CALL BCVELC(INEXT,ETIME,UBCVAL)
            RSOVZ2 = RSOVSZ*TWO
            DUDF(IFACE,1) = (URUN(1,ICELL)-UBCVAL(1))*RSOVZ2
            DUDF(IFACE,2) = (URUN(2,ICELL)-UBCVAL(2))*RSOVZ2
            DUDF(IFACE,3) = (URUN(3,ICELL)-UBCVAL(3))*RSOVZ2

          ELSEIF(IFCODE.EQ.NSBCI3)THEN

C           BOUNDARY CONDITION: INFLOW WITH SPECIFIED VELOCITY AND DENSITY
            CALL BCVELC(INEXT,ETIME,UBCVAL)
            RSOVZ2 = RSOVSZ*TWO
            DUDF(IFACE,1) = (URUN(1,ICELL)-UBCVAL(1))*RSOVZ2
            DUDF(IFACE,2) = (URUN(2,ICELL)-UBCVAL(2))*RSOVZ2
            DUDF(IFACE,3) = (URUN(3,ICELL)-UBCVAL(3))*RSOVZ2

          ELSEIF(IFCODE.EQ.NSBCW1)THEN

C           BOUNDARY CONDITION: ADIABATIC WALL
            DUDF(IFACE,1) = ZERO
            DUDF(IFACE,2) = ZERO
            DUDF(IFACE,3) = ZERO

          ELSEIF(IFCODE.EQ.NSBCW2)THEN

C           BOUNDARY CONDITION: ISOTHERMAL WALL
            DUDF(IFACE,1) = ZERO
            DUDF(IFACE,2) = ZERO
            DUDF(IFACE,3) = ZERO

          ELSE

C           ERROR CONDITION - invalid boundary condition code
            CALL ERHAND(15,32,4)

          ENDIF

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ERROR CONDITION - invalid cell face code
          CALL ERHAND(15,27,4)

        ENDIF
 
      ENDDO
C     RUN THROUGH FACES

C     =========================================================================

C     EVALUATE NORMALISED SECOND DERIVATIVES
C     --------------------------------------

C     VELOCITY
      DCURVX = DUDF(2,1) - DUDF(1,1)
      DCURVY = DUDF(4,1) - DUDF(3,1)
      DCURVZ = DUDF(6,1) - DUDF(5,1)
      URHS(1,ICELL) = DCURVX + DCURVY + DCURVZ

      DCURVX = DUDF(2,2) - DUDF(1,2)
      DCURVY = DUDF(4,2) - DUDF(3,2)
      DCURVZ = DUDF(6,2) - DUDF(5,2)
      URHS(2,ICELL) = DCURVX + DCURVY + DCURVZ

      DCURVX = DUDF(2,3) - DUDF(1,3)
      DCURVY = DUDF(4,3) - DUDF(3,3)
      DCURVZ = DUDF(6,3) - DUDF(5,3)
      URHS(3,ICELL) = DCURVX + DCURVY + DCURVZ

C     =========================================================================


      RETURN
      END
