      SUBROUTINE DIFCEL

C     *************************************************************************
C
C     DIFCEL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     31-MAR-2011:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DATA FOR DIFFUSIVE INITIALISATION OF TURBULENCE
C     FOR CLEAN CELLS ONLY
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION UNEXT(NFACMX,NCMPMX),DUDF(NFACMX,NCMPMX)
      DOUBLE PRECISION RSOVSZ
      DOUBLE PRECISION DCURVX,DCURVY,DCURVZ
      INTEGER IFACE,INEXT

C     =========================================================================
C
C     BEGIN
C     =====

C     =========================================================================

C     COLLECT NEAREST-NEIGHBOUR VALUES
C     --------------------------------

C     RUN THROUGH CELL FACES
      DO IFACE = 1,NFACE

C       NEXT-DOOR CELL INDEX
        INEXT = NXDOOR(IFACE,ICELL)

C       SIGN OF LOCAL CELL FACE
        RSOVSZ = RSIGN(IFACE)

C       VELOCITY
        UNEXT(IFACE,1) = URUN(1,INEXT)
        UNEXT(IFACE,2) = URUN(2,INEXT)
        UNEXT(IFACE,3) = URUN(3,INEXT)

C       FIRST DERIVATIVES
        DUDF(IFACE,1) = (URUN(1,ICELL)-UNEXT(IFACE,1))*RSOVSZ
        DUDF(IFACE,2) = (URUN(2,ICELL)-UNEXT(IFACE,2))*RSOVSZ
        DUDF(IFACE,3) = (URUN(3,ICELL)-UNEXT(IFACE,3))*RSOVSZ

      ENDDO
C     RUN THROUGH FACES

C     =========================================================================

C     EVALUATE SECOND DERIVATIVES
C     ---------------------------

C     VELOCITY
      DCURVX = DUDF(2,1) - DUDF(1,1)
      DCURVY = DUDF(4,1) - DUDF(3,1)
      DCURVZ = DUDF(6,1) - DUDF(5,1)
      URHS(1,ICELL) = DCURVX + DCURVY + DCURVZ

      DCURVX = DUDF(2,2) - DUDF(1,2)
      DCURVY = DUDF(4,2) - DUDF(3,2)
      DCURVZ = DUDF(6,2) - DUDF(5,2)
      URHS(2,ICELL) = DCURVX + DCURVY + DCURVZ

      DCURVX = DUDF(2,3) - DUDF(1,3)
      DCURVY = DUDF(4,3) - DUDF(3,3)
      DCURVZ = DUDF(6,3) - DUDF(5,3)
      URHS(3,ICELL) = DCURVX + DCURVY + DCURVZ

C     =========================================================================


      RETURN
      END
