      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOR SENGA
C     15-JAN-2011:  RSC UPDATED FOR HAMISH

C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SETS INITIAL THERMOCHEMICAL FIELD
C     1D LAMINAR FLAME PROFILE (LEFT OR RIGHT FACING)
C     SPECIAL FOR MULTI-STEP CHEMISTRY
C     SOLUTION READ FROM FILE
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     DOMAIN CENTRE
      DOUBLE PRECISION XDCENT,YDCENT,ZDCENT
      PARAMETER(XDCENT = 5.0D-3)
      PARAMETER(YDCENT = 5.0D-3)
      PARAMETER(ZDCENT = 5.0D-6)

C     SOLUTION OFFSET
      DOUBLE PRECISION ROFSET
      PARAMETER(ROFSET = 5.0D-4)

C     FLAME LEADING EDGE PARAMETER
      DOUBLE PRECISION YRUNLE
      PARAMETER(YRUNLE = 1.0D-4)

C     SIZE OF INPUT FLAME SOLUTION
      INTEGER NCLFMX
      PARAMETER(NCLFMX = 2048) 
C     FLAME INPUT FILE AND UNIT NUMBER
      CHARACTER*80 PNDMPF
      PARAMETER(PNDMPF = 'dmpi_flamin.dat')
      INTEGER NCDMPF
      PARAMETER(NCDMPF = 4)

      DOUBLE PRECISION SIXTH
      PARAMETER(SIXTH = 1.0D0/6.0D0)
      DOUBLE PRECISION RCETOL
      PARAMETER(RCETOL = 1.0D-9)


C     FUNCTION
C     ========
      DOUBLE PRECISION ERFUNC
      EXTERNAL ERFUNC


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YRUNF(NSPCMX,NCLFMX)
      DOUBLE PRECISION URUNF(NCMPMX,NCLFMX)
      DOUBLE PRECISION XCENTF(NCLFMX),YCENTF(NCLFMX),ZCENTF(NCLFMX)
      DOUBLE PRECISION SRUNF(NCLFMX),DRUNF(NCLFMX),ERUNF(NCLFMX)
      DOUBLE PRECISION HCSIZF(0:NLEVMX)
      DOUBLE PRECISION XGPHYF,YGPHYF,ZGPHYF
      DOUBLE PRECISION ETIMEF,TSTEPF,ERROLF,ERRLDF

      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL,TEVAL,UREVAL
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD
      DOUBLE PRECISION CCSIZF,OVCSZF
      DOUBLE PRECISION RREDUC,RREDM1,RREDP1,RREDM2
      DOUBLE PRECISION RCOEFF,RCOFM1,RCOFP1,RCOFP2
      DOUBLE PRECISION RCOORD,OVRCRD,RINTRP
      DOUBLE PRECISION FLXMAS,FLXVEL,RCOFLE,DRUNLE,URUNLE

      INTEGER NLEVEF(NCLFMX)
      INTEGER IRPROF,NRPROF
      INTEGER NBLEVF,NLEVLF,NCELLF,NSPECF,NCLFM1
      INTEGER INDX1D,INDXM1,INDXP1,INDXP2

      INTEGER JCELL,JSPEC
      CHARACTER*80 FNDMPF

CC     DIAGNOSTICS
C      INTEGER LPROC


C     BEGIN
C     =====

CC     DIAGNOSTICS
C      LPROC = 0

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In FLAMIN'

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================

C     READ THE 1D FLAME SOLUTION DUMP FILE
      FNDMPF = PNDMPF

C     SELECT DUMP FORMAT
      IF(NDIFMT.EQ.0)THEN

C       UNFORMATTED DUMP INPUT

C       READ THE DATA FROM DUMP INPUT FILE 1
C       NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
        OPEN(UNIT=NCDMPF,FILE=FNDMPF,STATUS='OLD',
     +       FORM='UNFORMATTED')
        REWIND(NCDMPF)

        READ(NCDMPF)IRPROF,NRPROF
        READ(NCDMPF)XGPHYF,YGPHYF,ZGPHYF
        READ(NCDMPF)HCSIZF(0),NBLEVF,NLEVLF,NCELLF,NSPECF
        READ(NCDMPF)ETIMEF,TSTEPF,ERROLF,ERRLDF
        READ(NCDMPF)(NLEVEF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(XCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(YCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(ZCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(SRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(DRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(ERUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(1,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(2,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF)(URUNF(3,JCELL),JCELL=1,NCELLF)
        DO JSPEC = 1,NSPECF
          READ(NCDMPF)(YRUNF(JSPEC,JCELL),JCELL=1,NCELLF)
        ENDDO

        CLOSE(NCDMPF)

      ELSE

C       FORMATTED DUMP INPUT

C       READ THE DATA FROM DUMP INPUT FILE 1
C       NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
        OPEN(UNIT=NCDMPF,FILE=FNDMPF,STATUS='OLD',
     +       FORM='FORMATTED')
        REWIND(NCDMPF)

        READ(NCDMPF,*)IRPROF,NRPROF
        READ(NCDMPF,*)XGPHYF,YGPHYF,ZGPHYF
        READ(NCDMPF,*)HCSIZF(0),NBLEVF,NLEVLF,NCELLF,NSPECF
        READ(NCDMPF,*)ETIMEF,TSTEPF,ERROLF,ERRLDF
        READ(NCDMPF,*)(NLEVEF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(XCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(YCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(ZCENTF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(SRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(DRUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(ERUNF(JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(1,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(2,JCELL),JCELL=1,NCELLF)
        READ(NCDMPF,*)(URUNF(3,JCELL),JCELL=1,NCELLF)
        DO JSPEC = 1,NSPEC
          READ(NCDMPF,*)(YRUNF(JSPEC,JCELL),JCELL=1,NCELLF)
        ENDDO

        CLOSE(NCDMPF)

      ENDIF

C     =========================================================================

C     USE 1D FLAME SOLUTION TO SET CELL VALUES

C     ASSUME UNIFORM MESH FOR FLAME SOLUTION
      CCSIZF = XCENTF(2) - XCENTF(1)
      OVCSZF = ONE/CCSIZF

C     FLAME LEADING EDGE LOCATION BASED ON PRODUCT MASS FRACTION
      JCELL = 0
1000  CONTINUE
        JCELL = JCELL + 1
        IF(YRUNF(2,JCELL).LT.YRUNLE)GOTO 1000
C     END OF LOOP 1000
      RCOFLE = XCENTF(JCELL)
      DRUNLE = DRUNF(JCELL)
      URUNLE = URUNF(1,JCELL)

C     ONE-DIMENSIONAL MASS FLOW
      FLXVEL = RCOFLE*URUNLE
      FLXMAS = FLXVEL*DRUNLE

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,'(5(1PE12.4))')
C     +                  CCSIZF,OVCSZF,RCOFLE,DRUNLE,URUNLE

C     WITH INTERPOLATION
      NCLFM1 = NCELLF - 1

C     RUN THROUGH THE DOMAIN
      DO JCELL =  1, NCELL

C       RADIAL COORDINATES
        XCOORD = XCENTR(JCELL) - XDCENT
        YCOORD = YCENTR(JCELL) - YDCENT
        ZCOORD = ZCENTR(JCELL) - ZDCENT
        RCOORD = XCOORD*XCOORD + YCOORD*YCOORD
        RCOORD = SQRT(RCOORD)

C       OFFSET INTO 1D FLAME SOLUTION
        RINTRP = RCOORD + ROFSET

C       INTERPOLATE THE 1D FLAME SOLUTION
        INDX1D = INT(RINTRP*OVCSZF)
        INDX1D = MAX(INDX1D,2)
        INDX1D = MIN(INDX1D,NCLFM1)

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(I5,4(1PE12.4))')JCELL,XCOORD,YCOORD,ZCOORD,RCOORD
C          WRITE(6,'(2I5,1PE12.4)')INDX1D,NCELLF,RINTRP
C        ENDIF

C       4TH ORDER INTERPOLATION
        INDXM1 = INDX1D - 1
        INDXP1 = INDX1D + 1
        INDXP2 = INDX1D + 2
        RREDUC = RINTRP - XCENTF(INDX1D)
        RREDP1 = RREDUC + CCSIZF
        RREDM1 = RREDUC - CCSIZF
        RREDM2 = RREDM1 - CCSIZF
        RREDUC = RREDUC*OVCSZF
        RREDP1 = RREDP1*OVCSZF
        RREDM1 = RREDM1*OVCSZF
        RREDM2 = RREDM2*OVCSZF
        RCOFM1 = -SIXTH*RREDUC*RREDM1*RREDM2
        RCOEFF = HALF*RREDP1*RREDM1*RREDM2
        RCOFP1 = -HALF*RREDP1*RREDUC*RREDM2
        RCOFP2 = SIXTH*RREDP1*RREDUC*RREDM1

        DEVAL = RCOFM1*DRUNF(INDXM1)
     +        + RCOEFF*DRUNF(INDX1D) 
     +        + RCOFP1*DRUNF(INDXP1) 
     +        + RCOFP2*DRUNF(INDXP2) 

C        UEVAL(1) = RCOFM1*URUNF(1,INDXM1)
C     +           + RCOEFF*URUNF(1,INDX1D) 
C     +           + RCOFP1*URUNF(1,INDXP1) 
C     +           + RCOFP2*URUNF(1,INDXP2) 
C
C        UEVAL(2) = RCOFM1*URUNF(2,INDXM1)
C     +           + RCOEFF*URUNF(2,INDX1D) 
C     +           + RCOFP1*URUNF(2,INDXP1) 
C     +           + RCOFP2*URUNF(2,INDXP2) 
C
C        UEVAL(3) = RCOFM1*URUNF(3,INDXM1)
C     +           + RCOEFF*URUNF(3,INDX1D) 
C     +           + RCOFP1*URUNF(3,INDXP1) 
C     +           + RCOFP2*URUNF(3,INDXP2) 

C       RESET VELOCITIES
        OVRCRD = ZERO
        IF(RCOORD.GT.RCETOL)OVRCRD = ONE/RCOORD
C        UREVAL = (FLXMAS/DEVAL - FLXVEL)*OVRCRD
        UREVAL = FLXVEL*OVRCRD*(DRUNLE/DEVAL - ONE)
        UEVAL(1) = UREVAL*XCOORD*OVRCRD
        UEVAL(2) = UREVAL*YCOORD*OVRCRD
        UEVAL(3) = ZERO

        EEVAL = RCOFM1*ERUNF(INDXM1)
     +        + RCOEFF*ERUNF(INDX1D) 
     +        + RCOFP1*ERUNF(INDXP1) 
     +        + RCOFP2*ERUNF(INDXP2) 

        DO JSPEC = 1,NSPEC
          YEVAL(JSPEC) = RCOFM1*YRUNF(JSPEC,INDXM1)
     +                 + RCOEFF*YRUNF(JSPEC,INDX1D) 
     +                 + RCOFP1*YRUNF(JSPEC,INDXP1) 
     +                 + RCOFP2*YRUNF(JSPEC,INDXP2) 
        ENDDO

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(4(1PE12.4))')DEVAL,UEVAL(1),UEVAL(2),UEVAL(3)
C          WRITE(6,'(4(1PE12.4))')UREVAL,EEVAL,YEVAL(1),YEVAL(2)
C        ENDIF

C       SET TEMPERATURE
        ICELL = JCELL
        TRUN(ICELL) = TRIN
        CALL TEMPER(DEVAL,UEVAL,EEVAL,YEVAL,TEVAL)

C       RESET VELOCITIES
        OVRCRD = ZERO
        IF(RCOORD.GT.ZERO)OVRCRD = ONE/RCOORD
        UREVAL = (FLXMAS/DEVAL - FLXVEL)*OVRCRD
        UEVAL(1) = UREVAL*XCOORD*OVRCRD
        UEVAL(2) = UREVAL*YCOORD*OVRCRD
        UEVAL(3) = ZERO

C       SET VALUES
        DRUN(JCELL) = DEVAL
        URUN(1,JCELL) = UEVAL(1)
        URUN(2,JCELL) = UEVAL(2)
        URUN(3,JCELL) = UEVAL(3)
        TRUN(JCELL) = TEVAL
        DO JSPEC = 1,NSPEC
          YRUN(JSPEC,JCELL)  = YEVAL(JSPEC)
        ENDDO

      ENDDO

C     =========================================================================


      RETURN
      END
