      SUBROUTINE FLAMIN 
 
C     *************************************************************************
C
C     FLAMIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     28-DEC-2003:  CREATED FOR SENGA
C     15-JAN-2011:  RSC UPDATED FOR HAMISH

C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SETS INITIAL THERMOCHEMICAL FIELD
C     1D LAMINAR FLAME PROFILE (LEFT OR RIGHT FACING)
C     SPECIAL FOR MULTI-STEP CHEMISTRY
C     
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     FLAME LOCATION, CENTRE AND THICKNESS
      DOUBLE PRECISION XCENTF,YCENTF,ZCENTF
      DOUBLE PRECISION RLOCAT,RTHICK
      PARAMETER(XCENTF = 5.0D-3)
      PARAMETER(YCENTF = 5.0D-3)
      PARAMETER(ZCENTF = 5.0D-6)
C      PARAMETER(RLOCAT = 2.5D-3, RTHICK = 5.0D-4)
C      PARAMETER(RLOCAT = 3.75D-3, RTHICK = 5.0D-4)
      PARAMETER(RLOCAT = 4.0D-3, RTHICK = 2.5D-4)

C     FLAME MASS FLOW PARAMETERS
      DOUBLE PRECISION UINFLE,CINFLE
C      DOUBLE PRECISION RINFLE,UINFLE
C      PARAMETER(RINFLE = 2.0D-3)
C      PARAMETER(RINFLE = 3.25D-3)
C      PARAMETER(RINFLE = 3.75D-3)
      PARAMETER(UINFLE = 3.9D-1)
      PARAMETER(CINFLE = 1.0D-2)


C     FUNCTIONS
C     =========
      DOUBLE PRECISION ERFUNC,ERFINV
      EXTERNAL ERFUNC,ERFINV


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION XCOORD,YCOORD,ZCOORD,ARGMNT
      DOUBLE PRECISION RCOORD,OVRCRD
      DOUBLE PRECISION CRIN(NSIZMX)
      DOUBLE PRECISION YRINR(NSPCMX),YRINP(NSPCMX)
      DOUBLE PRECISION TRINR,TRINP
      DOUBLE PRECISION RCGMIX,FLXMAS
      DOUBLE PRECISION RINFLE,DINFLE,URADIL,COSANG,SINANG
      INTEGER MCRUN(NINDMX)
      INTEGER MLEVEL,MX,MY,MZ
      INTEGER JCELL,JSPEC



C     BEGIN
C     =====

C     =========================================================================

C     SPECIFY INITIAL THERMOCHEMICAL FIELD HERE
C     =========================================

CC     DIAGNOSTICS
C      WRITE(6,*)'In FLAMIN'

C     SET PRODUCT TEMPERATURE
C     -----------------------
C     REACTANT TEMPERATURE SET IN CONTROL FILE
      TRINR = TRIN
      TRINP = 2300.0D0
C      TRINP = 301.0D0
C      TRINP = 1800.0D0


C     SET SPECIES MASS FRACTIONS
C     --------------------------
C     OVERRIDE MASS FRACTION VALUES SET IN CONTROL FILE

C     REACTANTS
      DO JSPEC = 1,NSPM1
        YRINR(JSPEC) = ZERO
      ENDDO
      YRINR(1) = YRIN(1)
      YRINR(2) = YRIN(2)

      YRINR(NSPEC) = ZERO
      DO JSPEC = 1,NSPM1
        YRINR(NSPEC) = YRINR(NSPEC) + YRINR(JSPEC)
      ENDDO
      YRINR(NSPEC) = ONE - YRINR(NSPEC)

C     PRODUCTS
      DO JSPEC = 1,NSPM1
        YRINP(JSPEC) = ZERO
      ENDDO
C      YRINP(3) = 2.54717D-1
C      YRINP(3) = 1.51376D-1
C      YRINP(4) = 1.23853D-1

      YRINP(NSPEC) = ZERO
      DO JSPEC = 1,NSPM1
        YRINP(NSPEC) = YRINP(NSPEC) + YRINP(JSPEC)
      ENDDO
      YRINP(NSPEC) = ONE - YRINP(NSPEC)

C     WRITE TO REPORT FILE
      IF(IPROC.EQ.0)THEN

        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='OLD',FORM='FORMATTED')

C       GO TO EOF
1000    CONTINUE
          READ(NCREPT,9000,END=1010)
          GOTO 1000
1010    BACKSPACE(NCREPT)

        WRITE(NCREPT,*)'Scalar field initialisation report'
        WRITE(NCREPT,*)'----------------------------------'
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: reactant mass fractions:'
        DO JSPEC = 1,NSPEC
          WRITE(NCREPT,'(I5,1PE15.7)')JSPEC,YRINR(JSPEC)
        ENDDO
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: product mass fractions:'
        DO JSPEC = 1,NSPEC
          WRITE(NCREPT,'(I5,1PE15.7)')JSPEC,YRINP(JSPEC)
        ENDDO
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'FLAMIN: reactant and product temperatures:'
        WRITE(NCREPT,'(2(1PE15.7))')TRINR,TRINP
        WRITE(NCREPT,*)

        WRITE(NCREPT,9900)CLDASH
        WRITE(NCREPT,*)

        CLOSE(NCREPT)

      ENDIF


C     SET REACTION PROGRESS VARIABLE
C     ------------------------------
C     INITIAL RADIAL MASS FLUX
C     EQUAL TO UNBURNED DENSITY TIMES LAMINAR BURNING VELOCITY
C              TIMES FLAME LEADING EDGE RADIUS
      ARGMNT = TWO*CINFLE - ONE
      RINFLE = RLOCAT + RTHICK*ERFINV(ARGMNT)
      DINFLE = DRIN
      FLXMAS = RINFLE*DINFLE*UINFLE

CC     DIAGNOSTICS
C      WRITE(6,'(3(1PE12.4))')RINFLE,DINFLE,UINFLE

      DO JCELL = 1,NCELL

C       LOCAL CELL-CENTRE COORDINATES
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JCELL)
        MLEVEL = NLEVEL(JCELL)
        CALL MCCXYZ(MCRUN,MLEVEL,MX,MY,MZ)
        XCOORD = (REAL(MX) + HALF)*HCSIZE(MLEVEL)
        YCOORD = (REAL(MY) + HALF)*HCSIZE(MLEVEL)
        ZCOORD = (REAL(MZ) + HALF)*HCSIZE(MLEVEL)

        XCOORD = XCOORD - XCENTF
        YCOORD = YCOORD - YCENTF
        ZCOORD = ZCOORD - ZCENTF
        RCOORD = XCOORD*XCOORD + YCOORD*YCOORD
        RCOORD = SQRT(RCOORD)

        ARGMNT = (RCOORD-RLOCAT)/RTHICK

C       SIMPLE 1D LEFT-FACING ERROR FUNCTION PROFILE
        CRIN(JCELL) = HALF*(ONE+ERFUNC(ARGMNT))


C       SET REACTANT SPECIES MASS FRACTIONS
C       -----------------------------------
        YRUN(NSPEC,JCELL) = ZERO
        DO JSPEC = 1, NSPM1

          YRUN(JSPEC,JCELL) = YRINR(JSPEC)
     +                      + CRIN(JCELL)*(YRINP(JSPEC) - YRINR(JSPEC))

          YRUN(NSPEC,JCELL) = YRUN(NSPEC,JCELL) + YRUN(JSPEC,JCELL)

        ENDDO
        YRUN(NSPEC,JCELL) = ONE - YRUN(NSPEC,JCELL)


C       SET TEMPERATURE
C       ---------------
        TRUN(JCELL) = TRINR +  CRIN(JCELL)*(TRINP - TRINR)


C       SET DENSITY ASSUMING CONSTANT PRESSURE
C       -----------
C       PRESSURE SET IN CONTROL FILE
C       SET VELOCITY ASSUMING CONSTANT MASS FLUX
C       ------------
        RCGMIX = ZERO
        DO JSPEC = 1,NSPEC
          RCGMIX = RCGMIX + RGSPEC(JSPEC)*YRUN(JSPEC,JCELL)
        ENDDO

        DRUN(JCELL) = PRIN/(RCGMIX*TRUN(JCELL))

        OVRCRD = ZERO
        IF(RCOORD.GT.ZERO)OVRCRD = ONE/RCOORD
        URADIL = OVRCRD*FLXMAS/DRUN(JCELL)
        URADIL = URADIL - UINFLE*RINFLE*OVRCRD
        COSANG = XCOORD*OVRCRD
        SINANG = YCOORD*OVRCRD
        URUN(1,JCELL) = URADIL*COSANG
        URUN(2,JCELL) = URADIL*SINANG

CC       DIAGNOSTICS
C        IF(RCOORD.LE.1.0D-4)THEN
C          WRITE(6,'(I5,3(1PE12.4))')JCELL,XCOORD,YCOORD,RCOORD
C          WRITE(6,'(3(1PE12.4))')URADIL,COSANG,SINANG
C          WRITE(6,'(2(1PE12.4))')DRUN(JCELL),TRUN(JCELL)
C        ENDIF

      ENDDO

C     =========================================================================


      RETURN

9000  FORMAT(A)
9900  FORMAT(A79)

      END
