      SUBROUTINE INDATA
 
C     *************************************************************************
C
C     INDATA
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT
C
C     CHANGE RECORD
C     -------------
C     16-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES ALL DATA
C
C     FILES: CHEMICAL DATA FILE - UNIT NCCHEM - FORMATTED INPUT
C            REPORT FILE        - UNIT NCREPT - FORMATTED OUTPUT
C            STATISTICS FILE    - UNIT NCSTAT - FORMATTED OUTPUT
C            DUMP OUTPUT FILES  - UNIT NCDMPO - UNFORMATTED      
C            DUMP INPUT FILES   - UNIT NCDMPI - UNFORMATTED  (RESTART ONLY)     
C        
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RGCMIX,FORNOW
      INTEGER JCELL,JSPEC,ITINT,ICP
      INTEGER IRPROC,NRPROC
      LOGICAL FXDUMP
      
C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 1

C     ==========================================================================

C     SET VALUE OF MATHEMATICAL CONSTANTS
C     -----------------------------------
      PI = FOUR*ATAN(ONE)
      ROOTPI = SQRT(PI)
      TWOPI = TWO*PI
      RLNTWO = LOG(TWO)
      CLNTEN = LOG(TEN)

C     =========================================================================

C     GET AND REPORT THE CONTROL DATA
C     ===============================
      CALL CONTIN
      CALL CONREP

C     =========================================================================

C     GET AND REPORT THE CHEMICAL DATA
C     ================================
      CALL CHEMIN
      CALL CHMREP

C     =========================================================================

C     INITIALISE ONLINE STATISTICS
C     ============================

C     STATISTICS ON ONE PROCESSOR ONLY
      IF(IPROC.EQ.0)THEN

C       INITIALISE THE STATISTICS FILE
        OPEN(UNIT=NCSTAT,FILE=FNSTAT,STATUS='NEW',FORM='FORMATTED')
        ENDFILE(NCSTAT)
        CLOSE(NCSTAT)

      ENDIF

C     ZERO THE STATISTICS STORAGE COUNTER
      ITSTAT = 0

C     =========================================================================

C     CHECK AND INITIALISE DUMP FILES
C     -------------------------------
      INQUIRE(FILE=FNDMPO(1),EXIST=FXDUMP)
      IF(.NOT.FXDUMP)THEN
        IF(NDOFMT.EQ.0)THEN
          OPEN(UNIT=NCDMPO,FILE=FNDMPO(1),STATUS='NEW',
     +         FORM='UNFORMATTED')
        ELSE
          OPEN(UNIT=NCDMPO,FILE=FNDMPO(1),STATUS='NEW',
     +         FORM='FORMATTED')
        ENDIF
        CLOSE(NCDMPO)
      ENDIF
      INQUIRE(FILE=FNDMPO(2),EXIST=FXDUMP)
      IF(.NOT.FXDUMP)THEN
        IF(NDOFMT.EQ.0)THEN
          OPEN(UNIT=NCDMPO,FILE=FNDMPO(2),STATUS='NEW',
     +         FORM='UNFORMATTED')
        ELSE
          OPEN(UNIT=NCDMPO,FILE=FNDMPO(2),STATUS='NEW',
     +         FORM='FORMATTED')
        ENDIF
        CLOSE(NCDMPO)
      ENDIF

C     ==========================================================================

C     TIME STEPPING
C     -------------
      ETIME = ZERO
      NTIME2 = NTIME1+NTIME-1
      ITIME = 0

C     =========================================================================

C     INITIAL DENSITY
C     ---------------
      RGCMIX = ZERO
      DO JSPEC = 1,NSPEC
        RGCMIX = RGCMIX + RGSPEC(JSPEC)*YRIN(JSPEC)
      ENDDO
      RGCMIX = RGCMIX*TRIN
      DRIN = PRIN/RGCMIX


C     INITIAL INTERNAL ENERGY
C     -----------------------
      ERIN = ZERO
      ITINT = 1
      DO JSPEC = 1,NSPEC

        FORNOW = AMASCH(NCPOLY(ITINT,JSPEC),ITINT,JSPEC)
        DO ICP = NCPOM1(ITINT,JSPEC),1,-1
          FORNOW = FORNOW*TRIN + AMASCH(ICP,ITINT,JSPEC)
        ENDDO
        FORNOW = AMASCH(NCENTH(ITINT,JSPEC),ITINT,JSPEC) + FORNOW*TRIN
        ERIN = ERIN + FORNOW*YRIN(JSPEC)

      ENDDO
      FORNOW = URIN*URIN + VRIN*VRIN + WRIN*WRIN
      ERIN = ERIN - RGCMIX + HALF*FORNOW

C     =========================================================================

C     EXECUTE REQUIRED STARTUP
C     ========================

C     COLD START SWITCH
C     -----------------
      IF(MOD(ICFLAG,2).EQ.0)THEN

C       =======================================================================
C       COLD START
C       =======================================================================

C       INITIAL DOMAIN DECOMPOSITION
C       ============================
        CALL PARDOM

C       =======================================================================

C       CELL CONNECTIVITY TABLES
        CALL CONTAB

C       =======================================================================

C       INITIAL MESH GENERATOR
C       ----------------------
        IF(INMESH.EQ.1)THEN
      
C         GENERATE AN INITIAL REFINED MESH
          CALL ADPTIN

        ENDIF

C       =======================================================================

C       CELL CONNECTIVITY
        CALL STINIT
        CALL STNCON
        CALL CELREP

C       =======================================================================

C       INITIALISE BOUNDARIES
C       =====================
        CALL BCINIT

C       =======================================================================

C       INITIALISE THE S-SCALAR FIELD
C       =============================
        DO JCELL = 1,NCELL

          SRUN(JCELL) = SRIN

        ENDDO

C       =======================================================================

C       INITIALISE THE VELOCITY FIELD
C       =============================

C       PRE-INITIALISE THE VELOCITY FIELD TO ZERO
C       ---------------------------------
        DO JCELL = 1,NCELL 

          URUN(1,JCELL) = ZERO
          URUN(2,JCELL) = ZERO
          URUN(3,JCELL) = ZERO

        ENDDO

C       -----------------------------------------------------------------------

C       INITIAL TURBULENCE GENERATOR
C       ----------------------------
        UPRIME(1) = ZERO
        UPRIME(2) = ZERO
        UPRIME(3) = ZERO

C       GENERATE FRESH INITIAL TURBULENCE
        IF(INTURB.EQ.1)CALL TURBIN

C       -----------------------------------------------------------------------

C       ADD ON THE DEFAULT MEAN VELOCITY
C       --------------------------------
        DO JCELL = 1,NCELL

          URUN(1,JCELL) = URIN + URUN(1,JCELL)
          URUN(2,JCELL) = VRIN + URUN(2,JCELL)
          URUN(3,JCELL) = WRIN + URUN(3,JCELL)

        ENDDO

C       =======================================================================

C       INITIALISE THE SCALAR FIELD
C       ===========================

C       DENSITY, TEMPERATURE, PRESSURE, SPECIES MASS FRACTIONS
C       ------------------------------------------------------
        DO JCELL = 1,NCELL

          DRUN(JCELL) = DRIN
          PRUN(JCELL) = PRIN
          TRUN(JCELL) = TRIN

          DO JSPEC = 1,NSPEC
            YRUN(JSPEC,JCELL) = YRIN(JSPEC)
          ENDDO

        ENDDO

C       -----------------------------------------------------------------------

C       INITIAL FLAME GENERATOR
C       -----------------------
        IF(INFLAM.EQ.1)THEN
      
C         GENERATE AN INITIAL THERMOCHEMICAL FIELD
C         PRESSURE, TEMPERATURE AND MASS FRACTIONS
C         MODIFY VELOCITIES AS REQUIRED
          CALL FLAMIN

        ENDIF

C       -----------------------------------------------------------------------

C       INTERNAL ENERGY
C       ---------------
        DO JCELL = 1,NCELL

C         INTERNAL ENERGY
          ERUN(JCELL) = ZERO

C         RUN THROUGH ALL SPECIES
          RGCMIX = ZERO
          ITINT = 1
          DO JSPEC = 1,NSPEC

C           USE THERMOCHEMICAL DATA
            FORNOW = AMASCH(NCPOLY(ITINT,JSPEC),ITINT,JSPEC)
            DO ICP = NCPOM1(ITINT,JSPEC),1,-1
              FORNOW = FORNOW*TRUN(JCELL) + AMASCH(ICP,ITINT,JSPEC)
            ENDDO
            FORNOW = AMASCH(NCENTH(ITINT,JSPEC),ITINT,JSPEC)
     +             + FORNOW*TRUN(JCELL)

            ERUN(JCELL) = ERUN(JCELL) + FORNOW*YRUN(JSPEC,JCELL)

            RGCMIX = RGCMIX + RGSPEC(JSPEC)*YRUN(JSPEC,JCELL)

          ENDDO

C         FINALISE INTERNAL ENERGY
          FORNOW = HALF*(URUN(1,JCELL)*URUN(1,JCELL)
     +           +       URUN(2,JCELL)*URUN(2,JCELL)
     +           +       URUN(3,JCELL)*URUN(3,JCELL))

          ERUN(JCELL) = ERUN(JCELL) - RGCMIX*TRUN(JCELL) + FORNOW

        ENDDO

C       -----------------------------------------------------------------------

C       CONVERT VARIABLES TO CONSERVATIVE FORM
C       --------------------------------------
C       AND INITIALISE TIME-STEPPING RHS TERMS
C       AND INITIALISE TIME-STEPPING SAVED-SOLUTION TERMS
C       AND INITIALISE PRIMITIVE VARIABLES
        DO JCELL = 1,NCELL

          USOR(1,JCELL) = URUN(1,JCELL)
          USOR(2,JCELL) = URUN(2,JCELL)
          USOR(3,JCELL) = URUN(3,JCELL)

          URUN(1,JCELL) = DRUN(JCELL)*URUN(1,JCELL)
          URUN(2,JCELL) = DRUN(JCELL)*URUN(2,JCELL)
          URUN(3,JCELL) = DRUN(JCELL)*URUN(3,JCELL)
          ERUN(JCELL) = DRUN(JCELL)*ERUN(JCELL)

          DRHS(JCELL) = DRUN(JCELL)
          URHS(1,JCELL) = URUN(1,JCELL)
          URHS(2,JCELL) = URUN(2,JCELL)
          URHS(3,JCELL) = URUN(3,JCELL)
          ERHS(JCELL) = ERUN(JCELL)

          DSAV(JCELL) = DRUN(JCELL)
          USAV(1,JCELL) = URUN(1,JCELL)
          USAV(2,JCELL) = URUN(2,JCELL)
          USAV(3,JCELL) = URUN(3,JCELL)
          ESAV(JCELL) = ERUN(JCELL)

          DO JSPEC = 1,NSPEC
            YSOR(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
            YRUN(JSPEC,JCELL) = DRUN(JCELL)*YRUN(JSPEC,JCELL)
            YRHS(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
            YSAV(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
          ENDDO

        ENDDO

C       =======================================================================

C       INITIAL PARALLEL DATA TRANSFER
        CALL PARFER

C       =======================================================================
C       END OF COLD  START
C       =======================================================================

      ELSEIF(ICFLAG.EQ.1)THEN

C       =======================================================================
C       WARM START
C       =======================================================================

C       -----------------------------------------------------------------------
C       THIS BLOCK MAY BE MODIFIED AS REQUIRED
C       TO BLEND INITIAL VELOCITY AND SCALAR FIELDS
C       WITH PREVIOUSLY DUMPED DATA
C       -----------------------------------------------------------------------

C       RESTART FROM FULL DUMP FILES
C       ----------------------------

C       SELECT DUMP FORMAT
        IF(NDIFMT.EQ.0)THEN

C         UNFORMATTED DUMP INPUT

C         READ THE DATA FROM DUMP INPUT FILE 1
C         NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
          OPEN(UNIT=NCDMPI,FILE=FNDMPO(1),STATUS='OLD',
     +         FORM='UNFORMATTED')
          REWIND(NCDMPI)

          READ(NCDMPI)IRPROC,NRPROC
          READ(NCDMPI)XGPHYR,YGPHYR,ZGPHYR
          READ(NCDMPI)HCSIZE(0),NBLEVL,NLEVL,NCELL,NSPEC
          READ(NCDMPI)ETIME,TSTEP,ERROLD,ERRLDR
          READ(NCDMPI)(NLEVEL(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(XCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(YCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(ZCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(SRUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(DRUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(ERUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(URUN(1,JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(URUN(2,JCELL),JCELL=1,NCELL)
          READ(NCDMPI)(URUN(3,JCELL),JCELL=1,NCELL)
          DO JSPEC = 1,NSPEC
            READ(NCDMPI)(YRUN(JSPEC,JCELL),JCELL=1,NCELL)
          ENDDO

          CLOSE(NCDMPI)

        ELSE

C         FORMATTED DUMP INPUT

C         READ THE DATA FROM DUMP INPUT FILE 1
C         NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
          OPEN(UNIT=NCDMPI,FILE=FNDMPO(1),STATUS='OLD',
     +         FORM='FORMATTED')
          REWIND(NCDMPI)

          READ(NCDMPI,*)IRPROC,NRPROC
          READ(NCDMPI,*)XGPHYR,YGPHYR,ZGPHYR
          READ(NCDMPI,*)HCSIZE(0),NBLEVL,NLEVL,NCELL,NSPEC
          READ(NCDMPI,*)ETIME,TSTEP,ERROLD,ERRLDR
          READ(NCDMPI,*)(NLEVEL(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(XCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(YCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(ZCENTR(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(SRUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(DRUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(ERUN(JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(URUN(1,JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(URUN(2,JCELL),JCELL=1,NCELL)
          READ(NCDMPI,*)(URUN(3,JCELL),JCELL=1,NCELL)
          DO JSPEC = 1,NSPEC
            READ(NCDMPI,*)(YRUN(JSPEC,JCELL),JCELL=1,NCELL)
          ENDDO

          CLOSE(NCDMPI)

        ENDIF

C       -----------------------------------------------------------------------

C       INITIALISE TIME-STEPPING RHS TERMS
C       ----------------------------------
        DO JCELL = 1,NCELL

          DRHS(JCELL) = DRUN(JCELL)
          URHS(1,JCELL) = URUN(1,JCELL)
          URHS(2,JCELL) = URUN(2,JCELL)
          URHS(3,JCELL) = URUN(3,JCELL)
          ERHS(JCELL) = ERUN(JCELL)

          DO JSPEC = 1,NSPEC
            YRHS(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
          ENDDO

        ENDDO

C       -----------------------------------------------------------------------

C       INITIALISE TIME-STEPPING SAVED-SOLUTION TERMS
C       ---------------------------------------------
        DO JCELL = 1,NCELL

          DSAV(JCELL) = DRUN(JCELL)
          USAV(1,JCELL) = URUN(1,JCELL)
          USAV(2,JCELL) = URUN(2,JCELL)
          USAV(3,JCELL) = URUN(3,JCELL)
          ESAV(JCELL) = ERUN(JCELL)

          DO JSPEC = 1,NSPEC
            YSAV(JSPEC,JCELL) = YRUN(JSPEC,JCELL)
          ENDDO

        ENDDO

C       -----------------------------------------------------------------------

C       INITIALISE PRIMITIVE VARIABLES
C       ------------------------------
        DO JCELL = 1,NCELL

          DSOR(JCELL) = DRUN(JCELL)
          FORNOW = ONE/DRUN(JCELL)
          USOR(1,JCELL) = URUN(1,JCELL)*FORNOW
          USOR(2,JCELL) = URUN(2,JCELL)*FORNOW
          USOR(3,JCELL) = URUN(3,JCELL)*FORNOW

          DO JSPEC = 1,NSPEC
            YSOR(JSPEC,JCELL) = YRUN(JSPEC,JCELL)*FORNOW
          ENDDO

        ENDDO

C       -----------------------------------------------------------------------

C       INITIALISE THE GLOBAL DOMAIN
        CALL GLODOM

C       REBUILD THE OCTREE
        CALL RETREE

C       CONNECT THE CELLS
        CALL RELEAF

C       CELL CONNECTIVITY TABLES
        CALL CONTAB

C       CELL CONNECTIVITY
        CALL STINIT
        CALL STNCON
        CALL CELREP

C       INITIALISE BOUNDARIES
        CALL BCINIT

C       =======================================================================
C       END OF WARM START
C       =======================================================================

      ENDIF

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        DO JCELL = 1,NCELL
C          WRITE(6,'(I5,5(1PE15.7))')JCELL,DRUN(JCELL),
C     +                                    USOR(1,JCELL),
C     +                                    USOR(2,JCELL),
C     +                                    USOR(3,JCELL),
C     +                                    TRUN(JCELL)
C        ENDDO
C      ENDIF

C     ==========================================================================

C     INITIAL GUESS FOR TEMPERATURE
C     =============================
      CALL TEMPIN

C     ==========================================================================

C     INITIALISE TIME STEPPING
C     ========================
      CALL DTINIT

C     ==========================================================================

C     INITIALISE SPATIAL ADAPTION
C     ===========================
      CALL DFINIT

C     ==========================================================================


      RETURN
      END
