      SUBROUTINE MCXYZC(MCODE,NLEVEL,MX,MY,MZ)
 
C     *************************************************************************
C
C     MCXYZC
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     25-JUN-2008:  CREATED
C     08-MAY-2011:  REFORMULATED FOR IMPROVED PERFORMANCE
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     MORTON CODE MANIPULATION
C     CONVERTS XYZ COORDINATES INTO A MORTON CODE AT THE SPECIFIED LEVEL
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_mcmlib.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER MCODE(MCSIZE)
      INTEGER NLEVEL,MX,MY,MZ


C     LOCAL DATA
C     ==========
      INTEGER MLOCL(MCBITS)
      INTEGER NPBITS(MCSIZE)
      INTEGER MXCRUN,MYCRUN,MZCRUN
      INTEGER NLEVM1,MCSLVL,MCSLM1
      INTEGER ICSIZE,ICBITS,IPBITS


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE
C     ----------
      NLEVM1 = NLEVEL - 1
      MCSLM1 = NLEVM1/NLOCTS
      DO ICSIZE = 1, MCSLM1
        NPBITS(ICSIZE) = NLBITS
      ENDDO
      MCSLVL = MCSLM1 + 1
      NPBITS(MCSLVL) = (NLEVEL - MCSLM1*NLOCTS)*3

C     =========================================================================

C     INCORPORATE XYZ INTEGERS INTO BIT STRING
C     ----------------------------------------
      ICBITS = NLEVEL*3
      MZCRUN = MZ
      MLOCL(ICBITS) = MOD(MZCRUN,2)
      ICBITS = ICBITS - 1
      MYCRUN = MY
      MLOCL(ICBITS) = MOD(MYCRUN,2)
      ICBITS = ICBITS - 1
      MXCRUN = MX
      MLOCL(ICBITS) = MOD(MXCRUN,2)
      DO IPBITS = 2, NLEVEL
        ICBITS = ICBITS - 1
        MZCRUN = MZCRUN/2
        MLOCL(ICBITS) = MOD(MZCRUN,2)
        ICBITS = ICBITS - 1
        MYCRUN = MYCRUN/2
        MLOCL(ICBITS) = MOD(MYCRUN,2)
        ICBITS = ICBITS - 1
        MXCRUN = MXCRUN/2
        MLOCL(ICBITS) = MOD(MXCRUN,2)
      ENDDO

C     =========================================================================

C     CONVERT BIT STRING TO MORTON CODE
C     ---------------------------------
C     INITIALISE
      DO ICSIZE = 1, MCSIZE
        MCODE(ICSIZE) = 0
      ENDDO

C     UP TO REQUIRED LEVEL ONLY
      ICBITS = 0
      DO ICSIZE = 1, MCSLVL
        DO IPBITS = 0, NPBITS(ICSIZE)-1
          ICBITS = ICBITS + 1
          MCODE(ICSIZE) = MCODE(ICSIZE) + MLOCL(ICBITS)*KCPWR2(IPBITS)
        ENDDO
      ENDDO

C     =========================================================================

C     SET THE LEVEL CODE
      MCODE(MCSIZE) = MCODE(MCSIZE) + NLEVEL*KCPWR2(NLVBLO)

C     =========================================================================


      RETURN
      END
