      SUBROUTINE PARBIN
 
C     *************************************************************************
C
C     PARBIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     CARRIES OUT PARALLEL LOAD-BALANCING
C     DURING INITIALISATION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      INTEGER MCMSIZ,MPARAY,MDARAY
      PARAMETER(MCMSIZ = 1000, MPARAY = 2000, MDARAY = 10000)
      INTEGER MPSNGL,NOTAG
      PARAMETER(MPSNGL = 1,NOTAG = 0)
      DOUBLE PRECISION GIMTOL,GIRTOL
      PARAMETER(GIMTOL = 1.0D-2, GIRTOL = 1.0D-6)
      INTEGER NGIMAX
      PARAMETER(NGIMAX = 1000)


C     FUNCTION
C     ========
      DOUBLE PRECISION RANUNP
      EXTERNAL RANUNP


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION GIMBAL,GIMBAV,GIMMAX,GIMOLD
      INTEGER MCMLO(NINDMX,MCMSIZ),MCMHI(NINDMX,MCMSIZ)
      INTEGER NCELLO,NCELHI,NDIFLO,NDIFHI,NMOVLO,NMOVHI
      INTEGER NRCVLO,NRCVHI,NSNDLO,NSNDHI
      INTEGER IDEST
      INTEGER ISEED
      INTEGER ICOUNT,NSTART,NNCOP1,NEND


C     BEGIN
C     =====

C     =========================================================================

C     TAKE ACTION ONLY IF THERE IS MORE THAN ONE PROCESSOR
      IF(NPROC.GT.1)THEN

C       =======================================================================

C       INITIALISE RANDOM NUMBER GENERATOR
        ISEED = IPROC
        CALL RANINP(ISEED)

C       =======================================================================

        GIMOLD = ZERO

        ICOUNT = 1
1000    CONTINUE

C         UPDATE NEAREST-NEIGHBOUR INFORMATION
C         ------------------------------------
          NCELLO = 0
          NCELHI = 0
          NDIFLO = 0
          NDIFHI = 0

          IF(MOD(IPROC,2).EQ.0)THEN

C           EVEN PROCESSORS
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              CALL P_RCVI(NCELHI,MPSNGL,IDEST,NOTAG)
              NDIFHI = NNCODE - NCELHI
            ENDIF
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              CALL P_RCVI(NCELLO,MPSNGL,IDEST,NOTAG)
              NDIFLO = NNCODE - NCELLO
            ENDIF

          ELSE

C           ODD PROCESSORS
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              CALL P_RCVI(NCELLO,MPSNGL,IDEST,NOTAG)
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              NDIFLO = NNCODE - NCELLO
            ENDIF
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              CALL P_RCVI(NCELHI,MPSNGL,IDEST,NOTAG)
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              NDIFHI = NNCODE - NCELHI
            ENDIF

          ENDIF

C         =====================================================================

C         BUFFER UP M-CODES AND DATA
C         --------------------------
C         NOTE THAT A POSITIVE VALUE OF NDIFLO OR NDIFHI
C         INDICATES AN EXCESS ON THE LOCAL PROCESSOR RELATIVE TO ITS NEIGHBOUR
          NNCOP1 = NNCODE + 1

C         HIGH END
          IF(NDIFHI.GT.0)THEN

C           NUMBER OF M-CODES TO MOVE
C           MAX OF HALF THE DIFFERENCE
            NMOVHI = NDIFHI/2

C           SPECIAL CASE: DIFFERENCE OF ONE CELL
            IF(NDIFHI.EQ.1)NMOVHI = NINT(RANUNP(ISEED))

C           BUFFER UP M-CODES
            NSTART = NNCOP1 - NMOVHI
            CALL MCMTRV(MCMHI,MCMSIZ,NMCODE,NSIZMX,NSTART,NNCODE)
            NNCODE = NNCODE - NMOVHI
            NSNDHI = 2*NMOVHI

          ENDIF

C         LOW END
          IF(NDIFLO.GT.0)THEN

C           NUMBER OF M-CODES TO MOVE
C           MAX OF HALF THE DIFFERENCE
            NMOVLO = NDIFLO/2

C           SPECIAL CASE: DIFFERENCE OF ONE CELL
            IF(NDIFLO.EQ.1)NMOVLO = NINT(RANUNP(ISEED))

C           BUFFER UP M-CODES
            NSTART = 1
            CALL MCMTRV(MCMLO,MCMSIZ,NMCODE,NSIZMX,NSTART,NMOVLO)
            NSNDLO = 2*NMOVLO

C           SHIFT M-CODES LEFT
            NEND = -1
            CALL MCSHFT(NMCODE,NSIZMX,NNCODE,NMOVLO,NEND)
            NNCODE = NNCODE - NMOVLO

          ENDIF

C         =====================================================================

C         MOVE M-CODES AND DATA
C         ---------------------
          NRCVLO = 0
          NRCVHI = 0

          IF(MOD(IPROC,2).EQ.0)THEN

C           EVEN PROCESSORS
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              IF(NDIFHI.GT.0)THEN
                CALL P_SNDI(NMOVHI,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMHI, MPARAY,NSNDHI,IDEST,NOTAG)
              ELSEIF(NDIFHI.LT.0)THEN
                CALL P_RCVI(NRCVHI,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMHI, MPARAY,IDEST,NOTAG)
              ENDIF
            ENDIF
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              IF(NDIFLO.GT.0)THEN
                CALL P_SNDI(NMOVLO,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMLO, MPARAY,NSNDLO,IDEST,NOTAG)
              ELSEIF(NDIFLO.LT.0)THEN
                CALL P_RCVI(NRCVLO,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMLO, MPARAY,IDEST,NOTAG)
              ENDIF
            ENDIF

          ELSE

C           ODD PROCESSORS
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              IF(NDIFLO.LT.0)THEN
                CALL P_RCVI(NRCVLO,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMLO, MPARAY,IDEST,NOTAG)
              ELSEIF(NDIFLO.GT.0)THEN
                CALL P_SNDI(NMOVLO,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMLO, MPARAY,NSNDLO,IDEST,NOTAG)
              ENDIF
            ENDIF
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              IF(NDIFHI.LT.0)THEN
                CALL P_RCVI(NRCVHI,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMHI, MPARAY,IDEST,NOTAG)
              ELSEIF(NDIFHI.GT.0)THEN
                CALL P_SNDI(NMOVHI,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMHI, MPARAY,NSNDHI,IDEST,NOTAG)
              ENDIF
            ENDIF

          ENDIF

C         =====================================================================

C         UNBUFFER M-CODES AND DATA
C         -------------------------
          IF(NRCVLO.GT.0)THEN

C           SHIFT M-CODES RIGHT
            NEND = 1
            CALL MCSHFT(NMCODE,NSIZMX,NNCODE,NRCVLO,NEND)
            NNCODE = NNCODE + NRCVLO

C           FILL LOW END WITH M-CODES
            NSTART = 1
            CALL MCMTOR(MCMLO,MCMSIZ,NMCODE,NSIZMX,NSTART,NRCVLO)

          ENDIF

          IF(NRCVHI.GT.0)THEN

C           FILL HIGH END WITH M-CODES
            NSTART = NNCODE + 1
            NNCODE = NNCODE + NRCVHI
            CALL MCMTOR(MCMHI,MCMSIZ,NMCODE,NSIZMX,NSTART,NNCODE)

          ENDIF

C         =====================================================================

C         CHECK CONVERGENCE AND DO IT AGAIN
C         ---------------------------------
          GIMBAL = REAL(NNCODE)
          GIMBAV = GIMBAL*OVPROC
          CALL P_SUMM(GIMBAV,GIMMAX)
          GIMBAL = ABS(GIMBAL-GIMMAX)/GIMMAX
          CALL P_SUMM(GIMBAL,GIMMAX)

          IF(GIMMAX.GT.GIMTOL)THEN
            IF(ABS(GIMMAX-GIMOLD).GT.GIRTOL)THEN
              IF(ICOUNT.LT.NGIMAX)THEN
                GIMOLD = GIMMAX
                ICOUNT = ICOUNT + 1
                GOTO 1000
              ELSE
C               ERROR CONDITION - load balancing failed to converge
                CALL ERHAND(17,20,2)
              ENDIF
            ENDIF
          ENDIF

C       END OF LOOP 1000

C       =======================================================================

      ENDIF

C     =========================================================================


      RETURN
      END
