      SUBROUTINE REFDAT

C     *************************************************************************
C
C     REFDAT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     30-JUN-2010:  CREATED
C     25-APR-2015:  RSC REWRITTEN FOR WENO
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DATA FOR REFINEMENT/DEREFINEMENT
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION ERRREF
      INTEGER JCELL,JSPEC
      INTEGER ICDREF,ICREFN
      INTEGER MINDEX

C     DIAGNOSTICS
      INTEGER MINDDR(NREMAX),MINDRR(NREMAX)
      INTEGER LPROC

C     DIAGNOSTICS
      LPROC = 20

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In REFDAT', ITIME

C     =========================================================================

C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE THE REFINEMENT CRITERIA ACCUMULATORS
      ERDREF = ZERO
      ERUREF(1) = ZERO
      ERUREF(2) = ZERO
      ERUREF(3) = ZERO
      ERTREF = ZERO
      ERPREF = ZERO
      DO JSPEC = 1,NSPEC
        ERYREF(JSPEC) = ZERO
      ENDDO
      ERMREF = ZERO

C     =========================================================================

C     ZERO THE COUNTERS
      N2DREF = 0
      N2REFN = 0
      ICDREF = 0
      ICREFN = 0

C     =========================================================================

C     TEMPERATURE, PRESSURE AND PRIMITIVE VARIABLES
      CALL TEMPRM

C     =========================================================================

C     LOOP OVER ALL CELLS
C     -------------------
      DO JCELL = 1, NCELL

C       INITIALISE CELL REFINEMENT STATUS CODE
        NRSTAT(JCELL) = NRNULL

C       INITIALISE OLD CELL INDEX LIST
        NCOCEL(JCELL) = JCELL

C       =======================================================================

C       CELL INDEX
        ICELL = JCELL

C       =======================================================================

C       RECONSTRUCTION
C       --------------
        IF(NREPOL(ICELL).GT.0)THEN
          CALL CRERES
        ELSE
          CALL CREREF
        ENDIF

C       =======================================================================

C       EVALUATE DE/REFINEMENT DATA
C       ---------------------------
        CALL REFCEL

C       =======================================================================

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(I5,4(1PE15.7))')JCELL,ERDREF,ERUREF(1),ERUREF(2),
C     +                                           ERUREF(3)
CC          WRITE(6,'(5X,4(1PE15.7))')ERTREF,ERPREF,ERYREF(1),ERYREF(2)
C        ENDIF

C       FIND THE MAXIMUM
C       ----------------
        ERRREF = ERDREF
        MINDEX = -4
        IF(ERUREF(1).GT.ERRREF)THEN
          ERRREF = ERUREF(1)
          MINDEX = -1
        ENDIF
        IF(ERUREF(2).GT.ERRREF)THEN
          ERRREF = ERUREF(2)
          MINDEX = -2
        ENDIF
        IF(ERUREF(3).GT.ERRREF)THEN
          ERRREF = ERUREF(3)
          MINDEX = -3
        ENDIF
        IF(ERTREF.GT.ERRREF)THEN
          ERRREF = ERTREF
          MINDEX = -6
        ENDIF
        DO JSPEC = 1, NSPEC
          IF(ERYREF(JSPEC).GT.ERRREF)THEN
            ERRREF = ERYREF(JSPEC)
            MINDEX = JSPEC
          ENDIF
        ENDDO
        IF(ERPREF.GT.ERRREF)THEN
          ERRREF = ERPREF
          MINDEX = -5
        ENDIF
        IF(ERMREF.GT.ERRREF)THEN
          ERRREF = ERMREF
          MINDEX = -7
        ENDIF
        
C       =======================================================================

CC       DIAGNOSTICS
C        IF(IPROC.EQ.LPROC)THEN
C          WRITE(6,'(4I5,3(1PE15.7))')JCELL,NLEVEL(JCELL),NMLMIN,NMLMAX,
C     +                               ERRREF,DDETOL,DRETOL
C        ENDIF

C       EVALUATE DEREFINEMENT CRITERIA
C       ------------------------------
        IF(NLEVEL(JCELL).GT.NLEVBQ)THEN

          IF(ERRREF.LT.DDETOL)THEN
            ICDREF = ICDREF + 1
            NINDDR(ICDREF) = JCELL
            MINDDR(ICDREF) = MINDEX
          ENDIF

        ENDIF

C       =======================================================================

C       EVALUATE REFINEMENT CRITERIA
C       ----------------------------
        IF(NLEVEL(JCELL).LT.NLEVRQ)THEN

          IF(ERRREF.GT.DRETOL)THEN
            ICREFN = ICREFN + 1
            NINDRR(ICREFN) = JCELL
            MINDRR(ICREFN) = MINDEX
          ENDIF

        ENDIF

C       =======================================================================

      ENDDO
C     END OF LOOP OVER CELLS

C     FINAL NUMBERS TO DE/REFINE
      N2DREF = ICDREF
      N2REFN = ICREFN

C     =========================================================================

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,'(2I5)')N2DREF,N2REFN
        DO ICDREF = 1, N2DREF
          JCELL = NINDDR(ICDREF)
          WRITE(6,'(5I5)')ICDREF,JCELL,NLEVEL(JCELL),NRSTAT(JCELL),
     +                    MINDDR(ICDREF)
        ENDDO
        WRITE(6,*)
        DO ICREFN = 1, N2REFN
          JCELL = NINDRR(ICREFN)
C          WRITE(6,'(5I5)')ICREFN,JCELL,NLEVEL(JCELL),NRSTAT(JCELL),
C     +                    MINDRR(ICREFN)
          WRITE(6,'(5I5,3(1PE12.4))')ICREFN,JCELL,
     +                               NLEVEL(JCELL),NRSTAT(JCELL),
     +                    MINDRR(ICREFN),
     +                    XCENTR(JCELL),YCENTR(JCELL),ZCENTR(JCELL)
        ENDDO
      ENDIF


      RETURN
      END
