      SUBROUTINE REFILT

C     *************************************************************************
C
C     REFILT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     25-NOV-2018:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     EXPLICIT FILTERING
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION FCOEFF(NCSTMX)
      DOUBLE PRECISION YTOTAL(NSPCMX)
      DOUBLE PRECISION UTOTAL(NCMPMX)
      DOUBLE PRECISION XCENTC,YCENTC,ZCENTC,OCSIZE
      DOUBLE PRECISION XSTENC,YSTENC,ZSTENC,OVSTEN
      DOUBLE PRECISION FORNOW,FTOTAL
      DOUBLE PRECISION DTOTAL,ETOTAL
      INTEGER JCELL,JSPEC,JSCELL,JCLEVL,JCSTEN,NJSTEN,NJSTP1


C     DIAGNOSTICS
      INTEGER LPROC

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In REFILT', ITIME

C     =========================================================================

C     BEGIN
C     =====

C     CHECK INTERVAL FOR FILTERING
C     ----------------------------
C      IF(MOD(ITIME,NTFILT).EQ.0)THEN

C       =======================================================================

C       PARALLEL TRANSFER
        CALL PARFER

C       RUN THROUGH ALL CELLS
        DO JCELL = 1, NCELL

C         CENTRAL CELL COORDINATES AND SIZE
          XCENTC = XCENTR(JCELL)
          YCENTC = YCENTR(JCELL)
          ZCENTC = ZCENTR(JCELL)
          JCLEVL = NLEVEL(JCELL)
          OCSIZE = OVSIZE(JCLEVL)

C         STENCIL SIZE
          NJSTEN = NOSSTN(JCELL)
          NJSTP1 = NJSTEN + 1

C         RUN THROUGH THE STENCIL
          FTOTAL = ZERO
          OVSTEN = ONE/REAL(NJSTP1)
          DO JCSTEN = 1, NJSTEN

C           STENCIL CELL COORDINATES AND SIZE
            JSCELL = NSSTEN(JCSTEN,JCELL)

C           INCLUDE PERIODIC BOUNDARY CONDITIONS
            XSTENC = (XCENTR(JSCELL) - XCENTC)
            XSTENC = XSTENC - XGDLEN*NINT(XSTENC/XGDLEN)
            XSTENC = XSTENC*OCSIZE
            YSTENC = (YCENTR(JSCELL) - YCENTC)
            YSTENC = YSTENC - YGDLEN*NINT(YSTENC/YGDLEN)
            YSTENC = YSTENC*OCSIZE
            ZSTENC = (ZCENTR(JSCELL) - ZCENTC)
            ZSTENC = ZSTENC - ZGDLEN*NINT(ZSTENC/ZGDLEN)
            ZSTENC = ZSTENC*OCSIZE

C           RADIAL DISTANCE
            FORNOW = XSTENC*XSTENC + YSTENC*YSTENC + ZSTENC*ZSTENC
C           4TH ORDER
            FORNOW = FORNOW*FORNOW
            FCOEFF(JCSTEN) = OVSTEN/FORNOW
            FTOTAL = FTOTAL + FCOEFF(JCSTEN)

          ENDDO
          FCOEFF(NJSTP1) = ONE - FTOTAL

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN

            IF(JCELL.LE.20)THEN

              WRITE(6,'(2I7)')JCELL,NJSTEN
              DO JCSTEN = 1, NJSTP1
                WRITE(6,'(I7,1PE15.7)')JCSTEN,FCOEFF(JCSTEN)
              ENDDO
C              WRITE(6,'(I7,2(1PE15.7))')JCSTEN,FTOTAL,ONE-FTOTAL

            ENDIF

          ENDIF

C         APPLY THE FILTER
          DTOTAL = FCOEFF(NJSTP1)*DRHS(JCELL)
          UTOTAL(1) = FCOEFF(NJSTP1)*URHS(1,JCELL)
          UTOTAL(2) = FCOEFF(NJSTP1)*URHS(2,JCELL)
          UTOTAL(3) = FCOEFF(NJSTP1)*URHS(3,JCELL)
          ETOTAL = FCOEFF(NJSTP1)*ERHS(JCELL)
          DO JSPEC = 1, NSPEC
            YTOTAL(JSPEC) = FCOEFF(NJSTP1)*YRHS(JSPEC,JCELL)
          ENDDO
          DO JCSTEN = 1, NJSTEN

            JSCELL = NSSTEN(JCSTEN,JCELL)
            DTOTAL = DTOTAL + FCOEFF(JCSTEN)*DRHS(JSCELL)
            UTOTAL(1) = UTOTAL(1) + FCOEFF(JCSTEN)*URHS(1,JSCELL)
            UTOTAL(2) = UTOTAL(2) + FCOEFF(JCSTEN)*URHS(2,JSCELL)
            UTOTAL(3) = UTOTAL(3) + FCOEFF(JCSTEN)*URHS(3,JSCELL)
            ETOTAL = ETOTAL + FCOEFF(JCSTEN)*ERHS(JSCELL)
            DO JSPEC = 1, NSPEC
              YTOTAL(JSPEC) = YTOTAL(JSPEC)
     +                      + FCOEFF(JCSTEN)*YRHS(JSPEC,JSCELL)
            ENDDO

          ENDDO
          DSAV(JCELL) = DTOTAL
          USAV(1,JCELL) = UTOTAL(1)
          USAV(2,JCELL) = UTOTAL(2)
          USAV(3,JCELL) = UTOTAL(3)
          ESAV(JCELL) = ETOTAL
          DO JSPEC = 1, NSPEC
            YSAV(JSPEC,JCELL) = YTOTAL(JSPEC)
          ENDDO

        ENDDO
C       RUN THROUGH ALL CELLS

C       UPDATE STORED VALUES
        DO JCELL = 1, NCELL

          DRHS(JCELL) = DSAV(JCELL)
          URHS(1,JCELL) = USAV(1,JCELL)
          URHS(2,JCELL) = USAV(2,JCELL)
          URHS(3,JCELL) = USAV(3,JCELL)
          ERHS(JCELL) = ESAV(JCELL)

          DRUN(JCELL) = DSAV(JCELL)
          URUN(1,JCELL) = USAV(1,JCELL)
          URUN(2,JCELL) = USAV(2,JCELL)
          URUN(3,JCELL) = USAV(3,JCELL)
          ERUN(JCELL) = ESAV(JCELL)

          DO JSPEC = 1, NSPEC
            YRHS(JSPEC,JCELL) = YSAV(JSPEC,JCELL)
            YRUN(JSPEC,JCELL) = YSAV(JSPEC,JCELL)
          ENDDO

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN

            IF(JCELL.LE.20)THEN

              WRITE(6,'(I7,2(1PE15.7))')NJSTEN,DRUN(JCELL),FTOTAL

            ENDIF

          ENDIF

        ENDDO
C       RUN THROUGH ALL CELLS

C       =======================================================================

C      ENDIF
C     CHECK INTERVAL FOR FILTERING


      RETURN
      END
