      SUBROUTINE RETREE
 
C     *************************************************************************
C
C     RETREE
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     31-AUG-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     REBUILDS THE LOCAL OCTREE
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      INTEGER MCRUN(NINDMX),MCLOC(NINDMX)
      INTEGER INCODE,MOCTNT
      INTEGER NFOUND,NFLEVL,NRLEVL
      CHARACTER*1 CFATTR,CRATTO,CRATTP,CRATTU

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)WRITE(6,*)'In RETREE'

C     =========================================================================

C     SET REQUIRED ATTRIBUTES
      CRATTO = CHOUTC
      CRATTP = CHPARL
      CRATTU = CHUNST

C     =========================================================================

C     REINITIALISE THE OCTREE
      CALL OCREIN(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM)

C     =========================================================================

C     LOW END
C     -------
      IF(IPROC.GT.0)THEN

C       LOWEST ALLOWABLE M-CODE ON THIS PROCESSOR
        CALL MCRTBL(MCLOC,NTABLE,NPROMX,IPROC-1)
        CALL MCINCR(MCLOC)
        CALL MCGLVL(MCLOC,NRLEVL)

1010    CONTINUE

C         SEARCH FOR THIS M-CODE IN THE OCTREE
          CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCLOC,NFLEVL,NFOUND,CFATTR)

          IF(NFLEVL.LT.NRLEVL)THEN

C           M-CODE NOT FOUND - ADD A NEW LEVEL TO THE OCTREE AND TRY AGAIN
            CALL OCTADL(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM,
     +                  MCLOC,NFLEVL,CRATTO)

            GOTO 1010

          ELSE

C           M-CODE FOUND - SET ATTRIBUTE AND MOVE ON
            CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCLOC,CRATTO)
  
          ENDIF

C       END OF LOOP 1010

      ENDIF

C     =========================================================================

C     RUN THROUGH ALL M-CODES ON THIS PROCESSOR
      DO INCODE = 1,NNCODE

C       FETCH THE M-CODE
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,INCODE)

C       FIND THE REQUIRED LEVEL
        CALL MCGLVL(MCRUN,NRLEVL)

1000    CONTINUE

C         SEARCH FOR THIS M-CODE IN THE OCTREE
          CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCRUN,NFLEVL,NFOUND,CFATTR)

          IF(NFLEVL.LT.NRLEVL)THEN

C           M-CODE NOT FOUND - ADD A NEW LEVEL TO THE OCTREE AND TRY AGAIN
            CALL OCTADL(NOCPAR,NOCHLD,COCODE,NOCTMX,NOCHWM,
     +                  MCRUN,NFLEVL,CRATTO)

            GOTO 1000

          ELSE

C           M-CODE FOUND - SET ATTRIBUTE AND MOVE ON
            CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCRUN,CRATTU)

          ENDIF

C       END OF LOOP 1000

      ENDDO
     
C     =========================================================================

C     SET ATTRIBUTES FOR REMOTE BRANCHES
C     ----------------------------------

C     LOW END
C     -------
      IF(IPROC.GT.0)THEN

C       LOWEST M-CODE ON THIS PROCESSOR
        CALL MCRTBL(MCRUN,NTABLE,NPROMX,IPROC-1)
        CALL MCINCR(MCRUN)
        CALL MCGLVL(MCRUN,NRLEVL)

2000    CONTINUE

C         SEARCH FOR THIS M-CODE IN THE OCTREE
          CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCRUN,NFLEVL,NFOUND,CFATTR)

C         CHECK LEVEL AS FOUND
          IF(NFLEVL.EQ.NRLEVL)THEN

C           OCTANT CONTAINING THE LOWEST M-CODE IS FOUND

C           EXTRACT HIGHEST OCTANT VALUE FROM M-CODE
            CALL MCGOCT(MCRUN,NRLEVL,MOCTNT)

C           MAKE A LOCAL COPY OF THE M-CODE
            CALL MCCOPY(MCRUN,MCLOC)

C           SET ATTRIBUTES WITHIN THE OCTANT
2010        CONTINUE

              IF(MOCTNT.GT.0)THEN

C               DECREMENT OCTANT AND M-CODE
                MOCTNT = MOCTNT - 1
                CALL MCDECR(MCLOC)

C               SET ATTRIBUTE TO REMOTE
                CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCLOC,CRATTP)
 
                GOTO 2010

              ENDIF
C           END OF LOOP 2010

          ELSE

C           LOWEST M-CODE NOT FOUND - DECREMENT LEVEL AND TRY AGAIN

            IF(NRLEVL.GT.1)THEN
              NRLEVL = NRLEVL - 1
              CALL MCPLVL(MCRUN,NRLEVL)
              GOTO 2000
            ENDIF

          ENDIF

C         MOVE BACK DOWN THE TREE
          IF(NRLEVL.GT.1)THEN
            NRLEVL = NRLEVL - 1
            CALL MCPLVL(MCRUN,NRLEVL)
            GOTO 2000
          ENDIF

C       END OF LOOP 2000

      ENDIF

C     -------------------------------------------------------------------------

C     HIGH END
C     --------
      IF(IPROC.LT.NPRM1)THEN

C       HIGHEST M-CODE ON THIS PROCESSOR
        CALL MCRTBL(MCRUN,NTABLE,NPROMX,IPROC)
        CALL MCGLVL(MCRUN,NRLEVL)

2100    CONTINUE

C         SEARCH FOR THIS M-CODE IN THE OCTREE
          CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCRUN,NFLEVL,NFOUND,CFATTR)

C         CHECK LEVEL AS FOUND
          IF(NFLEVL.EQ.NRLEVL)THEN

C           REMOTE M-CODE FOUND - EXTRACT HIGHEST OCTANT VALUE
            CALL MCGOCT(MCRUN,NRLEVL,MOCTNT)

C           MAKE A LOCAL COPY OF THE M-CODE
            CALL MCCOPY(MCRUN,MCLOC)

C           SET ATTRIBUTES WITHIN THE OCTANT
2110        CONTINUE

              IF(MOCTNT.LT.7)THEN

C               INCREMENT OCTANT AND M-CODE
                MOCTNT = MOCTNT + 1
                CALL MCINCR(MCLOC)

C               SET ATTRIBUTE TO REMOTE
                CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCLOC,CRATTP)
 
                GOTO 2110

              ENDIF
C           END OF LOOP 2110

C           MOVE BACK DOWN THE TREE
            IF(NRLEVL.GT.1)THEN
              NRLEVL = NRLEVL - 1
              CALL MCPLVL(MCRUN,NRLEVL)
              GOTO 2100
            ENDIF

          ELSE

C           ERROR CONDITION - remote cell code not found
            CALL ERHAND(4,16,4)

          ENDIF

C       END OF LOOP 2100

      ENDIF

C     -------------------------------------------------------------------------

C     HIGHEST END
C     -----------
      IF(IPROC.LT.NPRM1)THEN

C       HIGHEST M-CODE ON ANY PROCESSOR
        CALL MCRTBL(MCRUN,NTABLE,NPROMX,NPRM1)
        CALL MCGLVL(MCRUN,NRLEVL)

2200    CONTINUE

C         SEARCH FOR THIS M-CODE IN THE OCTREE
          CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCRUN,NFLEVL,NFOUND,CFATTR)

C         CHECK LEVEL AS FOUND
          IF(NFLEVL.EQ.NRLEVL)THEN

C           REMOTE M-CODE FOUND - EXTRACT HIGHEST OCTANT VALUE
            CALL MCGOCT(MCRUN,NRLEVL,MOCTNT)

C           MAKE A LOCAL COPY OF THE M-CODE
            CALL MCCOPY(MCRUN,MCLOC)

C           SET ATTRIBUTES WITHIN THE OCTANT
2210        CONTINUE

              IF(MOCTNT.LT.7)THEN

C               INCREMENT OCTANT AND M-CODE
                MOCTNT = MOCTNT + 1
                CALL MCINCR(MCLOC)

C               SET ATTRIBUTE TO OUT
                CALL OCSATT(NOCHLD,COCODE,NOCTMX,MCLOC,CRATTO)
 
                GOTO 2210

              ENDIF

C           END OF LOOP 2210

          ELSE

C           REMOTE M-CODE NOT FOUND - DECREMENT LEVEL AND TRY AGAIN
            IF(NRLEVL.GT.1)THEN
              NRLEVL = NRLEVL - 1
              CALL MCPLVL(MCRUN,NRLEVL)
              GOTO 2200
            ENDIF

          ENDIF

C         MOVE BACK DOWN THE TREE
          IF(NRLEVL.GT.1)THEN
            NRLEVL = NRLEVL - 1
            CALL MCPLVL(MCRUN,NRLEVL)
            GOTO 2200
          ENDIF

C       END OF LOOP 2200

      ENDIF

C     =========================================================================


      RETURN
      END
