      SUBROUTINE RHSCEL 

C     *************************************************************************
C
C     RHSCEL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     03-FEB-2008:  CREATED
C     11-AUG-2014:  RSC REWRITTEN FOR WENO
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES RIGHT-HAND-SIDES FOR TIME INTEGRATION OF ALL PDEs
C     INCLUDES MULTIPLE SCALARS AND MULTI-STEP CHEMISTRY
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     LOCAL DATA
C     ==========
      INTEGER IFCODE(NFACMX)
      INTEGER JCELL,JSPEC,JRNCEL,JFACE,ISCODE
      INTEGER IOCELL

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'IN RHSCEL'

C     =========================================================================

C     RUN THROUGH ALL CELLS
C     INCLUDING HALO CELLS
      DO JCELL = 1, NCELL+NRCELL

C       SAVE THE CONSERVED QUANTITIES
        DSOR(JCELL) = DRHS(JCELL)
        USOR(1,JCELL) = URHS(1,JCELL)
        USOR(2,JCELL) = URHS(2,JCELL)
        USOR(3,JCELL) = URHS(3,JCELL)
        ESOR(JCELL) = ERHS(JCELL)
        DO JSPEC = 1,NSPEC
          YSOR(JSPEC,JCELL) = YRHS(JSPEC,JCELL)
        ENDDO

C       REINITIALISE THE CELL FACE INDEX ARRAY
        DO JFACE = 1, NFACE
          NEFACE(JFACE,JCELL) = 0
        ENDDO

      ENDDO
C     END OF RUN THROUGH ALL CELLS

C     REINITIALISE THE CELL FACE FREE SLOT STACK
      NFSTAK(1) = 0
      ISTACK = 1

C     =========================================================================

C     LOOP OVER ALL CELLS
C     -------------------
      DO JCELL = 1, NCELL

C       =======================================================================

C       CELL INDEX
        ICELL = JCELL

C       REINITIALISE THE RHS ACCUMULATORS
        DRHS(ICELL) = ZERO
        URHS(1,ICELL) = ZERO
        URHS(2,ICELL) = ZERO
        URHS(3,ICELL) = ZERO
        ERHS(ICELL) = ZERO
        DO JSPEC = 1, NSPEC
          YRHS(JSPEC,ICELL) = ZERO
        ENDDO

C       CELL FACE STATUS CODES
        ISCODE = NSCODE(ICELL)
        IFCODE(NFWEST) = MOD(ISCODE,NSBASE)
        ISCODE = ISCODE/NSBASE
        IFCODE(NFEAST) = MOD(ISCODE,NSBASE)
        ISCODE = ISCODE/NSBASE
        IFCODE(NFSUTH) = MOD(ISCODE,NSBASE)
        ISCODE = ISCODE/NSBASE
        IFCODE(NFNRTH) = MOD(ISCODE,NSBASE)
        ISCODE = ISCODE/NSBASE
        IFCODE(NFLOWR) = MOD(ISCODE,NSBASE)
        ISCODE = ISCODE/NSBASE
        IFCODE(NFUPPR) = MOD(ISCODE,NSBASE)

C       =======================================================================

C       RECONSTRUCTION
C       --------------
        IF(NREPOL(ICELL).GT.0)THEN
          CALL CRESTR
        ELSE
          CALL CREPOL
        ENDIF
        CALL CEVLGP(IFCODE)

C       =======================================================================
C
C       SOURCE TERMS
C       ------------
        CALL CSTERM

C       =======================================================================

C       CELL FACE FLUXES
C       ----------------
        IF(NSCODE(JCELL).EQ.NCLEAN)THEN

C         =====================================================================

C         CLEAN CELL
C         ----------
          CALL CFLUXF(NFWEST)
          CALL CFLUXF(NFEAST)
          CALL CFLUXF(NFSUTH)
          CALL CFLUXF(NFNRTH)
          CALL CFLUXF(NFLOWR)
          CALL CFLUXF(NFUPPR)

C         =====================================================================

        ELSE IF(NSCODE(JCELL).GT.NCLEAN)THEN

C         =====================================================================

C         CELL WITH SIZE TRANSITION OR BC
C         -------------------------------

C         RUN THROUGH ALL FACES
          DO JFACE = 1, NFACE

C           -------------------------------------------------------------------

            IF(IFCODE(JFACE).EQ.NCLEAN)THEN

C             EVALUATE CLEAN CELL FACE FLUXES
              CALL CFLUXF(JFACE)

            ELSE IF(IFCODE(JFACE).EQ.N2HTOH)THEN

C             EVALUATE CELL FACE FLUXES FOR 2H TO H SIZE TRANSITION
              CALL C2H2HF(JFACE)

            ELSE IF(IFCODE(JFACE).GE.NHTO2H)THEN

C             EVALUATE CELL FACE FLUXES FOR H TO 2H SIZE TRANSITION
              CALL CH22HF(JFACE,IFCODE)

            ELSE IF(IFCODE(JFACE).GE.NBCOND)THEN

C             EVALUATE CELL FACE FLUXES FOR BOUNDARY FACE
              CALL CFLBCF(JFACE,IFCODE)

            ELSE

C             ERROR CONDITION - invalid cell face code
              CALL ERHAND(11,21,4)

            ENDIF

C           -------------------------------------------------------------------

          ENDDO
C         END OF RUN THROUGH ALL FACES

C         =====================================================================

        ELSE

C         =====================================================================

C         ERROR CONDITION - non-solvable cell
C         ---------------
          CALL ERHAND(11,8,4)

C         =====================================================================

        ENDIF
C       END OF CELL STATUS CODE

C       =======================================================================

      ENDDO
C     END OF LOOP OVER CELLS

C     =========================================================================

C     DEAL WITH REMAINING CELL FACE FLUXES
C     ------------------------------------

C     INITIALISE PREVIOUS CELL INDEX
      IOCELL = 0

C     LOOP OVER FACE NEIGHBOUR HALO CELLS
C     -----------------------------------
      DO JRNCEL = 1, NRNCEL

C       =======================================================================

C       CELL INDEXING
        JCELL = NRNLOC(JRNCEL)
        ICELL = JCELL

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("RHSHAL",5I5)')IPROC,JRNCEL,JCELL,ICELL,IOCELL
        ENDIF

C       CHECK PREVIOUS CELL
        IF(ICELL.NE.IOCELL)THEN

C         CELL FACE STATUS CODES
          ISCODE = NSCODE(ICELL)
          IFCODE(NFWEST) = MOD(ISCODE,NSBASE)
          ISCODE = ISCODE/NSBASE
          IFCODE(NFEAST) = MOD(ISCODE,NSBASE)
          ISCODE = ISCODE/NSBASE
          IFCODE(NFSUTH) = MOD(ISCODE,NSBASE)
          ISCODE = ISCODE/NSBASE
          IFCODE(NFNRTH) = MOD(ISCODE,NSBASE)
          ISCODE = ISCODE/NSBASE
          IFCODE(NFLOWR) = MOD(ISCODE,NSBASE)
          ISCODE = ISCODE/NSBASE
          IFCODE(NFUPPR) = MOD(ISCODE,NSBASE)

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("RHSPRE",15I4)')IPROC,JRNCEL,ICELL,
     +                               (NXDOOR(JFACE,ICELL),JFACE=1,6),
     +                               (IFCODE(JFACE),JFACE=1,6)
            WRITE(6,'("RHSPRE",30I4)')IPROC,JRNCEL,ICELL,
     +                      (NSSTEN(JFACE,ICELL),JFACE=1,NOSSTN(ICELL))
          ENDIF

C         RECONSTRUCTION
C         --------------
          CALL CREPOL
          CALL CEVLGP(IFCODE)

C         PREVIOUS CELL INDEX
          IOCELL = ICELL

        ENDIF

C       FACE INDEXING
        JFACE = NRNFAC(JRNCEL)

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,'("RHSFAC",7I5)')IPROC,JRNCEL,NRNLOC(JRNCEL),
     +                     NRNFAC(JRNCEL),NRNCOD(JRNCEL),IFCODE(JFACE),
     +                     NXDOOR(JFACE,JCELL)
        ENDIF

C       CELL FACE STATUS CODE
        IFCODE(JFACE) = NRNCOD(JRNCEL)

C       -----------------------------------------------------------------------

        IF(IFCODE(JFACE).EQ.NCLEAN)THEN

C         EVALUATE CLEAN CELL FACE FLUXES
          CALL CFLUXF(JFACE)

        ELSE IF(IFCODE(JFACE).EQ.N2HTOH)THEN

C         EVALUATE CELL FACE FLUXES FOR 2H TO H SIZE TRANSITION
          CALL C2H2HF(JFACE)

        ELSE IF(IFCODE(JFACE).GE.NHTO2H)THEN

C         EVALUATE CELL FACE FLUXES FOR H TO 2H SIZE TRANSITION
          CALL CH22HF(JFACE,IFCODE)

        ELSE IF(IFCODE(JFACE).GE.NBCOND)THEN

C         EVALUATE CELL FACE FLUXES FOR BOUNDARY FACE
          CALL CFLBCF(JFACE,IFCODE)

        ELSE

C         ERROR CONDITION - invalid cell face code
          CALL ERHAND(11,21,4)

        ENDIF

C       -----------------------------------------------------------------------

      ENDDO
C     END OF LOOP OVER FACE NEIGHBOUR HALO CELLS

C     =========================================================================


      RETURN
      END
