      SUBROUTINE SORTMC(MARRAY,NASIZE,N2SORT)
 
C     *************************************************************************
C
C     SORTMC
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     22-SEP-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SORTS AN ARRAY OF M-CODES
C     USING HEAPSORT
C
C     BASED ON NUMERICAL RECIPES pp229-232
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER MARRAY(NINDMX,NASIZE)
      INTEGER NASIZE,N2SORT


C     LOCAL DATA
C     ==========
      INTEGER MCRUN(NINDMX),MCTMP1(NINDMX),MCTMP2(NINDMX)
      INTEGER IINDEX,JINDEX,JINDP1,KINDEX,LINDEX
      INTEGER NMCOMP


C     BEGIN
C     =====

C     =========================================================================

C     PRELIMINARIES
C     -------------
      LINDEX = N2SORT/2 + 1

C     SORT ONLY IF ARRAY CONTAINS MORE THAN ONE ELEMENT
      IF(LINDEX.GT.1)THEN

C       =======================================================================

C       FORM THE HEAP
C       -------------
C       PICK A NEW ELEMENT FROM THE UPPER HALF OF THE HEAP
C                     I.E. FROM THE LOWER HALF OF THE ARRAY
1000    CONTINUE

          LINDEX = LINDEX - 1
          CALL MCRTRV(MCRUN,MARRAY,NASIZE,LINDEX)

          IINDEX = LINDEX
          JINDEX = LINDEX + LINDEX

C         LOOP TO SIFT THE NEW ELEMENT DOWN THE HEAP
1100      CONTINUE

            CALL MCRTRV(MCTMP1,MARRAY,NASIZE,JINDEX)

C           FIND THE GREATER OF EACH PAIR OF EXISTING LOWER ELEMENTS
            IF(JINDEX.LT.N2SORT)THEN
              JINDP1 = JINDEX + 1
              CALL MCRTRV(MCTMP2,MARRAY,NASIZE,JINDP1)
              CALL MCCOMP(MCTMP1,MCTMP2,NMCOMP)
              IF(NMCOMP.LT.0)THEN
                JINDEX = JINDP1
                CALL MCCOPY(MCTMP2,MCTMP1)
              ENDIF
            ENDIF

C           COMPARE THE NEW ELEMENT WITH THE EXISTING ELEMENT
            CALL MCCOMP(MCRUN,MCTMP1,NMCOMP)
            IF(NMCOMP.LT.0)THEN
              CALL MCMOVE(MARRAY,NASIZE,JINDEX,IINDEX)
              IINDEX = JINDEX
              JINDEX = JINDEX + JINDEX
              IF(JINDEX.LE.N2SORT)GOTO 1100
            ENDIF

C         END OF LOOP 1100

C         PUT THE NEW ELEMENT INTO ITS SLOT
          CALL MCSTOR(MCRUN,MARRAY,NASIZE,IINDEX)

          IF(LINDEX.GT.1)GOTO 1000

C       END OF LOOP 1000

C       =======================================================================

C       SORT THE HEAP
C       -------------
        KINDEX = N2SORT

2000    CONTINUE

C         PICK THE HIGHEST UNSORTED ELEMENT OF THE ARRAY
          CALL MCRTRV(MCRUN,MARRAY,NASIZE,KINDEX)

C         MOVE THE CURRENT TOP-OF-THE-HEAP INTO THAT SLOT
          CALL MCMOVE(MARRAY,NASIZE,1,KINDEX)

C         DECREMENT INDEX OF HIGHEST UNSORTED ELEMENT
          KINDEX = KINDEX - 1

C         CHECK FOR LOWEST ELEMENT
          IF(KINDEX.GT.1)THEN

C           NOT THE LOWEST ELEMENT
            IINDEX = 1
            JINDEX = 2

C           LOOP TO SIFT THE ELEMENT DOWN THE HEAP
2100        CONTINUE

              CALL MCRTRV(MCTMP1,MARRAY,NASIZE,JINDEX)

C             FIND THE GREATER OF EACH PAIR OF EXISTING LOWER ELEMENTS
              IF(JINDEX.LT.KINDEX)THEN
                JINDP1 = JINDEX + 1
                CALL MCRTRV(MCTMP2,MARRAY,NASIZE,JINDP1)
                CALL MCCOMP(MCTMP1,MCTMP2,NMCOMP)
                IF(NMCOMP.LT.0)THEN
                  JINDEX = JINDP1
                  CALL MCCOPY(MCTMP2,MCTMP1)
                ENDIF
              ENDIF

C             COMPARE THE NEW ELEMENT WITH THE EXISTING ELEMENT
              CALL MCCOMP(MCRUN,MCTMP1,NMCOMP)
              IF(NMCOMP.LT.0)THEN
                CALL MCMOVE(MARRAY,NASIZE,JINDEX,IINDEX)
                IINDEX = JINDEX
                JINDEX = JINDEX + JINDEX
                IF(JINDEX.LE.KINDEX)GOTO 2100
              ENDIF

C           END OF LOOP 2100

C           PUT THE NEW ELEMENT INTO ITS SLOT
            CALL MCSTOR(MCRUN,MARRAY,NASIZE,IINDEX)
            GOTO 2000

          ELSE

C           LOWEST ELEMENT
            CALL MCSTOR(MCRUN,MARRAY,NASIZE,1)

          ENDIF

C       END OF LOOP 2000

C       =======================================================================

      ENDIF
C     END OF SORT ONLY IF ARRAY CONTAINS MORE THAN ONE ELEMENT

C     =========================================================================


      RETURN
      END
