      SUBROUTINE STARTH
 
C     *************************************************************************
C
C     STARTH
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     CARRIES OUT INITIAL PARALLEL DOMAIN DECOMPOSITION
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
C     FILENAME COMPONENTS
      CHARACTER*4 PNCONT,PNCHEM,PNBCIO,PNREPT,PNSTAT,PNRESO
      CHARACTER*4 PNDMPI,PNDMPO
      CHARACTER*4 PNXDAT,PNXRES
      PARAMETER(PNCONT = 'cont', PNCHEM = 'chem', PNBCIO = 'bcio',
     +          PNREPT = 'rept', PNSTAT = 'stat', PNRESO = 'reso',
     +          PNDMPI = 'dmpi', PNDMPO = 'dmpo',
     +          PNXDAT = '.dat', PNXRES = '.res')

C     REPORT FILE LINE LENGTH
      INTEGER LENLIN
      PARAMETER(LENLIN = 79)


C     LOCAL DATA
C     ===========
      INTEGER IC
      CHARACTER*1 PNFLAG


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE PARALLEL MESSAGE PASSING
C     ===================================
C     INITIALISE MPI
      CALL P_INIT

C     GET THE NUMBER OF PROCESSORS NPROC
C     AND THE LOCAL PROCESSOR ID IPROC: IPROC = 0 TO NPROC-1
      CALL P_SIZE(NPROC,IPROC)
      NPRM1 = NPROC - 1
      OVPROC = ONE/REAL(NPROC)

C     =========================================================================

C     SET UP FILE I/O
C     ===============

C     BUILD THE FILENAMES
      WRITE(FNPROC,'(I6.6)')IPROC
      FNXDAT = PNXDAT
      FNXRES = PNXRES
      FNCONT = PNCONT//FNXDAT
      FNCHEM = PNCHEM//FNXDAT
      FNBCIO = PNBCIO//FNPROC//FNXDAT
      FNREPT = PNREPT//FNXRES
      FNSTAT = PNSTAT//FNXRES
      FNREST = PNRESO
      IRFLAG = 0
      IDFLAG = 0
      WRITE(PNFLAG,'(I1)')IDFLAG
      FNDMPO(1) = PNDMPI//FNPROC//PNFLAG//FNXDAT
      IDFLAG = 1
      WRITE(PNFLAG,'(I1)')IDFLAG
      FNDMPO(2) = PNDMPI//FNPROC//PNFLAG//FNXDAT
      IDTIME = -1

C     =========================================================================

C     WRITE THE INITIAL REPORT
C     ------------------------
      IF(IPROC.EQ.0)THEN

C       SET UP SEPARATOR LINES
        DO IC = 1,LENLIN
          CLSTAR(IC:IC) = '*'
          CLDASH(IC:IC) = '-'
        ENDDO

        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='NEW',FORM='FORMATTED')

C       WRITE A HEADER
        WRITE(NCREPT,9000)CLSTAR
        WRITE(NCREPT,9010)
        WRITE(NCREPT,9020)
        WRITE(NCREPT,9010)
        WRITE(NCREPT,9030)
        WRITE(NCREPT,9010)
        WRITE(NCREPT,9040)
        WRITE(NCREPT,9010)
        WRITE(NCREPT,9000)CLSTAR
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'==========='
        WRITE(NCREPT,*)'REPORT FILE'
        WRITE(NCREPT,*)'==========='
        WRITE(NCREPT,*)

        WRITE(NCREPT,9000)CLDASH

        WRITE(NCREPT,*)
        WRITE(NCREPT,'("Number of processors: Max:     ",I5)')NPROMX
        WRITE(NCREPT,'("                      Actual:  ",I5)')NPROC
        WRITE(NCREPT,*)

        WRITE(NCREPT,9000)CLDASH

        CLOSE(NCREPT)

      ENDIF

C     =========================================================================

C     INITIALISE THE ERROR HANDLER
      CALL ERINIT

C     =========================================================================

      RETURN

C     =========================================================================

C     FORMATS FOR REPORT FILE HEADER
9000  FORMAT(A79)
9010  FORMAT('**',75X,'**')
9020  FORMAT('**',18X,'DIRECT NUMERICAL SIMULATION CODE HAMISH',
     +            18X,'**')
9030  FORMAT('**',31X,'Stewart Cant',32X,'**')
9040  FORMAT('**',16X,'Cambridge University Engineering Department',
     +            16X,'**')

C     =========================================================================

      END
