      SUBROUTINE STNCAT
 
C     *************************************************************************
C
C     STNCAT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT
C
C     CHANGE RECORD
C     -------------
C     14-OCT-2019:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     CATEGORISES EACH STENCIL
C     USING A PRE-STORED LIBRARY OF CONFIGURATIONS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------

C     PARAMETERS
C     ==========
      DOUBLE PRECISION SSCALE
      PARAMETER(SSCALE = 1.0D1)
      DOUBLE PRECISION STOTOL
      PARAMETER(STOTOL = 1.0D-20)
      INTEGER NSTRMX
      PARAMETER(NSTRMX = 10)
      INTEGER NCSLIB
      PARAMETER(NCSLIB = 4)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION SCSTOR(NCSTMX,NSTRMX)
      DOUBLE PRECISION XSTORL(NCSTMX,NSTRMX)
      DOUBLE PRECISION YSTORL(NCSTMX,NSTRMX)
      DOUBLE PRECISION ZSTORL(NCSTMX,NSTRMX)
      DOUBLE PRECISION STLSIZ(NCSTMX,NSTRMX)
      DOUBLE PRECISION XSTORD(NCSTMX),YSTORD(NCSTMX),ZSTORD(NCSTMX)
      DOUBLE PRECISION STRSIZ(NCSTMX)
      DOUBLE PRECISION SCOORD(NCSTMX)
      DOUBLE PRECISION XCENTC,YCENTC,ZCENTC,CCSIZE,OCSIZE
      DOUBLE PRECISION XSTENC,YSTENC,ZSTENC
      DOUBLE PRECISION STOTAL,SCDIFF
      INTEGER NOSTOR(NSTRMX),NMATCH(NSTRMX)
      INTEGER NSTLOC(NCSTMX)
      INTEGER JCELL,JSCELL
      INTEGER JCSTEN,NJSTEN
      INTEGER JCLEVL
      INTEGER ISTORC,NSTORC
      LOGICAL FLSTLN,FLSTOT

C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     PRELIMINARIES
C     -------------

C     INITIALISE LIBRARY INDEX
      NSTORC = 1
      DO ISTORC = 1, NSTRMX
        NMATCH(ISTORC) = 0
      ENDDO

C     =========================================================================

C     INITIALISE THE LIBRARY
C     ----------------------
      CALL STNLIN(XSTORL,YSTORL,ZSTORL,STLSIZ,NSTRMX,NOSTOR,NSTORC)

      IF(NSTORC.GT.0)THEN

C       READ THE STORED STENCILS
        DO ISTORC = 1, NSTORC

C         SIZE OF STORED STENCIL
          NJSTEN = NOSTOR(ISTORC)

C         RUN THROUGH THE STORED STENCIL
          DO JCSTEN = 1, NJSTEN

C           STORED STENCIL RELATIVE COORDINATES
            XSTORD(JCSTEN) = XSTORL(JCSTEN,ISTORC)
            YSTORD(JCSTEN) = YSTORL(JCSTEN,ISTORC)
            ZSTORD(JCSTEN) = ZSTORL(JCSTEN,ISTORC)
            STRSIZ(JCSTEN) = STLSIZ(JCSTEN,ISTORC)

C           MAP TO 1D USING X-Y-Z ORDERING
            NSTLOC(JCSTEN) = JCSTEN
            SCOORD(JCSTEN) = XSTORD(JCSTEN)
     +                     +(YSTORD(JCSTEN)
     +                     + ZSTORD(JCSTEN)*SSCALE)*SSCALE

          ENDDO
C         RUN THROUGH THE STENCIL

C         SORT BY MAPPED RELATIVE COORDINATE
          CALL SORTRI(SCOORD,NSTLOC,NCSTMX,NJSTEN)

C         ADD SORTED STENCIL TO LIBRARY OF STORED CONFIGURATIONS
          DO JCSTEN = 1, NJSTEN
            SCSTOR(JCSTEN,ISTORC) = SCOORD(JCSTEN)
          ENDDO

C         STENCIL INDEX
          NMPINV = ISTORC

C         COMPUTE AND STORE THE MOORE-PENROSE PSEUDO-INVERSE
          CALL CRELIB(XSTORD,YSTORD,ZSTORD,STRSIZ,NSTLOC,NJSTEN)

        ENDDO
C       READ THE STORED STENCILS

C       =======================================================================

C       RUN THROUGH ALL INTERIOR CELLS
        DO JCELL = 1,NCELL

C         CENTRAL CELL COORDINATES AND SIZE
          NJSTEN = NOSSTN(JCELL)
          XCENTC = XCENTR(JCELL)
          YCENTC = YCENTR(JCELL)
          ZCENTC = ZCENTR(JCELL)
          JCLEVL = NLEVEL(JCELL)
          CCSIZE = HCSIZE(JCLEVL)
          OCSIZE = OVSIZE(JCLEVL)

C         RUN THROUGH THE STENCIL
          DO JCSTEN = 1, NJSTEN

C           STENCIL CELL COORDINATES AND SIZE
            JSCELL = NSSTEN(JCSTEN,JCELL)
            XSTENC = (XCENTR(JSCELL) - XCENTC)
            XSTENC = XSTENC - XGDLEN*NINT(XSTENC/XGDLEN)
            XSTENC = XSTENC*OCSIZE
            YSTENC = (YCENTR(JSCELL) - YCENTC)
            YSTENC = YSTENC - YGDLEN*NINT(YSTENC/YGDLEN)
            YSTENC = YSTENC*OCSIZE
            ZSTENC = (ZCENTR(JSCELL) - ZCENTC)
            ZSTENC = ZSTENC - ZGDLEN*NINT(ZSTENC/ZGDLEN)
            ZSTENC = ZSTENC*OCSIZE

C           MAP TO 1D USING X-Y-Z ORDERING
            NSTLOC(JCSTEN) = JSCELL
            SCOORD(JCSTEN) = XSTENC + (YSTENC + ZSTENC*SSCALE)*SSCALE

          ENDDO

C         SORT BY MAPPED RELATIVE COORDINATE
          CALL SORTRI(SCOORD,NSTLOC,NCSTMX,NJSTEN)

C         STORE THE SORTED STENCIL
          DO JCSTEN = 1, NJSTEN
            NSSTEN(JCSTEN,JCELL) = NSTLOC(JCSTEN)
          ENDDO

C         COMPARE TO LIBRARY OF STORED CONFIGURATIONS
          ISTORC = 1
          NREPOL(JCELL) = 0
1000      CONTINUE

C           CHECK THE SIZE OF THE STENCIL
            FLSTLN = (NJSTEN.EQ.NOSTOR(ISTORC))

C           INITIALISE THE TOTAL FOR COMPARISON
            STOTAL = ZERO

C           CHECK THE STRUCTURE OF THE STENCIL
            IF(FLSTLN)THEN

              DO JCSTEN = 1, NJSTEN
                SCDIFF = SCOORD(JCSTEN) - SCSTOR(JCSTEN,ISTORC)
                STOTAL = STOTAL + SCDIFF*SCDIFF
              ENDDO

            ENDIF
            FLSTOT = (STOTAL.LT.STOTOL)

C           CHECK FOR A MATCH TO A STENCIL ALREADY IN THE LIBRARY
            IF(FLSTLN.AND.FLSTOT)THEN

C             MATCH
              NREPOL(JCELL) = ISTORC
              NMATCH(ISTORC) = NMATCH(ISTORC) + 1

            ELSE

C             NO MATCH
              IF(ISTORC.LT.NSTORC)THEN

C               KEEP SEARCHING
                ISTORC =  ISTORC + 1
                GOTO 1000

              ENDIF
C             NO MATCH - GO TO NEXT CELL

            ENDIF
C           CHECK FOR A MATCH TO A STENCIL ALREADY IN THE LIBRARY

C         END OF LOOP 1000

        ENDDO
C       RUN THROUGH ALL INTERIOR CELLS

C       =======================================================================

        IF(IPROC.EQ.LPROC)THEN
          DO ISTORC = 1, NSTORC
            WRITE(6,'(2I7)')ISTORC,NMATCH(ISTORC)
          ENDDO
          DO JCELL = 1, 1000
            WRITE(6,'(2I8)')JCELL,NREPOL(JCELL)
          ENDDO
        ENDIF

C       ELSE STENCIL LIBRARY FILE DOES NOT EXIST
C       NO ACTION

      ENDIF

C     =========================================================================


      RETURN
      END
