      SUBROUTINE TEMPIN
 
C     *************************************************************************
C
C     TEMPIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     19-JUL-2008:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES TEMPERATURE AND PRESSURE
C     USES BISECTION METHOD FOR ROBUSTNESS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLTMP
C      PARAMETER(TOLTMP = 1.0D-10)
      PARAMETER(TOLTMP = 1.0D-5)
      DOUBLE PRECISION TININC
      PARAMETER(TININC = 5.0D1)
      DOUBLE PRECISION TLIMLO,TLIMHI
      PARAMETER(TLIMLO = 2.0D2, TLIMHI = 3.0D3)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TCOEFF(0:NCTMAX)
      DOUBLE PRECISION UKUK
      DOUBLE PRECISION TEMPOR,TUPPER,TLOWER,TRESID
      DOUBLE PRECISION RGCMIX
      INTEGER JCELL,ISPEC,ITINT,ICP
      LOGICAL FNCONV 


C     BEGIN
C     =====

C     =========================================================================

C     TEMPERATURE AND PRESSURE
C     ------------------------

C     REFERENCE PRESSURE
      PREFER = PRIN

C     =========================================================================

C     RUN THROUGH ALL CELLS
C     INCLUDING HALO CELLS
      DO JCELL = 1, NCELL+NRCELL

C       =======================================================================

C       KINETIC ENERGY TERM
        UKUK = HALF*(URHS(1,JCELL)*URHS(1,JCELL)
     +       +       URHS(2,JCELL)*URHS(2,JCELL)
     +       +       URHS(3,JCELL)*URHS(3,JCELL))/DRHS(JCELL)

C        TUK2ME = UKUK - ERHS(JCELL)

C       =======================================================================

C       SOLVE FOR TEMPERATURE
C       =====================
C       USING BISECTION
        TLOWER = TLIMLO
        TUPPER = TLIMHI

C       SET NON-CONVERGENCE FLAG
        FNCONV = .TRUE.

C       SET TEMPERATURE FROM INITIAL GUESS
        TEMPOR = TRIN

C       =======================================================================

C       FIRST BRACKET THE ROOT
C       ======================

C       -----------------------------------------------------------------------

C       INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
C        TCOEFF(0) = TUK2ME
        TCOEFF(0) = ZERO
        DO ICP = 1, NCTMAX
          TCOEFF(ICP) = ZERO
        ENDDO

C       TEMPERATURE POLYNOMIAL
        DO ISPEC = 1,NSPEC

C         LOCATE EXISTING TEMPERATURE IN AN INTERVAL
          ITINT = 1
1100      CONTINUE
            IF(TEMPOR.GT.TINTHI(ITINT,ISPEC))THEN
              IF(ITINT.LT.NTINT(ISPEC))THEN
                ITINT = ITINT + 1
                GOTO 1100
              ENDIF
            ENDIF
C         END OF LOOP 1100

C         CONSTRUCT COEFFICIENTS OF TEMPERATURE POLYNOMIAL
          TCOEFF(0) = TCOEFF(0)
     +              + YRHS(ISPEC,JCELL)*
     +                AMASCP(NCENTH(ITINT,ISPEC),ITINT,ISPEC)

          TCOEFF(1) = TCOEFF(1)
     +              + YRHS(ISPEC,JCELL)*AMASCT(1,ITINT,ISPEC)
          DO ICP = 2, NCPOLY(ITINT,ISPEC)
            TCOEFF(ICP) = TCOEFF(ICP)
     +              + YRHS(ISPEC,JCELL)*AMASCT(ICP,ITINT,ISPEC)
          ENDDO

        ENDDO
        TCOEFF(0) = TCOEFF(0) - ERHS(JCELL)
        TCOEFF(0) = TCOEFF(0) + UKUK
C       END OF TEMPERATURE POLYNOMIAL

C       -----------------------------------------------------------------------

C       EVALUATE TEMPERATURE RESIDUAL
        TRESID = TCOEFF(NCTMAX)
        DO ICP = NCTMM1,1,-1
          TRESID = TCOEFF(ICP) + TRESID*TEMPOR
        ENDDO
        TRESID = TCOEFF(0) + TRESID*TEMPOR

C       -----------------------------------------------------------------------

C       CHECK INITIAL GUESS FOR CONVERGENCE
        IF(ABS(TRESID).LT.TOLTMP)THEN

C         ---------------------------------------------------------------------

C         CONVERGED ON FIRST PASS
          FNCONV = .FALSE.

C         ---------------------------------------------------------------------

        ELSEIF(TRESID.LT.ZERO)THEN

C         ---------------------------------------------------------------------

C         INITIAL GUESS IS TOO LOW
2000      CONTINUE

C           SET INITIAL GUESS AS LOWER LIMIT AND TRY AGAIN
            TLOWER = TEMPOR
            TEMPOR = TEMPOR + TININC

C           -------------------------------------------------------------------

C           INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL
C            TCOEFF(0) = TUK2ME
            TCOEFF(0) = ZERO
            DO ICP = 1, NCTMAX
              TCOEFF(ICP) = ZERO
            ENDDO

C           FOR EACH SPECIES LOCATE TEMPERATURE IN AN INTERVAL
            DO ISPEC = 1,NSPEC

              ITINT = 1
2100          CONTINUE
                IF(TEMPOR.GT.TINTHI(ITINT,ISPEC))THEN
                  IF(ITINT.LT.NTINT(ISPEC))THEN
                    ITINT = ITINT + 1
                    GOTO 2100
                  ENDIF
                ENDIF
C             END OF LOOP 2100

C             CONSTRUCT COEFFICIENTS OF TEMPERATURE POLYNOMIAL
              TCOEFF(0) = TCOEFF(0)
     +                  + YRHS(ISPEC,JCELL)*
     +                    AMASCP(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
              TCOEFF(1) = TCOEFF(1)
     +                  + YRHS(ISPEC,JCELL)*AMASCT(1,ITINT,ISPEC)
              DO ICP = 2, NCPOLY(ITINT,ISPEC)
                TCOEFF(ICP) = TCOEFF(ICP)
     +                + YRHS(ISPEC,JCELL)*AMASCT(ICP,ITINT,ISPEC)
              ENDDO

            ENDDO
            TCOEFF(0) = TCOEFF(0) - ERHS(JCELL)
            TCOEFF(0) = TCOEFF(0) + UKUK
C           END OF TEMPERATURE POLYNOMIAL

C           -------------------------------------------------------------------

C           EVALUATE TEMPERATURE RESIDUAL
            TRESID = TCOEFF(NCTMAX)
            DO ICP = NCTMM1,1,-1
              TRESID = TCOEFF(ICP) + TRESID*TEMPOR
            ENDDO
            TRESID = TCOEFF(0) + TRESID*TEMPOR

C           -------------------------------------------------------------------

C           CHECK NEW GUESS FOR CONVERGENCE
            IF(ABS(TRESID).LT.TOLTMP)THEN

C             -----------------------------------------------------------------

C             NEW GUESS HAS CONVERGED
              FNCONV = .FALSE.

C             -----------------------------------------------------------------

            ELSEIF(TRESID.LT.ZERO)THEN

C             -----------------------------------------------------------------

C             NEW GUESS IS STILL TOO LOW: GO ROUND AGAIN
              IF(TEMPOR.LT.TLIMHI)THEN
                GO TO 2000
              ELSE
C               ERROR CONDITION - T upper bracket failed to converge
                CALL ERHAND(8,17,4)
              ENDIF

C             -----------------------------------------------------------------

            ELSEIF(TRESID.GT.ZERO)THEN

C             -----------------------------------------------------------------

C             ROOT IS BRACKETED
              TUPPER = TEMPOR

C             -----------------------------------------------------------------

            ENDIF

C           -------------------------------------------------------------------

C         END OF LOOP 2000

C         ---------------------------------------------------------------------

        ELSEIF(TRESID.GT.ZERO)THEN

C         ---------------------------------------------------------------------

C         INITIAL GUESS IS TOO HIGH
3000      CONTINUE

C           SET INITIAL GUESS AS UPPER LIMIT AND TRY AGAIN
            TUPPER = TEMPOR
            TEMPOR = TEMPOR - TININC

C           -------------------------------------------------------------------

C           INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL
C            TCOEFF(0) = TUK2ME
            TCOEFF(0) = ZERO
            DO ICP = 1, NCTMAX
              TCOEFF(ICP) = ZERO
            ENDDO

C           FOR EACH SPECIES LOCATE TEMPERATURE IN AN INTERVAL
            DO ISPEC = 1,NSPEC

              ITINT = 1
3100          CONTINUE
                IF(TEMPOR.GT.TINTHI(ITINT,ISPEC))THEN
                  IF(ITINT.LT.NTINT(ISPEC))THEN
                    ITINT = ITINT + 1
                    GOTO 3100
                  ENDIF
                ENDIF
C             END OF LOOP 3100

C             CONSTRUCT COEFFICIENTS OF TEMPERATURE POLYNOMIAL
              TCOEFF(0) = TCOEFF(0)
     +                  + YRHS(ISPEC,JCELL)*
     +                    AMASCP(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
              TCOEFF(1) = TCOEFF(1)
     +                  + YRHS(ISPEC,JCELL)*AMASCT(1,ITINT,ISPEC)
              DO ICP = 2, NCPOLY(ITINT,ISPEC)
                TCOEFF(ICP) = TCOEFF(ICP)
     +                + YRHS(ISPEC,JCELL)*AMASCT(ICP,ITINT,ISPEC)
              ENDDO

            ENDDO
            TCOEFF(0) = TCOEFF(0) - ERHS(JCELL)
            TCOEFF(0) = TCOEFF(0) + UKUK
C           END OF TEMPERATURE POLYNOMIAL

C           -------------------------------------------------------------------

C           EVALUATE TEMPERATURE RESIDUAL
            TRESID = TCOEFF(NCTMAX)
            DO ICP = NCTMM1,1,-1
              TRESID = TCOEFF(ICP) + TRESID*TEMPOR
            ENDDO
            TRESID = TCOEFF(0) + TRESID*TEMPOR

C           -------------------------------------------------------------------

C           CHECK NEW GUESS FOR CONVERGENCE
            IF(ABS(TRESID).LT.TOLTMP)THEN

C             -----------------------------------------------------------------

C             NEW GUESS HAS CONVERGED
              FNCONV = .FALSE.

C             -----------------------------------------------------------------

            ELSEIF(TRESID.GT.ZERO)THEN

C             -----------------------------------------------------------------

C             NEW GUESS IS STILL TOO HIGH: GO ROUND AGAIN
              IF(TEMPOR.GT.TLIMLO)THEN
                GO TO 3000
              ELSE
C               ERROR CONDITION - T lower bracket failed to converge
                CALL ERHAND(8,18,4)
              ENDIF

C             -----------------------------------------------------------------

            ELSEIF(TRESID.LT.ZERO)THEN

C             -----------------------------------------------------------------

C             ROOT IS BRACKETED
              TLOWER = TEMPOR

C             -----------------------------------------------------------------

            ENDIF

C           -------------------------------------------------------------------

C         END OF LOOP 3000

C         -----------------------------------------------------------------

        ENDIF
C       END OF CHECK INITIAL GUESS FOR CONVERGENCE

C       =======================================================================

C       ROOT IS BRACKETED
C       =================
C       NOW USE BISECTION TO REFINE THE ROOT

        IF(FNCONV)THEN

4000      CONTINUE

C           BISECT
            TEMPOR = HALF*(TLOWER+TUPPER)

C           -------------------------------------------------------------------

C           INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL
C            TCOEFF(0) = TUK2ME
            TCOEFF(0) = ZERO
            DO ICP = 1, NCTMAX
              TCOEFF(ICP) = ZERO
            ENDDO

C           FOR EACH SPECIES LOCATE TEMPERATURE IN AN INTERVAL
            DO ISPEC = 1,NSPEC

              ITINT = 1
4100          CONTINUE
                IF(TEMPOR.GT.TINTHI(ITINT,ISPEC))THEN
                  IF(ITINT.LT.NTINT(ISPEC))THEN
                    ITINT = ITINT + 1
                    GOTO 4100
                  ENDIF
                ENDIF
C             END OF LOOP 4100

C             CONSTRUCT COEFFICIENTS OF TEMPERATURE POLYNOMIAL
              TCOEFF(0) = TCOEFF(0)
     +                  + YRHS(ISPEC,JCELL)*
     +                    AMASCP(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
              TCOEFF(1) = TCOEFF(1)
     +                  + YRHS(ISPEC,JCELL)*AMASCT(1,ITINT,ISPEC)
              DO ICP = 2, NCPOLY(ITINT,ISPEC)
                TCOEFF(ICP) = TCOEFF(ICP)
     +                + YRHS(ISPEC,JCELL)*AMASCT(ICP,ITINT,ISPEC)
              ENDDO

            ENDDO
            TCOEFF(0) = TCOEFF(0) - ERHS(JCELL)
            TCOEFF(0) = TCOEFF(0) + UKUK
C           END OF TEMPERATURE POLYNOMIAL

C           -------------------------------------------------------------------

C           EVALUATE TEMPERATURE RESIDUAL
            TRESID = TCOEFF(NCTMAX)
            DO ICP = NCTMM1,1,-1
              TRESID = TCOEFF(ICP) + TRESID*TEMPOR
            ENDDO
            TRESID = TCOEFF(0) + TRESID*TEMPOR

C           -------------------------------------------------------------------

            IF(ABS(TRESID).LT.TOLTMP)THEN

C             CONVERGED
              TRUN(JCELL) = TEMPOR

            ELSEIF(TRESID.LT.ZERO)THEN

              TLOWER = TEMPOR
              GOTO 4000

            ELSEIF(TRESID.GT.ZERO)THEN

              TUPPER = TEMPOR
              GOTO 4000

            ENDIF

C         ---------------------------------------------------------------------
C         END OF LOOP 4000

        ENDIF
C       END OF BISECTION

C       =======================================================================

C       SET THE NEW TEMPERATURE
        TRUN(JCELL) = TEMPOR

C       =======================================================================

C       INITIALISE ACCUMULATORS FOR MIXTURE SPECIFIC HEAT AND GAS CONSTANT
        RGCMIX = ZERO

C       =======================================================================

C       FOR ALL SPECIES RELOCATE TEMPERATURE IN AN INTERVAL
        DO ISPEC = 1,NSPEC

          ITINT = 1
5100      CONTINUE
            IF(TRUN(JCELL).GT.TINTHI(ITINT,ISPEC))THEN
              IF(ITINT.LT.NTINT(ISPEC))THEN
                ITINT = ITINT + 1
                GOTO 5100
              ENDIF
            ENDIF
C         END OF LOOP 5100

C         =====================================================================

C         EVALUATE (DENSITY TIMES) GAS CONSTANT
          RGCMIX = RGCMIX + YRHS(ISPEC,JCELL)*RGSPEC(ISPEC)

C         =====================================================================

        ENDDO
C       END OF RELOCATE TEMPERATURE

C       =======================================================================

C       EVALUATE PRESSURE
        PRUN(JCELL) = RGCMIX*TRUN(JCELL) - PREFER

C       =======================================================================

      ENDDO
C     END OF RUN THROUGH ALL CELLS

C     =========================================================================


      RETURN
      END
