      SUBROUTINE TEMPOP(TOUT,POUT)
 
C     *************************************************************************
C
C     TEMPOP
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     16-MAR-2008:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES TEMPERATURE AND PRESSURE
C     VERSION FOR RESULTS OUTPUT
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      DOUBLE PRECISION TOLTMP
      PARAMETER(TOLTMP = 1.0E-6)
      INTEGER NTITRS
      PARAMETER(NTITRS = 100)


C     ARGUMENTS
C     ==========
      DOUBLE PRECISION TOUT(NSIZMX),POUT(NSIZMX)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION TCOEFF(0:NCTMAX),TDERIV(1:NCTMAX)
      DOUBLE PRECISION TEMPOR,TFPOLY,TDPOLY,DELTMP
      DOUBLE PRECISION UKUK,RGCMIX
      DOUBLE PRECISION YEVAL(NSPCMX)
      DOUBLE PRECISION UEVAL(NCMPMX)
      DOUBLE PRECISION DEVAL,EEVAL
      DOUBLE PRECISION XLOC,YLOC,ZLOC
      INTEGER JCELL,ICP,ISPEC,ITINT,ITITRS
 
C     DIAGNOSTICS
      INTEGER LPROC

C     =========================================================================

C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     REFERENCE PRESSURE
      PREFER = PRIN

C     =========================================================================

C     PARALLEL TRANSFER
      CALL PARFER

C     =========================================================================

C     SAVE THE CONSERVED QUANTITIES
C     INCLUDING HALO CELLS
      DO JCELL = 1, NCELL+NRCELL
        DSOR(JCELL) = DRHS(JCELL)
        USOR(1,JCELL) = URHS(1,JCELL)
        USOR(2,JCELL) = URHS(2,JCELL)
        USOR(3,JCELL) = URHS(3,JCELL)
        ESOR(JCELL) = ERHS(JCELL)
        DO ISPEC = 1, NSPEC
          YSOR(ISPEC,JCELL) = YRHS(ISPEC,JCELL)
        ENDDO
      ENDDO

C     =========================================================================

C     RUN THROUGH ALL CELLS
      DO JCELL = 1, NCELL

C       =======================================================================

CC       DIAGNOSTICS
C        WRITE(6,*)'TEMPOP:',JCELL

        ICELL = JCELL
        IF(NREPOL(ICELL).GT.0)THEN
          CALL CRESTR
        ELSE
          CALL CREPOL
        ENDIF
        XLOC = ZERO
        YLOC = ZERO
        ZLOC = ZERO
        CALL CEVALL(XLOC,YLOC,ZLOC,DEVAL,UEVAL,EEVAL,YEVAL)

C       =======================================================================

C       KINETIC ENERGY
        UKUK = HALF*(UEVAL(1)*UEVAL(1)
     +       +       UEVAL(2)*UEVAL(2)
     +       +       UEVAL(3)*UEVAL(3))/DEVAL

C       =======================================================================

C       INITIALISE COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
        TCOEFF(0) = UKUK - EEVAL
        DO ICP = 1, NCTMAX
          TCOEFF(ICP) = ZERO
          TDERIV(ICP) = ZERO
        ENDDO

C       =======================================================================

C       TEMPERATURE POLYNOMIAL
        DO ISPEC = 1,NSPEC

C         =====================================================================

C         LOCATE EXISTING TEMPERATURE IN AN INTERVAL
          ITINT = 1

C         =====================================================================

C         CONSTRUCT COEFFICIENTS OF TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
          TCOEFF(0) = TCOEFF(0)
     +      + YEVAL(ISPEC)*AMASCE(NCENTH(ITINT,ISPEC),ITINT,ISPEC)
          DO ICP = 1, NCPOLY(ITINT,ISPEC)
            TCOEFF(ICP) = TCOEFF(ICP)
     +                  + YEVAL(ISPEC)*AMASCE(ICP,ITINT,ISPEC)
            TDERIV(ICP) = TDERIV(ICP)
     +                  + YEVAL(ISPEC)*AMASCV(ICP,ITINT,ISPEC)
          ENDDO

C         =====================================================================

        ENDDO
C       END OF TEMPERATURE POLYNOMIAL

C       =======================================================================

C       INITIAL GUESS FOR TEMPERATURE
C        TEMPOR = -TCOEFF(0)/TCOEFF(1)
        TEMPOR = TRUN(JCELL)

C       =======================================================================

C       SOLVE FOR TEMPERATURE USING NEWTON-RAPHSON
C       ---------------------
        ITITRS = 1
1000    CONTINUE

C         EVALUATE TEMPERATURE POLYNOMIAL AND ITS DERIVATIVE
          TFPOLY = TCOEFF(NCTMAX)
          TDPOLY = TDERIV(NCTMAX)
          DO ICP = NCTMM1,1,-1
            TFPOLY = TCOEFF(ICP) + TFPOLY*TEMPOR
            TDPOLY = TDERIV(ICP) + TDPOLY*TEMPOR
          ENDDO
          TFPOLY = TCOEFF(0) + TFPOLY*TEMPOR

C         EVALUATE TEMPERATURE CORRECTION
          DELTMP = -TFPOLY/TDPOLY

C         CHECK FOR CONVERGENCE
          IF(ABS(DELTMP).GT.TOLTMP)THEN
            IF(ITITRS.LT.NTITRS)THEN
              TEMPOR = TEMPOR + DELTMP
              ITITRS = ITITRS + 1
              GOTO 1000
            ELSE
C             ERROR CONDITION - T iteration failed to converge'
              CALL ERHAND(9,18,4)
            ENDIF
          ENDIF

C       END OF LOOP 1000

C       =======================================================================

C       SET THE NEW TEMPERATURE FOR OUTPUT
        TOUT(JCELL) = TEMPOR

C       =======================================================================

C       EVALUATE PRESSURE FOR OUTPUT
        RGCMIX = ZERO
        DO ISPEC = 1,NSPEC
          RGCMIX = RGCMIX + YEVAL(ISPEC)*RGSPEC(ISPEC)
        ENDDO
        POUT(JCELL) = RGCMIX*TEMPOR - PREFER

C       =======================================================================

      ENDDO
C     END OF RUN THROUGH ALL CELLS

C     =========================================================================


      RETURN
      END
