      PROGRAM TOVIEW

C     *************************************************************************
C
C     TOVIEW
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT - CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     02-MAY-2010:  CREATED
C     29-APR-2014:  RSC MULTIDIMENSIONAL DOMIAN
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     POSTPROCESSES THE RESULTS
C     FOR INPUT TO PARAVIEW
C
C     FILES: RESULTS INPUT FILES     - UNIT NCRESO - NAMES FNRESO - UNFORMATTED
C            PARAVIEW OUTPUT FILES   - UNIT NCVIEW - NAMES FNVIEW - FORMATTED
C            PARAVIEW PARALLEL FILE  - UNIT NCVIEW - NAMES FNPVUE - FORMATTED
C            CONTROL FILE (OPTIONAL) - UNIT NCCONT - NAME  FNCONT - FORMATTED
C
C     *************************************************************************


C     PARAMETERS
C     ==========
C     NUMERICAL PARAMETERS
      DOUBLE PRECISION ZERO,ONE,TWO,HALF
      PARAMETER(ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0, HALF = 5.0D-1)

C     MAX NO OF CELLS,LEVELS,COMPONENTS,SPECIES
      INTEGER NCELMX,NLEVMX,NCMPMX,NSPCMX
      PARAMETER(NCELMX = 1000000, NLEVMX = 18, NCMPMX = 3, NSPCMX = 18)
C     MAX NO OF POINTS,VERTICES
      INTEGER NPTSMX,NVRTMX
      PARAMETER(NPTSMX = 8000000, NVRTMX = 8)
C     MAX SIZE OF M-CODE
      INTEGER NINDMX
      PARAMETER(NINDMX = 2)

C     RSC 29-APR-2014 MULTID
      DOUBLE PRECISION CELTOL
      PARAMETER(CELTOL = 1.0D-10)

C     FILTER TOLERANCES
      DOUBLE PRECISION DTOL,UTOL,ETOL,TTOL,PTOL,YTOL
      PARAMETER(DTOL = 1.0D-6, UTOL = 1.0D-6, ETOL = 1.0D-6)
      PARAMETER(TTOL = 1.0D-6, PTOL = 1.0D-6, YTOL = 1.0D-10)

C     FILENAME COMPONENTS
      CHARACTER*8 PNCONT
      CHARACTER*4 PNRESO,PNVIEW
      CHARACTER*4 PNXDAT,PNXRES,PNXVTU
      CHARACTER*5 PNPVUE,PNPVTU
      PARAMETER(PNCONT = 'contview')
      PARAMETER(PNRESO = 'reso', PNVIEW = 'view')
      PARAMETER(PNXDAT = '.dat', PNXRES = '.res')
      PARAMETER(PNXVTU = '.vtu', PNPVUE = 'pview',PNPVTU = '.pvtu')

      INTEGER LPCONT,LPRESO,LPVIEW,LPXDAT,LPXRES,LPXVTU,LPPVUE,LPPVTU
      PARAMETER(LPCONT = 8, LPRESO = 4, LPVIEW = 4)
      PARAMETER(LPXDAT = 4, LPXRES = 4)
      PARAMETER(LPXVTU = 4, LPPVUE = 5, LPPVTU = 5)

C     OUTPUT FILE LINE LENGTH
      INTEGER LENLIN
      PARAMETER(LENLIN = 79)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YRUN(NSPCMX,NCELMX)
      DOUBLE PRECISION URUN(NCMPMX,NCELMX)
      DOUBLE PRECISION POINTS(NCMPMX,NPTSMX)
      DOUBLE PRECISION VSIGN(NCMPMX,NVRTMX)
      DOUBLE PRECISION STRN12(NCELMX)
      DOUBLE PRECISION STRN13(NCELMX)
      DOUBLE PRECISION STRN23(NCELMX)
      DOUBLE PRECISION SRUN(NCELMX)
      DOUBLE PRECISION DRUN(NCELMX),TRUN(NCELMX),PRUN(NCELMX)
      DOUBLE PRECISION ERUN(NCELMX)
      DOUBLE PRECISION HCSIZE(0:NLEVMX)
      DOUBLE PRECISION XGPHYR,YGPHYR,ZGPHYR
      DOUBLE PRECISION ETIME,TSTEP,ERROLD,ERRLDR
      DOUBLE PRECISION HALFCX,HALFCY,HALFCZ

C      INTEGER MCLIST(NINDMX,NPTSMX)
      INTEGER ICNECT(NPTSMX)
      INTEGER NLEVEL(NCELMX)
      INTEGER MXINC(NVRTMX),MYINC(NVRTMX),MZINC(NVRTMX)
C      INTEGER MCRUN(NINDMX),MCCMP(NINDMX)
      INTEGER IRPROC,NRPROC
      INTEGER NBLEVL,NLEVL,NCELL,NSPEC
      INTEGER JCELL,JSPEC,JLEVL,JCOMP,JVERT,KVERT,JPOINT
C      INTEGER KPOINT
      INTEGER NCCONT,NCRESO,NCVIEW,NCPVUE
      INTEGER LNCONT,LNRESO,LNVIEW,LNPVUE
      INTEGER IPROC,NRPRM1
      INTEGER NRIFMT,IRFLAG
      INTEGER NPOINT,NVERT,ICTYPE
C      INTEGER MCOMPR
C      INTEGER MX,MY,MZ,MVX,MVY,MVZ
      CHARACTER*30 SPECNM(NSPCMX),SPECPM(NSPCMX)
      CHARACTER*30 SPECSC(NSPCMX),SPECPC(NSPCMX)
      CHARACTER*80 FNCONT,FNRESO,FNVIEW,FNPVUE
      CHARACTER*79 ALINE
      CHARACTER*6 PNPROC
      CHARACTER*4 PNFLAG
      CHARACTER*2 PNSPEC
      LOGICAL FXCONT
C      LOGICAL FPOINT

C     =========================================================================

C     SET PARAMETERS AND CONSTANTS
C     ============================
      NCRESO = 1
      NCCONT = 2
      NCPVUE = 7
      NCVIEW = 8

C     VERTEX INCREMENTS
      DO KVERT = 1,8
        MXINC(KVERT) = 0
        MYINC(KVERT) = 0
        MZINC(KVERT) = 0
      ENDDO
      DO KVERT = 5,8
        MZINC(KVERT) = 1
      ENDDO
      MYINC(3) = 1
      MYINC(4) = 1
      MYINC(7) = 1
      MYINC(8) = 1
      MXINC(2) = 1
      MXINC(4) = 1
      MXINC(6) = 1
      MXINC(8) = 1

C     VERTEX SIGNS
      DO KVERT = 1,8
        DO JVERT = 1,3
          VSIGN(JVERT,KVERT) = ONE
        ENDDO
      ENDDO
      DO KVERT = 1,4
        VSIGN(3,KVERT) = -ONE
      ENDDO
      VSIGN(2,1) = -ONE
      VSIGN(2,2) = -ONE
      VSIGN(2,5) = -ONE
      VSIGN(2,6) = -ONE
      VSIGN(1,1) = -ONE
      VSIGN(1,3) = -ONE
      VSIGN(1,5) = -ONE
      VSIGN(1,7) = -ONE

C     CELL TYPE
      ICTYPE = 11

C     -------------------------------------------------------------------------

C     INITIALISE M-CODE MANIPULATION
      CALL MCINIT

C     -------------------------------------------------------------------------

C     BUILD THE FILENAME FOR THE CONTROL FILE
      FNCONT = PNCONT//PNXDAT
      LNCONT = LPCONT + LPXDAT

C     =========================================================================

C     CONTROL PARAMETERS
C     ==================
      NRIFMT = 1
      IRFLAG = 0

C     -------------------------------------------------------------------------

C     CHECK FOR EXISTENCE OF CONTROL FILE
      INQUIRE(FILE=FNCONT,EXIST=FXCONT)
      IF(FXCONT)THEN

C       CONTROL FILE EXISTS
        OPEN(UNIT=NCCONT,FILE=FNCONT,STATUS='OLD',FORM='FORMATTED')

C       READ AND IGNORE THE HEADER
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)
        READ(NCCONT,*)

C       CONTROL PARAMETERS
C       --------------
C       INPUT FORMAT
        READ(NCCONT,*)
        READ(NCCONT,*)NRIFMT
        READ(NCCONT,*)

C       RESULTS INPUT FILE ID FLAG
        READ(NCCONT,*)
        READ(NCCONT,*)IRFLAG

        CLOSE(NCCONT)

      ENDIF
C     CHECK FOR EXISTENCE

C     =========================================================================

C     READ THE DUMP FILES
C     ===================

C     FIRST ONE FROM LOWEST RANKED PROCESSOR
      IPROC = 0
      NRPRM1 = IPROC

1000  CONTINUE

C       DUMP INPUT FILENAME
        WRITE(PNPROC,'(I6.6)')IPROC
        WRITE(PNFLAG,'(I4.4)')IRFLAG
        FNRESO = PNRESO//PNPROC//PNFLAG//PNXRES
        LNRESO = LPRESO + 6 + 4 + LPXRES

C       -----------------------------------------------------------------------

C       SELECT DUMP FORMAT
        IF(NRIFMT.EQ.0)THEN

C         ---------------------------------------------------------------------

C         UNFORMATTED DUMP INPUT

C         READ THE DATA FROM DUMP INPUT FILE 1
C         NOTE THAT STRNxx ARRAYS CONTAIN CELL-CENTRE COORDINATES
C         NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
          OPEN(UNIT=NCRESO,FILE=FNRESO,STATUS='OLD',FORM='UNFORMATTED')
          REWIND(NCRESO)

          READ(NCRESO)IRPROC,NRPROC
          READ(NCRESO)XGPHYR,YGPHYR,ZGPHYR
          READ(NCRESO)HCSIZE(0),NBLEVL,NLEVL,NCELL,NSPEC
          READ(NCRESO)ETIME,TSTEP,ERROLD,ERRLDR
          READ(NCRESO)(NLEVEL(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(STRN12(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(STRN13(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(STRN23(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(SRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(DRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(ERUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(TRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(PRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO)(URUN(1,JCELL),JCELL=1,NCELL)
          READ(NCRESO)(URUN(2,JCELL),JCELL=1,NCELL)
          READ(NCRESO)(URUN(3,JCELL),JCELL=1,NCELL)
          DO JSPEC = 1,NSPEC
            READ(NCRESO)(YRUN(JSPEC,JCELL),JCELL=1,NCELL)
          ENDDO

          CLOSE(NCRESO)

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         FORMATTED DUMP INPUT

C         READ THE DATA FROM DUMP INPUT FILE 1
C         NOTE THAT STRNxx ARRAYS CONTAIN CELL-CENTRE COORDINATES
C         NOTE THAT URUN,ERUN AND YRUN ARE ALL IN CONSERVATIVE FORM
          OPEN(UNIT=NCRESO,FILE=FNRESO,STATUS='OLD',FORM='FORMATTED')
          REWIND(NCRESO)

          READ(NCRESO,*)IRPROC,NRPROC
          READ(NCRESO,*)XGPHYR,YGPHYR,ZGPHYR
          READ(NCRESO,*)HCSIZE(0),NBLEVL,NLEVL,NCELL,NSPEC
          READ(NCRESO,*)ETIME,TSTEP,ERROLD,ERRLDR
          READ(NCRESO,*)(NLEVEL(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(STRN12(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(STRN13(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(STRN23(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(SRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(DRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(ERUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(TRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(PRUN(JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(URUN(1,JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(URUN(2,JCELL),JCELL=1,NCELL)
          READ(NCRESO,*)(URUN(3,JCELL),JCELL=1,NCELL)
          DO JSPEC = 1,NSPEC
            READ(NCRESO,*)(YRUN(JSPEC,JCELL),JCELL=1,NCELL)
          ENDDO

          CLOSE(NCRESO)

C         ---------------------------------------------------------------------

        ENDIF

C       =======================================================================

C       CHECK FOR FIRST PASS
        IF(IPROC.EQ.0)THEN

C         ---------------------------------------------------------------------

C         ACTIONS ON FIRST PASS
C         ---------------------

C         NUMBER OF PROCESSORS (MINUS ONE)
          NRPRM1 = NRPROC - 1

C         CELL SIZES
          DO JLEVL = 1,NLEVL
            HCSIZE(JLEVL) = HALF*HCSIZE(JLEVL-1)
          ENDDO

C         SPECIES STRINGS
          DO JSPEC = 1, NSPEC
            WRITE(PNSPEC,'(I2.2)')JSPEC
            SPECSC(JSPEC) = '         Scalars="Species'//PNSPEC//'"'
            SPECPC(JSPEC) = '      Scalars="Species'//PNSPEC//'"'
            SPECNM(JSPEC) = '           Name="Species'//PNSPEC//'"'
            SPECPM(JSPEC) = '        Name="Species'//PNSPEC//'"'
          ENDDO

C         BUILD THE FILENAME FOR THE PARALLEL FILE
          FNPVUE = PNPVUE//PNFLAG//PNPVTU
          LNPVUE = LPPVUE + 4 + LPPVTU

C         WRITE THE HEADER TO THE PARALLEL FILE
          OPEN(UNIT=NCPVUE,FILE=FNPVUE,STATUS='NEW',FORM='FORMATTED')
          WRITE(NCPVUE,'(A)')'<VTKFile type="PUnstructuredGrid">'
          WRITE(NCPVUE,'(A)')'  <PUnstructuredGrid GhostLevel="0">'

C         WRITE THE FILE INFORMATION TO THE PARALLEL FILE
          WRITE(NCPVUE,*)'   <PCellData'
          WRITE(NCPVUE,*)'     Vectors="Velocity"'
          WRITE(NCPVUE,*)'     Scalars="Density"'
C          WRITE(NCPVUE,*)'     Scalars="Energy"'
C          DO JSPEC = 1, NSPEC
C            WRITE(NCPVUE,'(A)')SPECPC(JSPEC)
C          ENDDO
          WRITE(NCPVUE,*)'   >'
          WRITE(NCPVUE,*)'     <PDataArray'
          WRITE(NCPVUE,*)'       NumberOfComponents="3"'
          WRITE(NCPVUE,*)'       type="Float32"'
          WRITE(NCPVUE,*)'       Name="Velocity"'
          WRITE(NCPVUE,*)'       format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          WRITE(NCPVUE,*)'     <PDataArray'
          WRITE(NCPVUE,*)'       type="Float32"'
          WRITE(NCPVUE,*)'       Name="Density"'
          WRITE(NCPVUE,*)'       format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          WRITE(NCPVUE,*)'     <PDataArray'
          WRITE(NCPVUE,*)'       type="Float32"'
          WRITE(NCPVUE,*)'       Name="Energy"'
          WRITE(NCPVUE,*)'       format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          WRITE(NCPVUE,*)'     <PDataArray'
          WRITE(NCPVUE,*)'       type="Float32"'
          WRITE(NCPVUE,*)'       Name="Temperature"'
          WRITE(NCPVUE,*)'       format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          WRITE(NCPVUE,*)'     <PDataArray'
          WRITE(NCPVUE,*)'       type="Float32"'
          WRITE(NCPVUE,*)'       Name="Pressure"'
          WRITE(NCPVUE,*)'       format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          DO JSPEC = 1, NSPEC
            WRITE(NCPVUE,*)'     <PDataArray'
            WRITE(NCPVUE,*)'       type="Float32"'
            WRITE(NCPVUE,'(A)')SPECPM(JSPEC)
            WRITE(NCPVUE,*)'       format="ascii"'
            WRITE(NCPVUE,*)'     >'
            WRITE(NCPVUE,*)'     </PDataArray>'
          ENDDO
          WRITE(NCPVUE,*)'   </PCellData>'
          WRITE(NCPVUE,*)'   <PPoints>'
          WRITE(NCPVUE,*)'     <PDataArray NumberOfComponents="3"'
          WRITE(NCPVUE,*)'                 type="Float32"'
          WRITE(NCPVUE,*)'                 format="ascii"'
          WRITE(NCPVUE,*)'     >'
          WRITE(NCPVUE,*)'     </PDataArray>'
          WRITE(NCPVUE,*)'   </PPoints>'

C         ---------------------------------------------------------------------

        ENDIF
C       FIRST PASS

C       =======================================================================

C       ACTIONS ON ALL PASSES
C       ---------------------

C       BUILD THE FILENAME FOR THE VIEW FILE
        FNVIEW = PNVIEW//PNPROC//PNFLAG//PNXVTU
        LNVIEW = LPVIEW + 6 + 4 + LPXVTU

C       WRITE THE FILENAME TO THE PARALLEL FILE
        ALINE = '    <Piece Source="'//FNVIEW(1:LNVIEW)//'"/>'
        WRITE(NCPVUE,'(A)')ALINE

CC       POINT COORDINATES
C        JPOINT = 0
C        JVERT  = 0
C        DO JCELL = 1, NCELL
C
CC         CELL LEVEL AND HALF-SIZE
C          JLEVL = NLEVEL(JCELL)
C          HALFCZ = HALF*HCSIZE(JLEVL)
C
CC         CELL INTEGER COORDINATES
C          MX = NINT(STRN12(JCELL)/HCSIZE(JLEVL) - HALF)
C          MY = NINT(STRN13(JCELL)/HCSIZE(JLEVL) - HALF)
C          MZ = NINT(STRN23(JCELL)/HCSIZE(JLEVL) - HALF)
C
CC         ---------------------------------------------------------------------
C
CC         COMPUTE M-CODE FOR EACH VERTEX AND SEARCH THE LIST
C          DO KVERT = 1,8
C
C            JVERT = JVERT + 1
C            MVX = MX + MXINC(KVERT)
C            MVY = MY + MYINC(KVERT)
C            MVZ = MZ + MZINC(KVERT)
CC            CALL MCXYZC(MCRUN,JLEVL,MVX,MVY,MVZ)
C            CALL MCXYZC(MCRUN,JLEVL+1,MVX,MVY,MVZ)
C            KPOINT = 0
C            FPOINT = .FALSE.
C2000        CONTINUE
C              KPOINT = KPOINT + 1
C              IF(KPOINT.LE.JPOINT)THEN
C                CALL MCRTRV(MCCMP,MCLIST,NPTSMX,KPOINT)
C                CALL MCCOMP(MCRUN,MCCMP,MCOMPR)
C                IF(MCOMPR.EQ.0)THEN
C                  FPOINT = .TRUE.
C                ELSE
C                  GOTO 2000
C                ENDIF
C              ENDIF
CC           END OF LOOP 2000
C            IF(FPOINT)THEN
CC             POINT IS ALREADY IN THE LIST
C              ICNECT(JVERT) = KPOINT - 1
C            ELSE
CC             ADD POINT TO THE LIST
C              JPOINT = JPOINT + 1
C              CALL MCSTOR(MCRUN,MCLIST,NPTSMX,JPOINT)
C              ICNECT(JVERT) = JPOINT - 1
CC             COMPUTE POINT COORDINATES
C              POINTS(1,JPOINT) = STRN12(JCELL) + VSIGN(1,KVERT)*HALFCZ
C              POINTS(2,JPOINT) = STRN13(JCELL) + VSIGN(2,KVERT)*HALFCZ
C              POINTS(3,JPOINT) = STRN23(JCELL) + VSIGN(3,KVERT)*HALFCZ
C            ENDIF
C
C          ENDDO
CC         END OF VERTICES
C
C        ENDDO
CC       END OF CELLS
C        NPOINT = JPOINT
C        NVERT = JVERT

C       POINT COORDINATES
        JPOINT = 0
        JVERT  = 0
        DO JCELL = 1, NCELL

C         CELL LEVEL AND HALF-SIZE
          JLEVL = NLEVEL(JCELL)
          HALFCX = HALF*HCSIZE(JLEVL)
          HALFCY = HALFCX
          HALFCZ = HALFCX
C         RSC 29-APR-2014 MULTID
          IF(ZGPHYR.LT.CELTOL)THEN
            HALFCZ = HALF*HCSIZE(NBLEVL)
            IF(YGPHYR.LT.CELTOL)THEN
              HALFCY = HALFCZ
            ENDIF
          ENDIF

C         RUN THROUGH VERTICES
          DO KVERT = 1,8

            JVERT = JVERT + 1
            ICNECT(JVERT) = JPOINT

C           COMPUTE POINT COORDINATES
            JPOINT = JPOINT + 1
            POINTS(1,JPOINT) = STRN12(JCELL) + VSIGN(1,KVERT)*HALFCX
            POINTS(2,JPOINT) = STRN13(JCELL) + VSIGN(2,KVERT)*HALFCY
            POINTS(3,JPOINT) = STRN23(JCELL) + VSIGN(3,KVERT)*HALFCZ

          ENDDO
C         END OF VERTICES

        ENDDO
C       END OF CELLS
        NPOINT = JPOINT
        NVERT = JVERT

C       CONVERT TO PRIMITIVE VARIABLES
        DO JCELL = 1, NCELL
          URUN(1,JCELL) = URUN(1,JCELL)/DRUN(JCELL)
          URUN(2,JCELL) = URUN(2,JCELL)/DRUN(JCELL)
          URUN(3,JCELL) = URUN(3,JCELL)/DRUN(JCELL)
          ERUN(JCELL) = ERUN(JCELL)/DRUN(JCELL)
          DO JSPEC = 1, NSPEC
            YRUN(JSPEC,JCELL) = YRUN(JSPEC,JCELL)/DRUN(JCELL)
          ENDDO
        ENDDO

C       FILTER THE QUANTITIES
        DO JCELL = 1,NCELL
          IF(ABS(URUN(1,JCELL)).LT.UTOL)URUN(1,JCELL) = ZERO
          IF(ABS(URUN(2,JCELL)).LT.UTOL)URUN(2,JCELL) = ZERO
          IF(ABS(URUN(3,JCELL)).LT.UTOL)URUN(3,JCELL) = ZERO
          IF(ABS(PRUN(JCELL)).LT.PTOL)PRUN(JCELL) = ZERO
          DO JSPEC = 1, NSPEC
            IF(ABS(YRUN(JSPEC,JCELL)).LT.YTOL)YRUN(JSPEC,JCELL) = ZERO
          ENDDO
        ENDDO

C       WRITE THE VIEW FILE
        OPEN(UNIT=NCVIEW,FILE=FNVIEW,STATUS='NEW',FORM='FORMATTED')
        WRITE(NCVIEW,*)'<VTKFile type="UnstructuredGrid">'
        WRITE(NCVIEW,*)'  <UnstructuredGrid>'
        WRITE(NCVIEW,*)'    <Piece'
        WRITE(NCVIEW,*)'      NumberOfPoints="',NPOINT,'"'
        WRITE(NCVIEW,*)'      NumberOfCells="',NCELL,'"'
        WRITE(NCVIEW,*)'    >'
        WRITE(NCVIEW,*)'      <CellData'
        WRITE(NCVIEW,*)'        Vectors="Velocity"'
        WRITE(NCVIEW,*)'        Scalars="Density"'
C        WRITE(NCVIEW,*)'        Scalars="Energy"'
C        DO JSPEC = 1, NSPEC
C          WRITE(NCVIEW,'(A)')SPECSC(JSPEC)
C        ENDDO
        WRITE(NCVIEW,*)'      >'
        WRITE(NCVIEW,*)'        <DataArray'
        WRITE(NCVIEW,*)'          NumberOfComponents="3"'
        WRITE(NCVIEW,*)'          type="Float32"'
        WRITE(NCVIEW,*)'          Name="Velocity"'
        WRITE(NCVIEW,*)'          format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')
     +       (URUN(1,JCELL),URUN(2,JCELL),URUN(3,JCELL),JCELL = 1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray'
        WRITE(NCVIEW,*)'          type="Float32"'
        WRITE(NCVIEW,*)'          Name="Density"'
        WRITE(NCVIEW,*)'          format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')(DRUN(JCELL),JCELL = 1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray'
        WRITE(NCVIEW,*)'          type="Float32"'
        WRITE(NCVIEW,*)'          Name="Energy"'
        WRITE(NCVIEW,*)'          format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')(ERUN(JCELL),JCELL = 1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray'
        WRITE(NCVIEW,*)'          type="Float32"'
        WRITE(NCVIEW,*)'          Name="Temperature"'
        WRITE(NCVIEW,*)'          format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')(TRUN(JCELL),JCELL = 1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray'
        WRITE(NCVIEW,*)'          type="Float32"'
        WRITE(NCVIEW,*)'          Name="Pressure"'
        WRITE(NCVIEW,*)'          format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')(PRUN(JCELL),JCELL = 1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        DO JSPEC = 1, NSPEC
          WRITE(NCVIEW,*)'        <DataArray'
          WRITE(NCVIEW,*)'          type="Float32"'
          WRITE(NCVIEW,'(A)')SPECNM(JSPEC)
          WRITE(NCVIEW,*)'          format="ascii"'
          WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(9(1PE12.4))')(YRUN(JSPEC,JCELL),JCELL = 1,NCELL)
          WRITE(NCVIEW,*)'        </DataArray>'
        ENDDO
        WRITE(NCVIEW,*)'      </CellData>'
        WRITE(NCVIEW,*)'      <Points>'
        WRITE(NCVIEW,*)'        <DataArray NumberOfComponents="3"'
        WRITE(NCVIEW,*)'                   type="Float32"'
        WRITE(NCVIEW,*)'                   format="ascii"'
        WRITE(NCVIEW,*)'        >'
        DO JPOINT = 1, NPOINT
          WRITE(NCVIEW,'(3(1PE12.4))')(POINTS(JCOMP,JPOINT),JCOMP=1,3)
        ENDDO
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'      </Points>'
        WRITE(NCVIEW,*)'      <Cells>'
        WRITE(NCVIEW,*)'        <DataArray type="Int32"'
        WRITE(NCVIEW,*)'                   Name="connectivity"'
        WRITE(NCVIEW,*)'                   format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(8I8)')(ICNECT(JVERT),JVERT=1,NVERT)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray type="Int32"'
        WRITE(NCVIEW,*)'                   Name="offsets"'
        WRITE(NCVIEW,*)'                   format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(8I8)')(JCELL*8,JCELL=1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'        <DataArray type="UInt8"'
        WRITE(NCVIEW,*)'                   Name="types"'
        WRITE(NCVIEW,*)'                   format="ascii"'
        WRITE(NCVIEW,*)'        >'
        WRITE(NCVIEW,'(42I3)')(ICTYPE,JCELL=1,NCELL)
        WRITE(NCVIEW,*)'        </DataArray>'
        WRITE(NCVIEW,*)'      </Cells>'
        WRITE(NCVIEW,*)'    </Piece>'
        WRITE(NCVIEW,*)'  </UnstructuredGrid>'
        WRITE(NCVIEW,*)'</VTKFile>'

        CLOSE(NCVIEW)

C       =======================================================================

C       CHECK FOR LAST PASS
        IF(IPROC.LT.NRPRM1)THEN

C         ---------------------------------------------------------------------

          IPROC = IPROC + 1
          GOTO 1000

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ACTIONS ON LAST PASS
C         --------------------

C         WRITE THE FOOTER TO THE PARALLEL FILE
          WRITE(NCPVUE,'(A)')'  </PUnstructuredGrid>'
          WRITE(NCPVUE,'(A)')'</VTKFile>'

          CLOSE(NCPVUE)

C         ---------------------------------------------------------------------

        ENDIF
C       LAST PASS

C       =======================================================================

C     END OF LOOP 1000

C     =========================================================================

      STOP
      END
