      SUBROUTINE TURBIN
 
C     *************************************************************************
C
C     TURBIN
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     INITIALISES THE TURBULENT FIELD
C     USING THE DIFFUSION METHOD OF
C     A.KEMPF, A.KLEIN, J.JANICKA: FTAC 74, 67-84, 2005
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETER
C     =========
      DOUBLE PRECISION DCSTAB
      PARAMETER(DCSTAB = 1.5D-1)


C     FUNCTIONS
C     =========
      DOUBLE PRECISION RANUNT
      EXTERNAL RANUNT


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RCELNO(NPROMX)
      DOUBLE PRECISION UPMEAN(NPROMX),UPVARN(NPROMX)
      DOUBLE PRECISION ULMEAN(NCMPMX),UGMEAN(NCMPMX)
      DOUBLE PRECISION ULVARN(NCMPMX),UGVARN(NCMPMX),TKEVAL
      DOUBLE PRECISION ULCVAR(NCMPMX),UGCVAR(NCMPMX),ULVDEV(NCMPMX)
      DOUBLE PRECISION BANS11,BANS22,BANS33,BANS12,BANS13,BANS23
      DOUBLE PRECISION DDDELT,DLSCAL
      DOUBLE PRECISION RGCELN
      DOUBLE PRECISION FORNOW
      INTEGER NCELNO(NPROMX)
      INTEGER MAXLVL,MXGLVL
      INTEGER JCELL,JPROC,JLEVEL,IPROP1
      INTEGER JSTEPS,NSTEPS
      INTEGER ISEED


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE
C     ----------

C     TURBULENCE PARAMETERS
      UPRIME(1) = SQRT(SPARAM(1))
      UPRIME(2) = SQRT(SPARAM(2))
      UPRIME(3) = SQRT(SPARAM(3))
      DLSCAL = SPARAM(7)

C     CELL MAX LEVELS
      MAXLVL = 0
      MXGLVL = 0

C     PROCESSOR ID
      IPROP1 = IPROC + 1


C     GENERATE THE INITIAL RANDOM FIELD
C     ---------------------------------
      IF(ITSEED.GE.0)THEN

C       LOCAL INITIALISATION
C       --------------------
C       EFFICIENT, BUT DOES NOT ENSURE REPEATABILITY

C       RANDOM SEED MUST BE DIFFERENT FOR EVERY PROCESSOR
        ISEED = ITSEED + IPROC
        CALL RANINT(ISEED)

C       RUN THROUGH ALL LOCAL CELLS
        DO JCELL = 1, NCELL

C         SCALING FACTOR
          JLEVEL = NLEVEL(JCELL)
          FORNOW = OVSIZE(JLEVEL)
          FORNOW = SQRT(FORNOW*FORNOW*FORNOW)

C         SAVE THE MAX LEVEL
          MAXLVL = MAX(JLEVEL,MAXLVL)

C         GET AND SAVE THREE RANDOM RUMBERS PER CELL
          URUN(1,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(1,JCELL) = URUN(1,JCELL)*FORNOW
          URUN(2,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(2,JCELL) = URUN(2,JCELL)*FORNOW
          URUN(3,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(3,JCELL) = URUN(3,JCELL)*FORNOW
              
        ENDDO

      ELSE

C       GLOBAL INITIALISATION
C       ---------------------
C       ENSURES REPEATABILITY OF THE PROBLEM
C       WHEN USING DIFFERENT PROCESSOR COUNTS

C       RANDOM SEED MUST BE IDENTICAL FOR EVERY PROCESSOR
        ISEED = ITSEED
        CALL RANINT(ISEED)

C       SEND THE LOCAL CELL COUNT AND GATHER ALL THE OTHERS
        CALL P_GTHI(NCELL,1,NCELNO,NPROMX)

C       SWEEP THROUGH GLOBAL PHYSICAL SPACE
        DO JPROC = 1, IPROC

C         LOWER-RANKED PROCESSORS
          DO JCELL = 1, NCELNO(JPROC)

C           GET AND DISCARD THREE RANDOM NUMBERS PER CELL
            FORNOW = RANUNT(ISEED)
            FORNOW = RANUNT(ISEED)
            FORNOW = RANUNT(ISEED)

          ENDDO

        ENDDO

C       LOCAL PROCESSOR
        DO JCELL = 1, NCELL

C         SCALING FACTOR
          JLEVEL = NLEVEL(JCELL)
          FORNOW = OVSIZE(JLEVEL)
          FORNOW = SQRT(FORNOW*FORNOW*FORNOW)

C         SAVE THE MAX LEVEL
          MAXLVL = MAX(JLEVEL,MAXLVL)

C         GET AND SAVE THREE RANDOM RUMBERS PER CELL
          URUN(1,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(1,JCELL) = URUN(1,JCELL)*FORNOW
          URUN(2,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(2,JCELL) = URUN(2,JCELL)*FORNOW
          URUN(3,JCELL) = TWO*RANUNT(ISEED) - ONE
          URUN(3,JCELL) = URUN(3,JCELL)*FORNOW
              
        ENDDO

      ENDIF

C     =========================================================================

C     DIFFUSE
C     -------

C     FIND THE SMALLEST CELL (HIGHEST LEVEL) IN THE GLOBAL DOMAIN
      CALL P_GMXI(MAXLVL,MXGLVL)
      FORNOW = HCSIZE(MXGLVL)

C     DIFFUSION TIMESTEP
      DDDELT = DCSTAB*FORNOW*FORNOW

C     REQUIRED NUMBER OF TIMESTEPS
      FORNOW = TWO*PI*DDDELT
      NSTEPS = 1 + INT(DLSCAL*DLSCAL/FORNOW)

C     TIME STEPPING
      DO JSTEPS = 1, NSTEPS

C       PARALLEL TRANSFER
        CALL PARTRB

C       EVALUATE RHS
        CALL DIFTRB

C       UPDATE
        DO JCELL = 1, NCELL

          URUN(1,JCELL) = URUN(1,JCELL) + DCSTAB*URHS(1,JCELL)
          URUN(2,JCELL) = URUN(2,JCELL) + DCSTAB*URHS(2,JCELL)
          URUN(3,JCELL) = URUN(3,JCELL) + DCSTAB*URHS(3,JCELL)

        ENDDO

      ENDDO

C     =========================================================================

C     NORMALISE
C     ---------

C     -------------------------------------------------------------------------

C     REAL VALUES OF CELL COUNTS
      RGCELN = ZERO
      DO JPROC = 1, NPROC
        RCELNO(JPROC) = REAL(NCELNO(JPROC))
        RGCELN = RGCELN + RCELNO(JPROC)
      ENDDO

C     -------------------------------------------------------------------------

C     MEANS
C     -----

C     LOCAL MEANS
      ULMEAN(1) = ZERO
      ULMEAN(2) = ZERO
      ULMEAN(3) = ZERO
      DO JCELL = 1, NCELL

        ULMEAN(1) = ULMEAN(1) + URUN(1,JCELL)
        ULMEAN(2) = ULMEAN(2) + URUN(2,JCELL)
        ULMEAN(3) = ULMEAN(3) + URUN(3,JCELL)

      ENDDO
      ULMEAN(1) = ULMEAN(1)/RCELNO(IPROP1)
      ULMEAN(2) = ULMEAN(2)/RCELNO(IPROP1)
      ULMEAN(3) = ULMEAN(3)/RCELNO(IPROP1)

C     GLOBAL MEANS
      CALL P_GTHR(ULMEAN(1),1,UPMEAN,NPROMX)
      UGMEAN(1) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(1) = UGMEAN(1) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(1) = UGMEAN(1)/RGCELN

      CALL P_GTHR(ULMEAN(2),1,UPMEAN,NPROMX)
      UGMEAN(2) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(2) = UGMEAN(2) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(2) = UGMEAN(2)/RGCELN

      CALL P_GTHR(ULMEAN(3),1,UPMEAN,NPROMX)
      UGMEAN(3) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(3) = UGMEAN(3) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(3) = UGMEAN(3)/RGCELN

C     ADJUST THE GLOBAL MEAN TO ZERO
      DO JCELL = 1, NCELL

        URUN(1,JCELL) = URUN(1,JCELL) - UGMEAN(1)
        URUN(2,JCELL) = URUN(2,JCELL) - UGMEAN(2)
        URUN(3,JCELL) = URUN(3,JCELL) - UGMEAN(3)

      ENDDO

C     -------------------------------------------------------------------------

C     VARIANCES
C     ---------

C     LOCAL VARIANCES
      ULVARN(1) = ZERO
      ULVARN(2) = ZERO
      ULVARN(3) = ZERO
      DO JCELL = 1, NCELL

        ULVARN(1) = ULVARN(1) + URUN(1,JCELL)*URUN(1,JCELL)
        ULVARN(2) = ULVARN(2) + URUN(2,JCELL)*URUN(2,JCELL)
        ULVARN(3) = ULVARN(3) + URUN(3,JCELL)*URUN(3,JCELL)

      ENDDO
      ULVARN(1) = ULVARN(1)/RCELNO(IPROP1)
      ULVARN(2) = ULVARN(2)/RCELNO(IPROP1)
      ULVARN(3) = ULVARN(3)/RCELNO(IPROP1)

C     GLOBAL VARIANCES
      CALL P_GTHR(ULVARN(1),1,UPVARN,NPROMX)
      UGVARN(1) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(1) = UGVARN(1) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(1) = UGVARN(1)/RGCELN

      CALL P_GTHR(ULVARN(2),1,UPVARN,NPROMX)
      UGVARN(2) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(2) = UGVARN(2) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(2) = UGVARN(2)/RGCELN

      CALL P_GTHR(ULVARN(3),1,UPVARN,NPROMX)
      UGVARN(3) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(3) = UGVARN(3) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(3) = UGVARN(3)/RGCELN

C     ADJUST THE GLOBAL VARIANCES TO UNITY
      DO JCELL = 1, NCELL

        URUN(1,JCELL) = URUN(1,JCELL)/SQRT(UGVARN(1))
        URUN(2,JCELL) = URUN(2,JCELL)/SQRT(UGVARN(2))
        URUN(3,JCELL) = URUN(3,JCELL)/SQRT(UGVARN(3))

      ENDDO

C     =========================================================================

C     APPLY REYNOLDS STRESSES
C     -----------------------

C     NORMAL STRESSES ONLY FOR NOW
      DO JCELL = 1, NCELL

        URUN(1,JCELL) = URUN(1,JCELL)*UPRIME(1)
        URUN(2,JCELL) = URUN(2,JCELL)*UPRIME(2)
        URUN(3,JCELL) = URUN(3,JCELL)*UPRIME(3)

      ENDDO

C     =========================================================================

C     RECOMPUTE MEANS AND VARIANCES TO CHECK
C     --------------------------------------

C     LOCAL MEANS
      ULMEAN(1) = ZERO
      ULMEAN(2) = ZERO
      ULMEAN(3) = ZERO
      DO JCELL = 1, NCELL

        ULMEAN(1) = ULMEAN(1) + URUN(1,JCELL)
        ULMEAN(2) = ULMEAN(2) + URUN(2,JCELL)
        ULMEAN(3) = ULMEAN(3) + URUN(3,JCELL)

      ENDDO
      ULMEAN(1) = ULMEAN(1)/RCELNO(IPROP1)
      ULMEAN(2) = ULMEAN(2)/RCELNO(IPROP1)
      ULMEAN(3) = ULMEAN(3)/RCELNO(IPROP1)

C     GLOBAL MEANS
      CALL P_GTHR(ULMEAN(1),1,UPMEAN,NPROMX)
      UGMEAN(1) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(1) = UGMEAN(1) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(1) = UGMEAN(1)/RGCELN

      CALL P_GTHR(ULMEAN(2),1,UPMEAN,NPROMX)
      UGMEAN(2) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(2) = UGMEAN(2) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(2) = UGMEAN(2)/RGCELN

      CALL P_GTHR(ULMEAN(3),1,UPMEAN,NPROMX)
      UGMEAN(3) = ZERO
      DO JPROC = 1, NPROC
        UGMEAN(3) = UGMEAN(3) + UPMEAN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGMEAN(3) = UGMEAN(3)/RGCELN


C     LOCAL VARIANCES
      ULVARN(1) = ZERO
      ULVARN(2) = ZERO
      ULVARN(3) = ZERO
      DO JCELL = 1, NCELL

        FORNOW = URUN(1,JCELL) - UGMEAN(1)
        ULVARN(1) = ULVARN(1) + FORNOW*FORNOW
        FORNOW = URUN(2,JCELL) - UGMEAN(2)
        ULVARN(2) = ULVARN(2) + FORNOW*FORNOW
        FORNOW = URUN(3,JCELL) - UGMEAN(3)
        ULVARN(3) = ULVARN(3) + FORNOW*FORNOW

      ENDDO
      ULVARN(1) = ULVARN(1)/RCELNO(IPROP1)
      ULVARN(2) = ULVARN(2)/RCELNO(IPROP1)
      ULVARN(3) = ULVARN(3)/RCELNO(IPROP1)

C     GLOBAL VARIANCES
      CALL P_GTHR(ULVARN(1),1,UPVARN,NPROMX)
      UGVARN(1) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(1) = UGVARN(1) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(1) = UGVARN(1)/RGCELN

      CALL P_GTHR(ULVARN(2),1,UPVARN,NPROMX)
      UGVARN(2) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(2) = UGVARN(2) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(2) = UGVARN(2)/RGCELN

      CALL P_GTHR(ULVARN(3),1,UPVARN,NPROMX)
      UGVARN(3) = ZERO
      DO JPROC = 1, NPROC
        UGVARN(3) = UGVARN(3) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGVARN(3) = UGVARN(3)/RGCELN


C     LOCAL COVARIANCES
      ULCVAR(1) = ZERO
      ULCVAR(2) = ZERO
      ULCVAR(3) = ZERO
      DO JCELL = 1, NCELL

        ULVDEV(1) = URUN(1,JCELL) - UGMEAN(1)
        ULVDEV(2) = URUN(2,JCELL) - UGMEAN(2)
        ULVDEV(3) = URUN(3,JCELL) - UGMEAN(3)
        ULCVAR(1) = ULCVAR(1) + ULVDEV(1)*ULVDEV(2)
        ULCVAR(2) = ULCVAR(2) + ULVDEV(1)*ULVDEV(3)
        ULCVAR(3) = ULCVAR(3) + ULVDEV(2)*ULVDEV(3)

      ENDDO
      ULCVAR(1) = ULCVAR(1)/RCELNO(IPROP1)
      ULCVAR(2) = ULCVAR(2)/RCELNO(IPROP1)
      ULCVAR(3) = ULCVAR(3)/RCELNO(IPROP1)

C     GLOBAL VARIANCES
      CALL P_GTHR(ULCVAR(1),1,UPVARN,NPROMX)
      UGCVAR(1) = ZERO
      DO JPROC = 1, NPROC
        UGCVAR(1) = UGCVAR(1) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGCVAR(1) = UGCVAR(1)/RGCELN

      CALL P_GTHR(ULCVAR(2),1,UPVARN,NPROMX)
      UGCVAR(2) = ZERO
      DO JPROC = 1, NPROC
        UGCVAR(2) = UGCVAR(2) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGCVAR(2) = UGCVAR(2)/RGCELN

      CALL P_GTHR(ULCVAR(3),1,UPVARN,NPROMX)
      UGCVAR(3) = ZERO
      DO JPROC = 1, NPROC
        UGCVAR(3) = UGCVAR(3) + UPVARN(JPROC)*RCELNO(JPROC)
      ENDDO
      UGCVAR(3) = UGCVAR(3)/RGCELN


C     TURBULENCE KINETIC ENERGY
C     -------------------------
      FORNOW = UGVARN(1) + UGVARN(2) + UGVARN(3)
      TKEVAL = HALF*FORNOW

C     ANISOTROPY
C     ----------
      BANS11 = UGVARN(1)/FORNOW - THRD
      BANS22 = UGVARN(2)/FORNOW - THRD
      BANS33 = UGVARN(3)/FORNOW - THRD
      BANS12 = UGCVAR(1)/FORNOW
      BANS13 = UGCVAR(2)/FORNOW
      BANS23 = UGCVAR(3)/FORNOW

C     =========================================================================

C     INITIAL TURBULENCE REPORT
C     -------------------------

C     WRITE TO REPORT FILE
      IF(IPROC.EQ.0)THEN

        OPEN(UNIT=NCREPT,FILE=FNREPT,STATUS='OLD',FORM='FORMATTED')

C       GO TO EOF
1000    CONTINUE
          READ(NCREPT,9000,END=1010)
          GOTO 1000
1010    BACKSPACE(NCREPT)

        WRITE(NCREPT,*)'Turbulence field initialisation report'
        WRITE(NCREPT,*)'--------------------------------------'
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Number of diffusion steps:'
        WRITE(NCREPT,'(I7)')NSTEPS
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Velocity means:'
        WRITE(NCREPT,'(3(1PE15.7))')UGMEAN(1),UGMEAN(2),UGMEAN(3)
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Velocity variances:'
        WRITE(NCREPT,'(3(1PE15.7))')UGVARN(1),UGVARN(2),UGVARN(3)
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Velocity covariances:'
        WRITE(NCREPT,'(3(1PE15.7))')UGCVAR(1),UGCVAR(2),UGCVAR(3)
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Turbulence kinetic energy; length scale'
        WRITE(NCREPT,'(2(1PE15.7))')TKEVAL,DLSCAL
        WRITE(NCREPT,*)

        WRITE(NCREPT,*)'Anisotropy'
        WRITE(NCREPT,'(6(1PE12.4))')BANS11,BANS22,BANS33,
     +                              BANS12,BANS13,BANS23
        WRITE(NCREPT,*)

        WRITE(NCREPT,9900)CLDASH
        WRITE(NCREPT,*)

        CLOSE(NCREPT)

      ENDIF

C     =========================================================================

      RETURN

9000  FORMAT(A)
9900  FORMAT(A79)

      END
