      SUBROUTINE MCCXYZ(MCODE,NLEVEL,MX,MY,MZ)
 
C     *************************************************************************
C
C     MCCXYZ
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     25-JUN-2008:  CREATED
C     05-MAY-2011:  REFORMULATED FOR IMPROVED PERFORMANCE
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     MORTON CODE MANIPULATION
C     EXTRACTS XYZ COORDINATES FROM A MORTON CODE AT THE SPECIFIED LEVEL
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_mcmlib.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER MCODE(MCSIZE)
      INTEGER NLEVEL,MX,MY,MZ


C     LOCAL DATA
C     ==========
      INTEGER MLOCL(MCBITS)
      INTEGER NPBITS(MCSIZE)
      INTEGER MCCRUN
      INTEGER NLEVM1,MCSLVL,MCSLM1
      INTEGER ICSIZE,ICBITS,IPBITS,IPOWER


C     BEGIN
C     =====

C     =========================================================================

C     INITIALISE
C     ----------
      NLEVM1 = NLEVEL - 1
      MCSLM1 =  NLEVM1/NLOCTS
      DO ICSIZE = 1, MCSLM1
        NPBITS(ICSIZE) = NLBITS
      ENDDO
      MCSLVL =  MCSLM1 + 1
      NPBITS(MCSLVL) = (NLEVEL - MCSLM1*NLOCTS)*3

C     =========================================================================

C     CONVERT MORTON CODE TO BIT STRING
C     ---------------------------------
C     UP TO REQUIRED LEVEL ONLY
      ICBITS = 0
      DO ICSIZE = 1, MCSLVL
        MCCRUN = MCODE(ICSIZE)
        ICBITS = ICBITS + 1
        MLOCL(ICBITS) = MOD(MCCRUN,2)
        DO IPBITS = 2, NPBITS(ICSIZE)
          MCCRUN = MCCRUN/2
          ICBITS = ICBITS + 1
          MLOCL(ICBITS) = MOD(MCCRUN,2)
        ENDDO
      ENDDO

C     =========================================================================

C     EXTRACT XYZ INTEGERS FROM BIT STRING
C     ------------------------------------
      MX = 0
      MY = 0
      MZ = 0
      ICBITS = 0
      DO IPBITS = 1, NLEVEL
        IPOWER = NLEVEL - IPBITS
        ICBITS = ICBITS + 1
        MX = MX + MLOCL(ICBITS)*KCPWR2(IPOWER)
        ICBITS = ICBITS + 1
        MY = MY + MLOCL(ICBITS)*KCPWR2(IPOWER)
        ICBITS = ICBITS + 1
        MZ = MZ + MLOCL(ICBITS)*KCPWR2(IPOWER)
      ENDDO

C     =========================================================================


      RETURN
      END
