      SUBROUTINE PARBAL
 
C     *************************************************************************
C
C     PARBAL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     11-JUN-2008:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     CARRIES OUT PARALLEL LOAD-BALANCING
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
C     ==========
      INTEGER MCMSIZ,MPARAY,MDARAY
      PARAMETER(MCMSIZ = 1000, MPARAY = 2000, MDARAY = 10000)
      INTEGER MPSNGL,NOTAG
      PARAMETER(MPSNGL = 1,NOTAG = 0)
      DOUBLE PRECISION GIMTOL,GIRTOL
      PARAMETER(GIMTOL = 1.0D-2, GIRTOL = 1.0D-6)
      INTEGER NGIMAX
      PARAMETER(NGIMAX = 1000)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION DATALO(MDARAY),DATAHI(MDARAY)
      DOUBLE PRECISION GIMBAL,GIMBAV,GIMMAX,GIMOLD
      INTEGER MCMLO(NINDMX,MCMSIZ),MCMHI(NINDMX,MCMSIZ)
      INTEGER MCRUN(NINDMX)
      INTEGER NCELLO,NCELHI,NDIFLO,NDIFHI,NMOVLO,NMOVHI
      INTEGER NRCVLO,NRCVHI,NSNDLO,NSNDHI
      INTEGER IDEST
      INTEGER ICOUNT,JOCTNT,JLEVEL,NSTART,NNCOP1,NEND
      INTEGER NDATLO,NDATHI


C     DIAGNOSTICS
      INTEGER LPROC


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     TAKE ACTION ONLY IF THERE IS MORE THAN ONE PROCESSOR
      IF(NPROC.GT.1)THEN

C       =======================================================================

C       DIAGNOSTICS
        IF(IPROC.EQ.LPROC)THEN
          WRITE(6,*)'In PARBAL',IPROC,ITIME,NNCODE
          DO JOCTNT = 1, NNCODE
            CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JOCTNT)
            CALL MCRITE(MCRUN)
          ENDDO
        ENDIF

        GIMOLD = ZERO

        ICOUNT = 1
1000    CONTINUE

C         UPDATE NEAREST-NEIGHBOUR INFORMATION
C         ------------------------------------

C         INITIALISE COUNTERS
          NCELLO = 0
          NCELHI = 0
          NDIFLO = 0
          NDIFHI = 0
          NMOVLO = 0
          NMOVHI = 0
          NSNDLO = 0
          NSNDHI = 0
          NDATLO = 0
          NDATHI = 0

          IF(MOD(IPROC,2).EQ.0)THEN

C           EVEN PROCESSORS
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              CALL P_RCVI(NCELHI,MPSNGL,IDEST,NOTAG)
              NDIFHI = NNCODE - NCELHI
            ENDIF
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              CALL P_RCVI(NCELLO,MPSNGL,IDEST,NOTAG)
              NDIFLO = NNCODE - NCELLO
            ENDIF

          ELSE

C           ODD PROCESSORS
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              CALL P_RCVI(NCELLO,MPSNGL,IDEST,NOTAG)
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              NDIFLO = NNCODE - NCELLO
            ENDIF
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              CALL P_RCVI(NCELHI,MPSNGL,IDEST,NOTAG)
              CALL P_SNDI(NNCODE,MPSNGL,MPSNGL,IDEST,NOTAG)
              NDIFHI = NNCODE - NCELHI
            ENDIF

          ENDIF

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,*)'PARBAL',IPROC,ICOUNT,NDIFLO,NDIFHI
          ENDIF

C         =====================================================================

C         BUFFER UP M-CODES AND DATA
C         --------------------------
C         NOTE THAT A POSITIVE VALUE OF NDIFLO OR NDIFHI
C         INDICATES AN EXCESS ON THE LOCAL PROCESSOR RELATIVE TO ITS NEIGHBOUR
          NNCOP1 = NNCODE + 1

C         HIGH END

C         NUMBER OF M-CODES TO MOVE
C         MAX OF HALF THE DIFFERENCE
          NMOVHI = NDIFHI/2

          IF(NMOVHI.GT.0)THEN

C           ENSURE THAT LOWEST TRANSFERRED M-CODE LIES ON AN OCTANT BOUNDARY
C           FOR ALL LEVELS EXCEPT THE BASE LEVEL
2000        CONTINUE
              NSTART = NNCOP1 - NMOVHI
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NSTART)
              CALL MCGLVL(MCRUN,JLEVEL)
              IF(JLEVEL.GT.NBLEVL)THEN
                CALL MCGOCR(MCRUN,JOCTNT)
                IF(JOCTNT.NE.0)THEN
                  NMOVHI = NMOVHI - 1
                  IF(NMOVHI.GT.0)GOTO 2000
                ENDIF
              ENDIF
C           END OF LOOP 2000
            NMOVHI = MAX(NMOVHI,0)

C           BUFFER UP M-CODES
            CALL MCMTRV(MCMHI,MCMSIZ,NMCODE,NSIZMX,NSTART,NNCODE)
            NNCODE = NNCODE - NMOVHI
            NSNDHI = 2*NMOVHI

C           BUFFER UP DATA
            NEND = 1
            CALL DBUFUP(DATAHI,MDARAY,NSTART,NCELL,NDATHI)
            NCELL = NCELL - NMOVHI

          ENDIF

C         LOW END

C         NUMBER OF M-CODES TO MOVE
C         MAX OF HALF THE DIFFERENCE
          NMOVLO = NDIFLO/2

          IF(NMOVLO.GT.0)THEN

C           ENSURE THAT HIGHEST TRANSFERRED M-CODE LIES ON AN OCTANT BOUNDARY
3000        CONTINUE
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NMOVLO)
              CALL MCGLVL(MCRUN,JLEVEL)
C             DIAGNOSTICS
              IF(IPROC.EQ.LPROC)WRITE(6,*)'PB LE',NDIFLO,NMOVLO,JLEVEL
              IF(JLEVEL.GT.NBLEVL)THEN
                CALL MCGOCR(MCRUN,JOCTNT)
C               DIAGNOSTICS
                IF(IPROC.EQ.LPROC)WRITE(6,*)'PB LE',JOCTNT
                IF(JOCTNT.NE.NOFSET)THEN
                  NMOVLO = NMOVLO - 1
                  IF(NMOVLO.GT.0)GOTO 3000
                ENDIF
              ENDIF
C           END OF LOOP 3000
            NMOVLO = MAX(NMOVLO,0)

C           BUFFER UP M-CODES
            NSTART = 1
            CALL MCMTRV(MCMLO,MCMSIZ,NMCODE,NSIZMX,NSTART,NMOVLO)
            NSNDLO = 2*NMOVLO

C           SHIFT M-CODES LEFT
            NEND = -1
            CALL MCSHFT(NMCODE,NSIZMX,NNCODE,NMOVLO,NEND)
            NNCODE = NNCODE - NMOVLO

C           BUFFER UP DATA
            CALL DBUFUP(DATALO,MDARAY,NSTART,NMOVLO,NDATLO)

C           SHIFT DATA LEFT
            CALL DSHIFT(NCELL,NMOVLO,NEND)
            NCELL = NCELL - NMOVLO

          ENDIF

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("PARBAL",8I5)')IPROC,ICOUNT,NMOVLO,NMOVHI,
     +                               NSNDLO,NSNDHI,NDATLO,NDATHI
          ENDIF

C         =====================================================================

C         MOVE M-CODES AND DATA
C         ---------------------
          NRCVLO = 0
          NRCVHI = 0

          IF(MOD(IPROC,2).EQ.0)THEN

C           EVEN PROCESSORS
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              IF(NDIFHI.GT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ERS",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NSNDHI,NDATHI
                CALL P_SNDI(NMOVHI,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMHI, MPARAY,NSNDHI,IDEST,NOTAG)
                CALL P_SEND(DATAHI,MDARAY,NDATHI,IDEST,NOTAG)
              ELSEIF(NDIFHI.LT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ERR",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NRCVHI
                CALL P_RCVI(NRCVHI,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMHI, MPARAY,IDEST,NOTAG)
                CALL P_RECV(DATAHI,MDARAY,IDEST,NOTAG)
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ERR",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NRCVHI
              ENDIF
            ENDIF
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              IF(NDIFLO.GT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ELS",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                       NSNDLO,NDATLO
                CALL P_SNDI(NMOVLO,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMLO, MPARAY,NSNDLO,IDEST,NOTAG)
                CALL P_SEND(DATALO,MDARAY,NDATLO,IDEST,NOTAG)
              ELSEIF(NDIFLO.LT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ELR",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                      NRCVLO
                CALL P_RCVI(NRCVLO,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMLO, MPARAY,IDEST,NOTAG)
                CALL P_RECV(DATALO,MDARAY,IDEST,NOTAG)
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ELR",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                      NRCVLO
              ENDIF
            ENDIF

          ELSE

C           ODD PROCESSORS
C           EXCHANGE TO THE LEFT
            IF(IPROC.GT.0)THEN
              IDEST = IPROC-1
              IF(NDIFLO.LT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_OLR",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                       NRCVLO
                CALL P_RCVI(NRCVLO,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMLO, MPARAY,IDEST,NOTAG)
                CALL P_RECV(DATALO,MDARAY,IDEST,NOTAG)
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_OLR",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                       NRCVLO
              ELSEIF(NDIFLO.GT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_OLS",5I5)')IPROC,ICOUNT,NMOVLO,
C     +                                       NSNDLO,NDATLO
                CALL P_SNDI(NMOVLO,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMLO, MPARAY,NSNDLO,IDEST,NOTAG)
                CALL P_SEND(DATALO,MDARAY,NDATLO,IDEST,NOTAG)
              ENDIF
            ENDIF
C           EXCHANGE TO THE RIGHT
            IF(IPROC.LT.NPRM1)THEN
              IDEST = IPROC+1
              IF(NDIFHI.LT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ORR",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NRCVHI
                CALL P_RCVI(NRCVHI,MPSNGL,IDEST,NOTAG)
                CALL P_RCVI(MCMHI, MPARAY,IDEST,NOTAG)
                CALL P_RECV(DATAHI,MDARAY,IDEST,NOTAG)
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ORR",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NRCVHI
              ELSEIF(NDIFHI.GT.0)THEN
CC               DIAGNOSTICS
C                WRITE(6,'("PARBAL_ORS",5I5)')IPROC,ICOUNT,NMOVHI,
C     +                                       NSNDHI,NDATHI
                CALL P_SNDI(NMOVHI,MPSNGL,MPSNGL,IDEST,NOTAG)
                CALL P_SNDI(MCMHI, MPARAY,NSNDHI,IDEST,NOTAG)
                CALL P_SEND(DATAHI,MDARAY,NDATHI,IDEST,NOTAG)
              ENDIF
            ENDIF

          ENDIF

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            WRITE(6,'("PARBAL",4I5)')IPROC,ICOUNT,NRCVLO,NRCVHI
          ENDIF

C         =====================================================================

C         UNBUFFER M-CODES AND DATA
C         -------------------------
          IF(NRCVLO.GT.0)THEN

C           SHIFT M-CODES RIGHT
            NEND = 1
            CALL MCSHFT(NMCODE,NSIZMX,NNCODE,NRCVLO,NEND)
            NNCODE = NNCODE + NRCVLO

C           FILL LOW END WITH M-CODES
            NSTART = 1
            CALL MCMTOR(MCMLO,MCMSIZ,NMCODE,NSIZMX,NSTART,NRCVLO)

C           SHIFT DATA RIGHT
            CALL DSHIFT(NCELL,NRCVLO,NEND)
            NCELL = NCELL + NRCVLO

C           FILL LOW END WITH DATA
            CALL DUNBUF(DATALO,MDARAY,NSTART,NRCVLO)

          ENDIF

          IF(NRCVHI.GT.0)THEN

C           FILL HIGH END WITH M-CODES
            NSTART = NNCODE + 1
            NNCODE = NNCODE + NRCVHI
            CALL MCMTOR(MCMHI,MCMSIZ,NMCODE,NSIZMX,NSTART,NNCODE)

C           FILL HIGH END WITH DATA
            NSTART = NCELL + 1
            NCELL = NCELL + NRCVHI
            CALL DUNBUF(DATAHI,MDARAY,NSTART,NCELL)

          ENDIF

C         DIAGNOSTICS
          IF(IPROC.EQ.LPROC)THEN
            DO JOCTNT = 1, NNCODE
              CALL MCRTRV(MCRUN,NMCODE,NSIZMX,JOCTNT)
              CALL MCRITE(MCRUN)
            ENDDO
          ENDIF

C         =====================================================================

C         CHECK CONVERGENCE AND DO IT AGAIN
C         ---------------------------------
          GIMBAL = REAL(NNCODE)
          GIMBAV = GIMBAL*OVPROC
          CALL P_SUMM(GIMBAV,GIMMAX)
          GIMBAL = ABS(GIMBAL-GIMMAX)/GIMMAX
          CALL P_SUMM(GIMBAL,GIMMAX)

          IF(GIMMAX.GT.GIMTOL)THEN
            IF(ABS(GIMMAX-GIMOLD).GT.GIRTOL)THEN
              IF(ICOUNT.LT.NGIMAX)THEN
                GIMOLD = GIMMAX
                ICOUNT = ICOUNT + 1
                GOTO 1000
              ELSE
C               ERROR CONDITION - load balancing failed to converge
                CALL ERHAND(18,20,2)
              ENDIF
            ENDIF
          ENDIF

C       END OF LOOP 1000

C       =======================================================================

C       REBUILD THE PARTITION INTERVAL TABLE
C       ------------------------------------
        CALL MCRTRV(MCRUN,NMCODE,NSIZMX,NNCODE)
        CALL PTABLE(MCRUN)

C       =======================================================================

      ENDIF

C     =========================================================================

C     GARBAGE COLLECTION FOLLOWING LOAD-BALANCING
C     ------------------

C     REBUILD THE LOCAL OCTREE
      CALL RETREE
      CALL RELEAF

C     =========================================================================


      RETURN
      END
