      SUBROUTINE PARCEL(NACODE)
 
C     *************************************************************************
C
C     PARCEL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     27-JUL-2008:  CREATED
C     29-AUG-2014:  RSC MODIFIED FOR WENO
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     EXCHANGES HALO CELL DATA BETWEEN PROCESSORS
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER NACODE(NINDMX,NREMAX)


C     LOCAL DATA
C     ==========
      INTEGER NARECV(NINDMX,NREMAX)
      INTEGER MCRUN(NINDMX),MCNXT(NINDMX)
      INTEGER NMSEND(NPROMX),NMRECV(NPROMX)
      INTEGER NDSEND(NPROMX),NDRECV(NPROMX)
      INTEGER NXSEND
      INTEGER JRCELL,JRLEVL,JRLIST,KRCELL,KRLEVL
      INTEGER JPROC,JPROP1,NMCOMP,JRPROC
      CHARACTER*1 CFOUND

C     DIAGNOSTICS
      INTEGER LPROC

C     =========================================================================

C     Arguments:
C     NACODE contains the M-code of each requested cell
C
C     Global data:
C     NRRLOC local-cell indirection list: contains the cell ID numbers on the
C            local processor for all the cells requested by other processors
C     NRRCEL total number of local cells requested by other processors
C     NRCLOC halo-cell indirection list: contains the cell ID numbers on the
C            local processor for the halo cells requested from other processors
C     NRCELL total number of halo cells requested from other processors
C     N2SEND number of cells requested from each processor
C     N2RECV number of cells requested by each processor
C     NCSEND cumulative total of cells requested from each processor
C     NCRECV cumulative total of cells requested by each processor
C
C     =========================================================================


C     BEGIN
C     =====

C     DIAGNOSTICS
      LPROC = 20

C     =========================================================================

C     GET THE ATTRIBUTES OF THE REMOTE CELLS
C     --------------------------------------

C     PARTITION THE LIST OF REMOTE CELLS
      DO JPROP1 = 1, NPROC
        N2SEND(JPROP1) = 0
        N2RECV(JPROP1) = 0
      ENDDO

      JPROC = 0
      JPROP1 = 1
      CALL MCRTBL(MCNXT,NTABLE,NPROMX,JPROC)
      CALL MCGLVL(MCNXT,KRLEVL)

      DO JRCELL = 1,NRCELL

        CALL MCRTRV(MCRUN,NACODE,NREMAX,JRCELL)
1000    CONTINUE
          CALL MCCOML(MCRUN,MCNXT,KRLEVL,NMCOMP)
          IF(NMCOMP.GT.0)THEN
            JPROC = JPROP1
            JPROP1 = JPROP1 + 1
            CALL MCRTBL(MCNXT,NTABLE,NPROMX,JPROC)
            CALL MCGLVL(MCNXT,KRLEVL)
            GOTO 1000
          ENDIF
C       END OF LOOP 1000
        N2SEND(JPROP1) = N2SEND(JPROP1) + 1

      ENDDO

C     -------------------------------------------------------------------------

C     EXCHANGE NUMBERS OF CELLS
      CALL P_ATOI(N2SEND,NPROMX,1,N2RECV,NPROMX,1)

C     TOTAL NUMBER OF LOCAL CELLS REQUESTED
      NRRCEL = 0
      DO JPROP1 = 1,NPROC
        NRRCEL = NRRCEL + N2RECV(JPROP1)
      ENDDO

C     -------------------------------------------------------------------------

C     EXCHANGE M-CODES
      NMSEND(1) = 2*N2SEND(1)
      NMRECV(1) = 2*N2RECV(1)
      NDSEND(1) = 0
      NDRECV(1) = 0
      DO JPROP1 = 2,NPROC
        JPROC = JPROP1 - 1
        NMSEND(JPROP1) = 2*N2SEND(JPROP1)
        NMRECV(JPROP1) = 2*N2RECV(JPROP1)
        NDSEND(JPROP1) = NDSEND(JPROC) + NMSEND(JPROC)
        NDRECV(JPROP1) = NDRECV(JPROC) + NMRECV(JPROC)
      ENDDO

      CALL P_A2IV(NACODE,NREMAX,NMSEND,NDSEND,
     +            NARECV,NREMAX,NMRECV,NDRECV,NPROC)

C     -------------------------------------------------------------------------

C     INITIALISE PROCESSOR NUMBER FOR EACH RECEIVED CELL
      JPROP1 = 1
      JRPROC = N2RECV(1)

C     INITIALISE COUNTER FOR EXTRA HALO CELL LIST
      JRLIST = 0

C     RUN THROUGH ALL RECEIVED M-CODES
      DO JRCELL = 1, NRRCEL

C       GET PROCESSOR NUMBER FOR EACH RECEIVED CELL
2000    CONTINUE
          IF(JRCELL.GT.JRPROC)THEN
            JPROP1 = JPROP1 + 1
            JRPROC = JRPROC + N2RECV(JPROP1)
            GOTO 2000
          ENDIF
C       END OF LOOP 2000

C       FETCH THE M-CODE
        CALL MCRTRV(MCRUN,NARECV,NREMAX,JRCELL)

C       GET THE LEVEL
        CALL MCGLVL(MCRUN,JRLEVL)

C       SEARCH FOR THE CORRESPONDING CELL
        CALL OCFIND(NOCHLD,COCODE,NOCTMX,MCRUN,KRLEVL,KRCELL,CFOUND)

C       CHECK FINDINGS
        IF(CFOUND.EQ.CHTWIG)THEN

C         ---------------------------------------------------------------------

C         CELL IS LOCAL
C         -------------
C         CHECK THE LEVEL
          IF(KRLEVL.EQ.JRLEVL)THEN

C           CLEAN FACE
C           ----------
C           SAVE THE CELL ID
            JRLIST = JRLIST + 1
            NRRLOC(JRLIST) = KRCELL

          ELSE

C           ERROR CONDITION - received neighbour cell not found'
C           ---------------
C           DIAGNOSTICS
            IF(IPROC.EQ.NPROC)THEN
              WRITE(6,*)"PARCEL rn",IPROC,JRCELL,JRLEVL,KRCELL,KRLEVL
              CALL MCRITE(MCRUN)
            ENDIF
            CALL ERHAND(3,11,4)

          ENDIF

C         ---------------------------------------------------------------------

        ELSE

C         ---------------------------------------------------------------------

C         ERROR CONDITION - received cell not found
C         ---------------
C         DIAGNOSTICS
          IF(IPROC.EQ.NPROC)THEN
            WRITE(6,*)"PARCEL rc",IPROC,JRCELL,JRLEVL
            CALL MCRITE(MCRUN)
          ENDIF
          CALL ERHAND(3,13,4)

C         ---------------------------------------------------------------------

        ENDIF
C       END OF CHECK FINDINGS

C       -----------------------------------------------------------------------

      ENDDO
C     END OF RUN THROUGH ALL RECEIVED M-CODES

C     =========================================================================

C     NOTE THAT FOR PARALLEL TRANSFER OF STENCIL DATA
C     LOCAL CELL DATA IS *SENT*
C     HALO CELL DATA IS *RECEIVED*

C     UPDATE PARALLEL TRANSFER PARTITION ARRAYS
      NXSEND = N2SEND(1)
      N2SEND(1) = N2RECV(1)
      N2RECV(1) = NXSEND
      NCSEND(1) = 0
      NCRECV(1) = 0
      DO JPROP1 = 2, NPROC
        NXSEND = N2SEND(JPROP1)
        N2SEND(JPROP1) = N2RECV(JPROP1)
        N2RECV(JPROP1) = NXSEND
        JPROC = JPROP1 - 1
        NCSEND(JPROP1) = NCSEND(JPROC) + N2SEND(JPROC)
        NCRECV(JPROP1) = NCRECV(JPROC) + N2RECV(JPROC)
      ENDDO

C     =========================================================================


      RETURN
      END
