      SUBROUTINE REFCEL

C     *************************************************************************
C
C     REFCEL
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     30-JUN-2010:  CREATED
C     
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     COMPUTES DATA FOR REFINEMENT/DEREFINEMENT
C     FOR CLEAN CELLS ONLY
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     PARAMETERS
      DOUBLE PRECISION DELTOL,EPSTOL
      PARAMETER(DELTOL = 1.0D-5, EPSTOL = 1.0D-1)


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION YCURVF(NSPCMX,NCMPMX)
      DOUBLE PRECISION UCURVF(NCMPMX,NCMPMX)
      DOUBLE PRECISION DCURVF(NCMPMX),TCURVF(NCMPMX),PCURVF(NCMPMX)
      DOUBLE PRECISION FORNOW
      INTEGER JSPEC

C     DIAGNOSTICS
      INTEGER LPROC

C     DIAGNOSTICS
      LPROC = 20

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)WRITE(6,*)'In REFCEL'


C     BEGIN
C     =====

C     =========================================================================

C     ASSEMBLE DATA FOR REFINEMENT/DEREFINEMENT
C     -----------------------------------------
C     USING CELL-CENTRED SECOND DERIVATIVES

C     DENSITY
C      FORNOW = DSOR(ICELL)
      FORNOW = DRIN
      DCURVF(1) = ABS(PCOEFD(2))/FORNOW
      DCURVF(2) = ABS(PCOEFD(6))/FORNOW
      DCURVF(3) = ABS(PCOEFD(13))/FORNOW

C     VELOCITY
C      FORNOW = ABS(USOR(1,ICELL)) + 1.0D-2
      FORNOW = ONE
C      FORNOW = URIN
      UCURVF(1,1) = ABS(PCOEFU(2,1))/FORNOW
      UCURVF(1,2) = ABS(PCOEFU(6,1))/FORNOW
      UCURVF(1,3) = ABS(PCOEFU(13,1))/FORNOW

C      FORNOW = ABS(URUN(2,ICELL)) + 1.0D-2
      FORNOW = ONE
      UCURVF(2,1) = ABS(PCOEFU(2,2))/FORNOW
      UCURVF(2,2) = ABS(PCOEFU(6,2))/FORNOW
      UCURVF(2,3) = ABS(PCOEFU(13,2))/FORNOW

C      FORNOW = ABS(URUN(3,ICELL)) + 1.0D-2
      FORNOW = ONE
      UCURVF(3,1) = ABS(PCOEFU(2,3))/FORNOW
      UCURVF(3,2) = ABS(PCOEFU(6,3))/FORNOW
      UCURVF(3,3) = ABS(PCOEFU(13,3))/FORNOW

C     TEMPERATURE
C      FORNOW = ABS(TRUN(ICELL)-TRIN) + 1.0D-2
      FORNOW = TRIN
      TCURVF(1) = ABS(PCOEFT(2))/FORNOW
      TCURVF(2) = ABS(PCOEFT(6))/FORNOW
      TCURVF(3) = ABS(PCOEFT(13))/FORNOW

C     PRESSURE
C      FORNOW = ONE
      FORNOW = DRIN*URIN*URIN
      PCURVF(1) = ABS(PCOEFP(2))/FORNOW
      PCURVF(2) = ABS(PCOEFP(6))/FORNOW
      PCURVF(3) = ABS(PCOEFP(13))/FORNOW

C     SPECIES MASS FRACTIONS
      DO JSPEC = 1,NSPEC

C        FORNOW = ABS(YSOR(JSPEC,ICELL)) + 1.0D-10
        FORNOW = ONE
        YCURVF(JSPEC,1) = ABS(PCOEFY(2,JSPEC))/FORNOW
        YCURVF(JSPEC,2) = ABS(PCOEFY(6,JSPEC))/FORNOW
        YCURVF(JSPEC,3) = ABS(PCOEFY(13,JSPEC))/FORNOW

      ENDDO

CC     DIAGNOSTICS
C      IF(IPROC.EQ.LPROC)THEN
C        WRITE(6,'(I5,3(1PE12.4))')ICELL,DSOR(ICELL),
C     +                                  USOR(1,ICELL),USOR(2,ICELL)
C        WRITE(6,'(I5,6(1PE12.4))')ICELL,PCOEFD(2),PCOEFD(6),
C     +                                  PCOEFU(2,1),PCOEFU(6,1),
C     +                                  PCOEFU(2,2),PCOEFU(6,2)
C        WRITE(6,'(I5,6(1PE12.4))')ICELL,PCOEFD(2),PCOEFU(2,1),PCOEFT(2),
C     +                            PCOEFY(2,1),PCOEFY(2,2),D2PDX2
C        WRITE(6,'(I5,6(1PE12.4))')ICELL,DCURVF(1),UCURVF(1,1),TCURVF(1),
C     +                            YCURVF(1,1),PCURVF(1),PLOCAL
C     +                            YCURVF(1,1),YCURVF(2,1),PCURVF(1)
C      ENDIF

C     =========================================================================

C     DIRECTIONAL MAXIMA OF NORMALISED SECOND DERIVATIVES
C     ---------------------------------------------------
      ERDREF = TWO*(DCURVF(1)+DCURVF(2)+DCURVF(3))

      ERUREF(1) = TWO*(UCURVF(1,1)+UCURVF(1,2)+UCURVF(1,3))
      ERUREF(2) = TWO*(UCURVF(2,1)+UCURVF(2,2)+UCURVF(2,3))
      ERUREF(3) = TWO*(UCURVF(3,1)+UCURVF(3,2)+UCURVF(3,3))

      ERTREF = TWO*(TCURVF(1)+TCURVF(2)+TCURVF(3))
      ERPREF = TWO*(PCURVF(1)+PCURVF(2)+PCURVF(3))

      DO JSPEC = 1,NSPEC
        ERYREF(JSPEC)
     +    = TWO*(YCURVF(JSPEC,1)+YCURVF(JSPEC,2)+YCURVF(JSPEC,3))
      ENDDO

C     SCALING
      ERDREF = SQRT(ERDREF)
      ERUREF(1) = SQRT(ERUREF(1))
      ERUREF(2) = SQRT(ERUREF(2))
      ERUREF(3) = SQRT(ERUREF(3))
      ERTREF = SQRT(ERTREF)
      ERPREF = SQRT(ERPREF)
      DO JSPEC = 1,NSPEC
        ERYREF(JSPEC) = SQRT(ERYREF(JSPEC))
      ENDDO

C     EXCLUDE PRESSURE FOR NOW
      ERPREF = ZERO

C     EXCLUDE VELOCITY FOR NOW
      ERUREF(1) = ZERO
      ERUREF(2) = ZERO
      ERUREF(3) = ZERO

C     EXCLUDE DUMMY VARIABLE FOR NOW
      ERMREF = ZERO

C     DIAGNOSTICS
      IF(IPROC.EQ.LPROC)THEN
        WRITE(6,'(I5,5(1PE15.7))')ICELL,ERDREF,ERUREF(1),
     +                            ERTREF,ERPREF,
     +                            ERYREF(1),ERYREF(2)
      ENDIF

C     =========================================================================


      RETURN
      END
