      SUBROUTINE SORTI3(N1ARAY,N2ARAY,N3ARAY,NASIZE,N2SORT)
 
C     *************************************************************************
C
C     SORTI3
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     27-AUG-2014:  CREATED
C
C     DESCRIPTION
C     -----------
C     DNS CODE HAMISH
C     SORTS AN ARRAY OF INTEGERS
C     AND MAKES THE SAME ARRANGEMENT IN TWO OTHER ARRAYS OF INTEGERS
C     USING HEAPSORT
C
C     BASED ON NUMERICAL RECIPES pp229-232
C
C     *************************************************************************


C     GLOBAL DATA
C     ===========
C     -------------------------------------------------------------------------
      INCLUDE 'com_hamish.h'
C     -------------------------------------------------------------------------


C     ARGUMENTS
C     =========
      INTEGER N1ARAY(NASIZE),N2ARAY(NASIZE),N3ARAY(NASIZE)
      INTEGER NASIZE,N2SORT


C     LOCAL DATA
C     ==========
      INTEGER N1RTMP,N2RTMP,N3RTMP
      INTEGER MRTMP1,MRTMP2
      INTEGER IINDEX,JINDEX,JINDP1,KINDEX,LINDEX


C     BEGIN
C     =====

C     =========================================================================

C     PRELIMINARIES
C     -------------
      LINDEX = N2SORT/2 + 1

C     SORT ONLY IF ARRAY CONTAINS MORE THAN ONE ELEMENT
      IF(LINDEX.GT.1)THEN

C       =======================================================================

C       FORM THE HEAP
C       -------------
C       PICK A NEW ELEMENT FROM THE UPPER HALF OF THE HEAP
C                     I.E. FROM THE LOWER HALF OF THE ARRAY
1000    CONTINUE

          LINDEX = LINDEX - 1
          N1RTMP = N1ARAY(LINDEX)
          N2RTMP = N2ARAY(LINDEX)
          N3RTMP = N3ARAY(LINDEX)

          IINDEX = LINDEX
          JINDEX = LINDEX + LINDEX

C         LOOP TO SIFT THE NEW ELEMENT DOWN THE HEAP
1100      CONTINUE

            MRTMP1 = N1ARAY(JINDEX)

C           FIND THE GREATER OF EACH PAIR OF EXISTING LOWER ELEMENTS
            IF(JINDEX.LT.N2SORT)THEN
              JINDP1 = JINDEX + 1
              MRTMP2 = N1ARAY(JINDP1)
              IF(MRTMP1.LT.MRTMP2)THEN
                JINDEX = JINDP1
                MRTMP1 = MRTMP2
              ENDIF
            ENDIF

C           COMPARE THE NEW ELEMENT WITH THE EXISTING ELEMENT
            IF(N1RTMP.LT.MRTMP1)THEN
              N1ARAY(IINDEX) = N1ARAY(JINDEX)
              N2ARAY(IINDEX) = N2ARAY(JINDEX)
              N3ARAY(IINDEX) = N3ARAY(JINDEX)
              IINDEX = JINDEX
              JINDEX = JINDEX + JINDEX
              IF(JINDEX.LE.N2SORT)GOTO 1100
            ENDIF

C         END OF LOOP 1100

C         PUT THE NEW ELEMENT INTO ITS SLOT
          N1ARAY(IINDEX) = N1RTMP
          N2ARAY(IINDEX) = N2RTMP
          N3ARAY(IINDEX) = N3RTMP

          IF(LINDEX.GT.1)GOTO 1000

C       END OF LOOP 1000

C       =======================================================================

C       SORT THE HEAP
C       -------------
        KINDEX = N2SORT

2000    CONTINUE

C         PICK THE HIGHEST UNSORTED ELEMENT OF THE ARRAY
          N1RTMP = N1ARAY(KINDEX)
          N2RTMP = N2ARAY(KINDEX)
          N3RTMP = N3ARAY(KINDEX)

C         MOVE THE CURRENT TOP-OF-THE-HEAP INTO THAT SLOT
          N1ARAY(KINDEX) = N1ARAY(1)
          N2ARAY(KINDEX) = N2ARAY(1)
          N3ARAY(KINDEX) = N3ARAY(1)

C         DECREMENT INDEX OF HIGHEST UNSORTED ELEMENT
          KINDEX = KINDEX - 1

C         CHECK FOR LOWEST ELEMENT
          IF(KINDEX.GT.1)THEN

C           NOT THE LOWEST ELEMENT
            IINDEX = 1
            JINDEX = 2

C           LOOP TO SIFT THE ELEMENT DOWN THE HEAP
2100        CONTINUE

              MRTMP1 = N1ARAY(JINDEX)

C             FIND THE GREATER OF EACH PAIR OF EXISTING LOWER ELEMENTS
              IF(JINDEX.LT.KINDEX)THEN
                JINDP1 = JINDEX + 1
                MRTMP2 = N1ARAY(JINDP1)
                IF(MRTMP1.LT.MRTMP2)THEN
                  JINDEX = JINDP1
                  MRTMP1 = MRTMP2
                ENDIF
              ENDIF

C             COMPARE THE NEW ELEMENT WITH THE EXISTING ELEMENT
              IF(N1RTMP.LT.MRTMP1)THEN
                N1ARAY(IINDEX) = N1ARAY(JINDEX)
                N2ARAY(IINDEX) = N2ARAY(JINDEX)
                N3ARAY(IINDEX) = N3ARAY(JINDEX)
                IINDEX = JINDEX
                JINDEX = JINDEX + JINDEX
                IF(JINDEX.LE.KINDEX)GOTO 2100
              ENDIF

C           END OF LOOP 2100

C           PUT THE NEW ELEMENT INTO ITS SLOT
            N1ARAY(IINDEX) = N1RTMP
            N2ARAY(IINDEX) = N2RTMP
            N3ARAY(IINDEX) = N3RTMP
            GOTO 2000

          ELSE

C           LOWEST ELEMENT
            N1ARAY(1) = N1RTMP
            N2ARAY(1) = N2RTMP
            N3ARAY(1) = N3RTMP

          ENDIF

C       END OF LOOP 2000

C       =======================================================================

      ENDIF
C     END OF SORT ONLY IF ARRAY CONTAINS MORE THAN ONE ELEMENT

C     =========================================================================


      RETURN
      END
